/******************************************************************************************************

 *  Rohde&Schwarz SMxV Signal Generator instrument driver
 *  Callbacks
 *
 *  Original Release: April 2020
 *  By: Petr Kominek
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *

*****************************************************************************************************/

#include "rssmxv.h"
#include "rssmxv_utility.h"
#include <stdlib.h>

#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
#define ANSI
#endif
#ifndef ANSI                /* UNIX compatible */
    #include <sys/timeb.h>
    #include <sys/types.h>
    #include <unistd.h>
    #define Sleep sleep
#else                       /* ANSI compatible */
#include <windows.h>
#endif

#pragma warning (disable:4100) // unreferenced formal parameter

/******************************************************************************************************
 * Function: rssmxv_logging_WriteCallback
 * Purpose:  This function overrides standard callback function.
*****************************************************************************************************/
ViStatus rssmxv_logging_WriteCallback(ViSession          instrSession,
                                               ViConstString      repCapName,
                                               RsCoreAttributePtr attr,
                                               void*              value)
{
    return VI_SUCCESS;
}

/******************************************************************************************************
 * Function: rssmxv_logging_ReadCallback
 * Purpose:  This function overrides standard callback function. Returns value
 *           from cache
*****************************************************************************************************/
ViStatus rssmxv_logging_ReadCallback(ViSession          instrSession,
                                              ViConstString      repCapName,
                                              RsCoreAttributePtr attr)
{
    return VI_SUCCESS;
}

/******************************************************************************************************
 * Function: rssmxv_hexString_ReadCallback
 * Purpose:  Reads string pattern starting with #B or #H and returns the payload only. Also cuts out the length at the end
*****************************************************************************************************/
ViStatus rssmxv_hexString_ReadCallback(ViSession          instrSession,
                                                ViConstString      repCapName,
                                                RsCoreAttributePtr attr)
{
    return RsCore_SpecialCallback_BinOrHexPatternRead(instrSession, repCapName, attr);
}

/******************************************************************************************************
 * Function: rssmxv_hexString_WriteCallback
 * Purpose:  This function overrides standard callback function.
*****************************************************************************************************/
ViStatus rssmxv_hexString_WriteCallback(ViSession          instrSession,
                                                 ViConstString      repCapName,
                                                 RsCoreAttributePtr attr,
                                                 void*              value)
{
    return RsCore_SpecialCallback_HexPatternWrite(instrSession, repCapName, attr, value);
}

/******************************************************************************************************
 * Function: rssmxv_ReadOnlyCollection_ReadCallback
 * Purpose:  This function overrides standard callback function.
*****************************************************************************************************/
ViStatus rssmxv_ReadOnlyCollection_ReadCallback(ViSession          instrSession,
                                                         ViConstString      repCapName,
                                                         RsCoreAttributePtr attr)
{
    return RsCore_ReadCallback(instrSession, repCapName, attr);
}