/*****************************************************************************
 *  Rohde&Schwarz SMxV Signal Generator instrument driver
 *
 *  VXIpnp Instrument Driver
 *****************************************************************************/

#ifndef __RSSMXV_HEADER
#define __RSSMXV_HEADER

#include "rscore.h"
#include "rssmxv_attributes.h"
#include "rssmxv_utility.h"
#include "rssmxv_internal.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/*****************************************************************************
 *- Useful Macros & Constants -----------------------------------------------*
 *****************************************************************************/

#define RSSMXV_SIMULATION_ID_QUERY      "Rohde&Schwarz,SMCV100B,123456,5.20"  /* Default definition of ID Query Response for simulation */
#define RSSMXV_VALID_ID_RESPONSE_STRING "Rohde&Schwarz"      /* Valid response for identification query */
#define RSSMXV_SIMULATION_OPT_QUERY     ""      /* Simulated response for *OPT? command */

#define RSSMXV_OPC_TIMEOUT              30000L   /* Maximum time to wait for OPC in milliseconds */

/****************************************************************************
 *----------------- Instrument Driver Revision Information -----------------*
 ****************************************************************************/

/* Class Driver Identification */

#define RSSMXV_CLASS_DRIVER_DESCRIPTION           ""
#define RSSMXV_CLASS_DRIVER_PREFIX                ""
#define RSSMXV_CLASS_DRIVER_VENDOR                ""
#define RSSMXV_CLASS_DRIVER_REVISION              ""
#define RSSMXV_CLASS_SPEC_MAJOR_VERSION           1L    /* Class specification major version */
#define RSSMXV_CLASS_SPEC_MINOR_VERSION           6L    /* Class specification minor version */

/* Driver Identification */

#define RSSMXV_SPECIFIC_DRIVER_DESCRIPTION        ""
#define RSSMXV_SPECIFIC_DRIVER_PREFIX             "RSSMXV"
#define RSSMXV_SPECIFIC_DRIVER_LOCATOR            ""
#define RSSMXV_SPECIFIC_DRIVER_VENDOR             ""
#define RSSMXV_MAJOR_VERSION                      5L    /* Instrument driver major version          */
#define RSSMXV_MINOR_VERSION                      20L    /* Instrument driver minor version          */
#define RSSMXV_MINOR_MINOR_VERSION                0L   /* Instrument driver minor minor version    */

/* Driver Capabilities */

#define RSSMXV_SUPPORTED_INSTRUMENT_MODELS        "SMCV100B"    /* Instrument driver supported model(s)     */
#define RSSMXV_GROUP_CAPABILITIES                 ""
#define RSSMXV_FUNCTION_CAPABILITIES              ""

/****************************************************************************
 *---------------------------- Attribute Defines ---------------------------*
 ****************************************************************************/

    /*- Inherent Instrument Specific Attributes ----------------------------*/

        /* User Options */

#define RSCMW_ATTR_RANGE_CHECK                                           RS_ATTR_RANGE_CHECK                                     /* ViBoolean */
#define RSCMW_ATTR_QUERY_INSTRUMENT_STATUS                               RS_ATTR_QUERY_INSTRUMENT_STATUS                         /* ViBoolean */
#define RSCMW_ATTR_CACHE                                                 RS_ATTR_CACHE                                           /* ViBoolean */
#define RSCMW_ATTR_SIMULATE                                              RS_ATTR_SIMULATE                                        /* ViBoolean */
#define RSCMW_ATTR_RECORD_COERCIONS                                      RS_ATTR_RECORD_COERCIONS                                /* ViBoolean */
#define RSCMW_ATTR_INTERCHANGE_CHECK                                     RS_ATTR_INTERCHANGE_CHECK                               /* ViBoolean */
#define RSCMW_ATTR_SPY                                                   RS_ATTR_SPY                                             /* ViBoolean */
#define RSCMW_ATTR_USE_SPECIFIC_SIMULATION                               RS_ATTR_USE_SPECIFIC_SIMULATION                         /* ViBoolean */

        /* Class Driver Identification */

#define RSCMW_ATTR_CLASS_DRIVER_DESCRIPTION                              RS_ATTR_CLASS_DRIVER_DESCRIPTION                        /* ViString, read-only */
#define RSCMW_ATTR_CLASS_DRIVER_PREFIX                                   RS_ATTR_CLASS_DRIVER_PREFIX                             /* ViString, read-only */
#define RSCMW_ATTR_CLASS_DRIVER_VENDOR                                   RS_ATTR_CLASS_DRIVER_VENDOR                             /* ViString, read-only */
#define RSCMW_ATTR_CLASS_DRIVER_REVISION                                 RS_ATTR_CLASS_DRIVER_REVISION                           /* ViString, read-only */
#define RSCMW_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION           /* ViInt32,  read-only */
#define RSCMW_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION           /* ViInt32,  read-only */

        /* Driver Identification */

#define RSCMW_ATTR_SPECIFIC_DRIVER_DESCRIPTION                           RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION                     /* ViString, read-only */
#define RSCMW_ATTR_SPECIFIC_DRIVER_PREFIX                                RS_ATTR_SPECIFIC_DRIVER_PREFIX                          /* ViString, read-only */
#define RSCMW_ATTR_SPECIFIC_DRIVER_LOCATOR                               RS_ATTR_SPECIFIC_DRIVER_LOCATOR                         /* ViString, read-only */
#define RSCMW_ATTR_SPECIFIC_DRIVER_VENDOR                                RS_ATTR_SPECIFIC_DRIVER_VENDOR                          /* ViString, read-only */
#define RSCMW_ATTR_SPECIFIC_DRIVER_REVISION                              RS_ATTR_SPECIFIC_DRIVER_REVISION                        /* ViString, read-only */
#define RSCMW_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION        /* ViInt32,  read-only */
#define RSCMW_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION        /* ViInt32,  read-only */

        /* Driver Capabilities */

#define RSCMW_ATTR_SUPPORTED_INSTRUMENT_MODELS                           RS_ATTR_SUPPORTED_INSTRUMENT_MODELS                     /* ViString, read-only */
#define RSCMW_ATTR_GROUP_CAPABILITIES                                    RS_ATTR_GROUP_CAPABILITIES                              /* ViString, read-only */
#define RSCMW_ATTR_FUNCTION_CAPABILITIES                                 RS_ATTR_FUNCTION_CAPABILITIES                           /* ViString, read-only */
#define RSCMW_ATTR_CHANNEL_COUNT                                         RS_ATTR_CHANNEL_COUNT                                   /* ViInt32,  read-only */

        /* Driver Setup */

#define RSCMW_ATTR_DRIVER_SETUP                                          RS_ATTR_DRIVER_SETUP                                    /* ViString            */

        /* Instrument Identification */

#define RSCMW_ATTR_INSTRUMENT_MANUFACTURER                               RS_ATTR_INSTRUMENT_MANUFACTURER                         /* ViString, read-only */
#define RSCMW_ATTR_INSTRUMENT_MODEL                                      RS_ATTR_INSTRUMENT_MODEL                                /* ViString, read-only */
#define RSCMW_ATTR_INSTRUMENT_FIRMWARE_REVISION                          RS_ATTR_INSTRUMENT_FIRMWARE_REVISION                    /* ViString, read-only */

        /* Advanced Session Information */

#define RSCMW_ATTR_IO_RESOURCE_DESCRIPTOR                                RS_ATTR_IO_RESOURCE_DESCRIPTOR                          /* ViString, read-only */
#define RSCMW_ATTR_LOGICAL_NAME                                          RS_ATTR_LOGICAL_NAME                                    /* ViString, read-only */

    /*- Instrument-Specific Attributes -------------------------------------*/

/* BEGIN GENERATE */
#define RSSMXV_ATTR_RESET_SIGNAL_PATH                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1
#define RSSMXV_ATTR_BB_FREQUENCY_OFFSET                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2
#define RSSMXV_ATTR_BB_SIGNAL_ROUTING                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3
#define RSSMXV_ATTR_BB_PHASE_OFFSET                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4
#define RSSMXV_ATTR_AM_AUDIO_ENABLED                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5
#define RSSMXV_ATTR_AM_AUDIO_SET_TO_DEFAULT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6
#define RSSMXV_ATTR_AM_AUDIO_RECALL_SETTINGS                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7
#define RSSMXV_ATTR_AM_AUDIO_SAVE_SETTINGS                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 9
#define RSSMXV_ATTR_AM_AUDIO_SETTINGS_CATALOG                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 11
#define RSSMXV_ATTR_AM_AUDIO_DELETE_SETTINGS                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 13
#define RSSMXV_ATTR_AM_AUDIO_INPUT_SIGNAL                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 15
#define RSSMXV_ATTR_AM_AUDIO_EXTERNAL_SOURCE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 16
#define RSSMXV_ATTR_AM_AUDIO_CHANNEL_ENABLED                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 17
#define RSSMXV_ATTR_AM_AUDIO_GENERATOR_LEVEL                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 18
#define RSSMXV_ATTR_AM_AUDIO_GENERATOR_FREQUENCY                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 19
#define RSSMXV_ATTR_AM_AUDIO_PLAYER_SOURCE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 20
#define RSSMXV_ATTR_AM_AUDIO_PLAYER_CATALOG                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 21
#define RSSMXV_ATTR_AM_AUDIO_PLAYER_ATTENUATION                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 23
#define RSSMXV_ATTR_AM_AUDIO_MODULATION_NOMINAL_DEPTH                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 24
#define RSSMXV_ATTR_AM_AUDIO_MODULATION_DEPTH                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 25
#define RSSMXV_ATTR_FM_AUDIO_ENABLED                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 26
#define RSSMXV_ATTR_FM_AUDIO_SET_TO_DEFAULT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 27
#define RSSMXV_ATTR_FM_AUDIO_RECALL_SETTINGS                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 28
#define RSSMXV_ATTR_FM_AUDIO_SAVE_SETTINGS                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 30
#define RSSMXV_ATTR_FM_AUDIO_SETTINGS_CATALOG                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 32
#define RSSMXV_ATTR_FM_AUDIO_DELETE_SETTINGS                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 34
#define RSSMXV_ATTR_FM_AUDIO_INPUT_SIGNAL                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 36
#define RSSMXV_ATTR_FM_AUDIO_EXTERNAL_SOURCE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 37
#define RSSMXV_ATTR_FM_AUDIO_STEREO_MODE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 38
#define RSSMXV_ATTR_FM_AUDIO_STEREO_CHANNEL_ENABLED                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 39
#define RSSMXV_ATTR_FM_AUDIO_GENERATOR_LEVEL                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 40
#define RSSMXV_ATTR_FM_AUDIO_GENERATOR_CHANNEL_FREQUENCY                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 41
#define RSSMXV_ATTR_FM_AUDIO_PLAYER_SOURCE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 42
#define RSSMXV_ATTR_FM_AUDIO_PLAYER_CATALOG                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 43
#define RSSMXV_ATTR_FM_AUDIO_PLAYER_CHANNEL_ATTENUATION                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 45
#define RSSMXV_ATTR_FM_AUDIO_MODULATION_MODE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 46
#define RSSMXV_ATTR_FM_AUDIO_MODULATION_PREEMPHASIS                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 47
#define RSSMXV_ATTR_FM_AUDIO_MODULATION_NOMINAL_DEVIATION                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 48
#define RSSMXV_ATTR_FM_AUDIO_MODULATION_ACTUAL_DEVIATION                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 49
#define RSSMXV_ATTR_FM_AUDIO_MODULATION_PILOT_DEVIATION                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 50
#define RSSMXV_ATTR_FM_AUDIO_RDS_ENABLED                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 51
#define RSSMXV_ATTR_FM_AUDIO_RDS_DEVIATION                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 52
#define RSSMXV_ATTR_FM_AUDIO_RDS_GROUP_SEQUENCE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 53
#define RSSMXV_ATTR_FM_AUDIO_RDS_PROGRAM_ID                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 54
#define RSSMXV_ATTR_FM_AUDIO_RDS_PROGRAM_SERVICE_NAME                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 55
#define RSSMXV_ATTR_FM_AUDIO_RDS_TRAFFIC_PROGRAM_FLAG                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 56
#define RSSMXV_ATTR_FM_AUDIO_RDS_TRAFFIC_ANNOUNCEMENT_FLAG                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 57
#define RSSMXV_ATTR_FM_AUDIO_RDS_PROGRAM_TYPE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 58
#define RSSMXV_ATTR_FM_AUDIO_RDS_PROGRAM_TYPE_NAME                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 59
#define RSSMXV_ATTR_FM_AUDIO_RDS_MUSIC_SPEECH_FLAG                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 60
#define RSSMXV_ATTR_FM_AUDIO_RDS_RADIO_TEXT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 61
#define RSSMXV_ATTR_FM_AUDIO_RDS_DI_DYNAMIC_PTY_ENABLED                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 62
#define RSSMXV_ATTR_FM_AUDIO_RDS_DI_COMPRESSED                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 63
#define RSSMXV_ATTR_FM_AUDIO_RDS_DI_ARTIFICIAL_HEAD_ENABLED                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 64
#define RSSMXV_ATTR_FM_AUDIO_RDS_DI_STEREO_DECODER                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 65
#define RSSMXV_ATTR_FM_AUDIO_RDS_CLOCK_ENABLED                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 66
#define RSSMXV_ATTR_FM_AUDIO_RDS_CLOCK_OFFSET                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 67
#define RSSMXV_ATTR_FM_AUDIO_RDS_ALTERNATIVE_FREQUENCY_METHOD               RS_SPECIFIC_PUBLIC_ATTR_BASE + 68
#define RSSMXV_ATTR_FM_AUDIO_RDS_AF_METHOD_A_FREQUENCY_COUNT                RS_SPECIFIC_PUBLIC_ATTR_BASE + 69
#define RSSMXV_ATTR_FM_AUDIO_RDS_AF_METHOD_A_FREQUENCY                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 70
#define RSSMXV_ATTR_FM_AUDIO_RDS_AF_METHOD_B_LIST_FREQUENCY_COUNT           RS_SPECIFIC_PUBLIC_ATTR_BASE + 71
#define RSSMXV_ATTR_FM_AUDIO_RDS_AF_METHOD_B_LIST_TUNING_FREQUENCY          RS_SPECIFIC_PUBLIC_ATTR_BASE + 72
#define RSSMXV_ATTR_FM_AUDIO_RDS_AF_METHOD_B_LIST_ENTRY_FREQUENCY           RS_SPECIFIC_PUBLIC_ATTR_BASE + 73
#define RSSMXV_ATTR_FM_AUDIO_RDS_AF_METHOD_B_LIST_ENTRY_ORDER               RS_SPECIFIC_PUBLIC_ATTR_BASE + 74
#define RSSMXV_ATTR_FM_AUDIO_RDS_EON_PROGRAM_ID                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 75
#define RSSMXV_ATTR_FM_AUDIO_RDS_EON_PROGRAM_SERVICE_NAME                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 76
#define RSSMXV_ATTR_FM_AUDIO_RDS_EON_TRAFFIC_PROGRAM_FLAG                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 77
#define RSSMXV_ATTR_FM_AUDIO_RDS_EON_TRAFFIC_ANNOUNCEMENT_FLAG              RS_SPECIFIC_PUBLIC_ATTR_BASE + 78
#define RSSMXV_ATTR_FM_AUDIO_RDS_EON_LINKAGE_ACTUATOR_ENABLED               RS_SPECIFIC_PUBLIC_ATTR_BASE + 79
#define RSSMXV_ATTR_FM_AUDIO_RDS_EON_EXTENDED_GENERIC_ENABLED               RS_SPECIFIC_PUBLIC_ATTR_BASE + 80
#define RSSMXV_ATTR_FM_AUDIO_RDS_EON_INTERNATIONAL_LINKAGE_SET_ENABLED      RS_SPECIFIC_PUBLIC_ATTR_BASE + 81
#define RSSMXV_ATTR_FM_AUDIO_RDS_EON_LINKAGE_SET_NUMBER                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 82
#define RSSMXV_ATTR_FM_AUDIO_RDS_EON_PROGRAM_TYPE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 83
#define RSSMXV_ATTR_FM_AUDIO_RDS_EON_PROGRAM_ITEM_NUMBER                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 84
#define RSSMXV_ATTR_FM_AUDIO_RDS_EON_ALTERNATIVE_FREQUENCY_METHOD           RS_SPECIFIC_PUBLIC_ATTR_BASE + 85
#define RSSMXV_ATTR_FM_AUDIO_RDS_EON_AF_METHOD_A_FREQUENCY_COUNT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 86
#define RSSMXV_ATTR_FM_AUDIO_RDS_EON_AF_METHOD_A_FEQUENCY                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 87
#define RSSMXV_ATTR_FM_AUDIO_RDS_EON_AF_MAPPED_FREQUENCY_COUNT              RS_SPECIFIC_PUBLIC_ATTR_BASE + 88
#define RSSMXV_ATTR_FM_AUDIO_RDS_EON_AF_MAPPED_TUNING_FREQUENCY             RS_SPECIFIC_PUBLIC_ATTR_BASE + 89
#define RSSMXV_ATTR_FM_AUDIO_RDS_EON_AF_MAPPED_FREQUENCY                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 90
#define RSSMXV_ATTR_FM_AUDIO_RDS_TMC_ENABLED                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 91
#define RSSMXV_ATTR_TRANSMIT_FM_AUDIO_RDS_TMC                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 92
#define RSSMXV_ATTR_QUERY_FM_AUDIO_RDS_TMC_READY                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 93
#define RSSMXV_ATTR_FM_AUDIO_RDS_TMC_GROUP_3A_BLOCK_3                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 94
#define RSSMXV_ATTR_FM_AUDIO_RDS_TMC_GROUP_8A_COUNT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 95
#define RSSMXV_ATTR_FM_AUDIO_RDS_TMC_GROUP_8A_BLOCK_2                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 96
#define RSSMXV_ATTR_FM_AUDIO_RDS_TMC_GROUP_8A_BLOCK_3                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 97
#define RSSMXV_ATTR_FM_AUDIO_RDS_TMC_GROUP_8A_BLOCK_4                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 98
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_ENABLED                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 99
#define RSSMXV_ATTR_TRANSMIT_FM_AUDIO_RDS_OPF                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 100
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_1A_BLOCK_2                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 101
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_1A_BLOCK_3                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 102
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_1A_BLOCK_4                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 103
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_1B_BLOCK_2                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 104
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_1B_BLOCK_3                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 105
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_1B_BLOCK_4                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 106
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_3A_BLOCK_2                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 107
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_3A_BLOCK_3                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 108
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_3A_BLOCK_4                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 109
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_3B_BLOCK_2                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 110
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_3B_BLOCK_3                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 111
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_3B_BLOCK_4                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 112
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_4B_BLOCK_2                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 113
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_4B_BLOCK_3                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 114
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_4B_BLOCK_4                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 115
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_5A_BLOCK_2                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 116
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_5A_BLOCK_3                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 117
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_5A_BLOCK_4                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 118
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_5B_BLOCK_2                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 119
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_5B_BLOCK_3                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 120
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_5B_BLOCK_4                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 121
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_6A_BLOCK_2                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 122
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_6A_BLOCK_3                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 123
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_6A_BLOCK_4                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 124
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_6B_BLOCK_2                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 125
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_6B_BLOCK_3                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 126
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_6B_BLOCK_4                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 127
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_7A_BLOCK_2                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 128
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_7A_BLOCK_3                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 129
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_7A_BLOCK_4                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 130
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_7B_BLOCK_2                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 131
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_7B_BLOCK_3                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 132
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_7B_BLOCK_4                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 133
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_8A_BLOCK_2                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 134
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_8A_BLOCK_3                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 135
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_8A_BLOCK_4                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 136
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_8B_BLOCK_2                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 137
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_8B_BLOCK_3                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 138
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_8B_BLOCK_4                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 139
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_9A_BLOCK_2                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 140
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_9A_BLOCK_3                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 141
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_9A_BLOCK_4                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 142
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_9B_BLOCK_2                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 143
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_9B_BLOCK_3                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 144
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_9B_BLOCK_4                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 145
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_10B_BLOCK_2                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 146
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_10B_BLOCK_3                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 147
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_10B_BLOCK_4                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 148
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_11A_BLOCK_2                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 149
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_11A_BLOCK_3                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 150
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_11A_BLOCK_4                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 151
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_11B_BLOCK_2                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 152
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_11B_BLOCK_3                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 153
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_11B_BLOCK_4                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 154
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_12A_BLOCK_2                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 155
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_12A_BLOCK_3                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 156
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_12A_BLOCK_4                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 157
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_12B_BLOCK_2                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 158
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_12B_BLOCK_3                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 159
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_12B_BLOCK_4                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 160
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_13A_BLOCK_2                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 161
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_13A_BLOCK_3                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 162
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_13A_BLOCK_4                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 163
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_13B_BLOCK_2                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 164
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_13B_BLOCK_3                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 165
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_13B_BLOCK_4                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 166
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_15A_BLOCK_2                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 167
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_15A_BLOCK_3                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 168
#define RSSMXV_ATTR_FM_AUDIO_RDS_OPF_GROUP_15A_BLOCK_4                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 169
#define RSSMXV_ATTR_FM_AUDIO_DARC_ENABLED                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 170
#define RSSMXV_ATTR_FM_AUDIO_DARC_DEVIATION                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 171
#define RSSMXV_ATTR_FM_AUDIO_DARC_BIC_DATA                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 172
#define RSSMXV_ATTR_FM_AUDIO_DARC_INFORMATION                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 173
#define RSSMXV_ATTR_FM_AUDIO_SPECIAL_SETTINGS_ENABLED                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 174
#define RSSMXV_ATTR_FM_AUDIO_SPECIAL_PILOT_ENABLED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 175
#define RSSMXV_ATTR_FM_AUDIO_SPECIAL_PILOT_PHASE_OFFSET                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 176
#define RSSMXV_ATTR_FM_AUDIO_SPECIAL_RDS_PHASE_OFFSET                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 177
#define RSSMXV_ATTR_TDMB_DAB_ENABLED                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 178
#define RSSMXV_ATTR_TDMB_DAB_SET_TO_DEFAULT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 179
#define RSSMXV_ATTR_TDMB_DAB_RECALL_SETTINGS                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 180
#define RSSMXV_ATTR_TDMB_DAB_SAVE_SETTINGS                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 182
#define RSSMXV_ATTR_TDMB_DAB_SETTINGS_CATALOG                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 184
#define RSSMXV_ATTR_TDMB_DAB_DELETE_SETTINGS                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 186
#define RSSMXV_ATTR_TDMB_DAB_SOURCE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 188
#define RSSMXV_ATTR_TDMB_DAB_EXTERNAL_INPUT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 189
#define RSSMXV_ATTR_TDMB_DAB_EXTERNAL_INPUT_FORMAT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 190
#define RSSMXV_ATTR_TDMB_DAB_IP_ETI_CHANNEL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 191
#define RSSMXV_ATTR_TDMB_DAB_ETI_SIGNAL_TYPE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 192
#define RSSMXV_ATTR_TDMB_DAB_ETI_NUMBER_OF_STREAMS                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 193
#define RSSMXV_ATTR_TDMB_DAB_MODE_IDENTITY                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 194
#define RSSMXV_ATTR_TDMB_DAT_ETI_INFO_SUBCHANNEL_ID                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 195
#define RSSMXV_ATTR_TDMB_DAT_ETI_INFO_PROTECTION_PROFILE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 196
#define RSSMXV_ATTR_TDMB_DAT_ETI_INFO_PROTECTION_LEVEL                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 197
#define RSSMXV_ATTR_TDMB_DAT_ETI_INFO_DATARATE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 198
#define RSSMXV_ATTR_TDMB_DAB_NETWORK_MODE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 199
#define RSSMXV_ATTR_TDMB_DAB_NETWORK_TII_STATE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 200
#define RSSMXV_ATTR_TDMB_DAB_NETWORK_TII_MAIN_ID                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 201
#define RSSMXV_ATTR_TDMB_DAB_NETWORK_TII_SUB_ID                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 202
#define RSSMXV_ATTR_TDMB_DAB_DELAY_NETWORK                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 203
#define RSSMXV_ATTR_TDMB_DAB_DELAY_STATIC                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 204
#define RSSMXV_ATTR_TDMB_DAB_DELAY_TX_PROCESSING                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 205
#define RSSMXV_ATTR_TDMB_DAB_DELAY_TX_COMPENSATION                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 206
#define RSSMXV_ATTR_TDMB_DAB_DELAY_TX_DELAY                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 207
#define RSSMXV_ATTR_TDMB_DAB_DELAY_TOTAL                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 208
#define RSSMXV_ATTR_TDMB_DAB_DELAY_MAX_DEVIATION                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 209
#define RSSMXV_ATTR_TDMB_DAB_SPECIAL_SETTINGS_ENABLED                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 210
#define RSSMXV_ATTR_TDMB_DAB_SPECIAL_TRANSMISSION_MODE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 211
#define RSSMXV_ATTR_TDMB_DAB_SPECIAL_TRANSMISSION_MODE_SELECT               RS_SPECIFIC_PUBLIC_ATTR_BASE + 212
#define RSSMXV_ATTR_T_DMB_DAB_SPECIAL_PRBS_SEQUENCE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1553
#define RSSMXV_ATTR_T_DMB_DAB_SPECIAL_TEST_SIGNAL_ENABLED                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1554
#define RSSMXV_ATTR_T_DMB_DAB_SPECIAL_TEST_SIGNAL_SUBCHANNEL_ID             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1555
#define RSSMXV_ATTR_DRM_ENABLED                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 213
#define RSSMXV_ATTR_DRM_SET_TO_DEFAULT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 214
#define RSSMXV_ATTR_DRM_RECALL_SETTINGS                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 215
#define RSSMXV_ATTR_DRM_SAVE_SETTINGS                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 217
#define RSSMXV_ATTR_DRM_DELETE_SETTINGS                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 219
#define RSSMXV_ATTR_DRM_SETTINGS_CATALOG                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 221
#define RSSMXV_ATTR_DRM_INPUT_SOURCE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 223
#define RSSMXV_ATTR_DRM_INTPUT_SELECT_FILE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 224
#define RSSMXV_ATTR_DRM_INPUT_UDP_PORT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 226
#define RSSMXV_ATTR_DRM_INPUT_AUDIO_SERVICES_NUMBER                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 227
#define RSSMXV_ATTR_DRM_INPUT_DATA_SERVICES_NUMBER                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 228
#define RSSMXV_ATTR_DRM_INPUT_LAYER_TYPE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 229
#define RSSMXV_ATTR_DRM_INPUT_LABEL                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 230
#define RSSMXV_ATTR_DRM_CODING_ROBUSTNESS_MODE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 231
#define RSSMXV_ATTR_DRM_CODING_CONSTELLATION_MSC                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 232
#define RSSMXV_ATTR_DRM_CODING_CONSTELLATION_SDC                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 233
#define RSSMXV_ATTR_DRM_CODING_INTERLEAVER_DEPTH                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 234
#define RSSMXV_ATTR_DRM_CODING_CHANNEL_BANDWIDTH                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 235
#define RSSMXV_ATTR_DRM_CODE_RATE_SDC_PROTECTION_PROFILE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 236
#define RSSMXV_ATTR_DRM_CODE_RATE_SDC_PROTECTION_LEVEL                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 237
#define RSSMXV_ATTR_DRM_SDC_CODE_RATE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 238
#define RSSMXV_ATTR_DRM_CODE_RATE_MSC_PROTECTION_PROFILE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 239
#define RSSMXV_ATTR_DRM_CODE_RATE_MSC_PROTECTION_LEVEL                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 240
#define RSSMXV_ATTR_DRM_MSC_CODE_RATE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 241
#define RSSMXV_ATTR_ATSCMH_ENABLED                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 242
#define RSSMXV_ATTR_ATSCMH_VSB_FREQUENCY                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 243
#define RSSMXV_ATTR_ATSCMH_SET_TO_DEFAULT                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 244
#define RSSMXV_ATTR_ATSCMH_RECALL_SETTINGS                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 245
#define RSSMXV_ATTR_ATSCMH_SAVE_SETTINGS                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 247
#define RSSMXV_ATTR_ATSCMH_SETTINGS_CATALOG                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 249
#define RSSMXV_ATTR_ATSCMH_DELETE_SETTINGS                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 251
#define RSSMXV_ATTR_ATSCMH_SOURCE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 253
#define RSSMXV_ATTR_ATSCMH_EXTERNAL_INPUT                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 254
#define RSSMXV_ATTR_ATSCMH_SERIAL_INPUT_FORMAT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 255
#define RSSMXV_ATTR_ATSCMH_IP_TS_CHANNEL                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 256
#define RSSMXV_ATTR_ATSCMH_STUFFING_ENABLED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 257
#define RSSMXV_ATTR_ATSCMH_INFO_PACKET_LENGTH                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 258
#define RSSMXV_ATTR_ATSCMH_INFO_MAX_USEFUL_DATARATE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 259
#define RSSMXV_ATTR_ATSCMH_INFO_MEASURED_DATARATE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 260
#define RSSMXV_ATTR_ATSCMH_USEFUL_DATARATE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 261
#define RSSMXV_ATTR_ATSCMH_TEST_SIGNAL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 262
#define RSSMXV_ATTR_ATSCMH_TEST_TS_PACKET                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 263
#define RSSMXV_ATTR_ATSCMH_TEST_PID_TEST_PACKET                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 264
#define RSSMXV_ATTR_ATSCMH_TEST_PID                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 265
#define RSSMXV_ATTR_ATSCMH_TEST_PAYLOAD                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 266
#define RSSMXV_ATTR_ATSCMH_TEST_PRBS                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 267
#define RSSMXV_ATTR_ATSCMH_CODING_SYMBOL_RATE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 268
#define RSSMXV_ATTR_ATSCMH_CODING_CONSTELLATION                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 269
#define RSSMXV_ATTR_ATSCMH_CODING_ROLL_OFF                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 270
#define RSSMXV_ATTR_ATSCMH_CODING_MH_STATE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 271
#define RSSMXV_ATTR_ATSCMH_NETWORKING_MHE_PID                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 272
#define RSSMXV_ATTR_ATSCMH_NETWORKING_RF_WATERMARK                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 273
#define RSSMXV_ATTR_ATSCMH_NETWORKING_TX_ADDRESS                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 274
#define RSSMXV_ATTR_ATSCMH_NETWORKING_NETWORK_ID                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 275
#define RSSMXV_ATTR_ATSCMH_NETWORKING_BURY_RATIO                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 276
#define RSSMXV_ATTR_ATSCMH_NETWORKING_MTXID_TRANSMISSION                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 277
#define RSSMXV_ATTR_ATSCMH_NETWORKING_TRANSMITTER_ID                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 278
#define RSSMXV_ATTR_ATSCMH_NETWORKING_MARKET_ID                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 279
#define RSSMXV_ATTR_ATSC3_ENABLED                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 280
#define RSSMXV_ATTR_ATSC3_SET_TO_DEFAULT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 281
#define RSSMXV_ATTR_ATSC3_RECALL_SETTINGS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 282
#define RSSMXV_ATTR_ATSC3_SAVE_SETTINGS                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 284
#define RSSMXV_ATTR_ATSC3_SETTINGS_CATALOG                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 286
#define RSSMXV_ATTR_ATSC3_DELETE_SETTINGS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 288
#define RSSMXV_ATTR_ATSC3_STL_INTERFACE_ENABLED                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 290
#define RSSMXV_ATTR_ATSC3_STL_RESET_LOG                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 291
#define RSSMXV_ATTR_ATSC3_CODER_STATUS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 292
#define RSSMXV_ATTR_ATSC3_INPUT_CONFORMANCE_CHECK                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 293
#define RSSMXV_ATTR_ATSC3_INPUT_NUMBER_OF_PLPS                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 294
#define RSSMXV_ATTR_ATSC3_INPUT_SOURCE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 295
#define RSSMXV_ATTR_ATSC3_INPUT_DESTINATION_IP                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 296
#define RSSMXV_ATTR_ATSC3_INPUT_DESTINATION_PORT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 297
#define RSSMXV_ATTR_ATSC3_INPUT_TYPE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 298
#define RSSMXV_ATTR_ATSC3_INPUT_PROTOCOL                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 299
#define RSSMXV_ATTR_ATSC3_INFO_PLP_PACKET_LENGTH                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 300
#define RSSMXV_ATTR_ATSC3_INFO_PLP_MAX_USEFUL_DATARATE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 301
#define RSSMXV_ATTR_ATSC3_INFO_PLP_MEASURED_DATARATE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 302
#define RSSMXV_ATTR_ATSC3_INFO_PLP_USEFUL_DATARATE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 303
#define RSSMXV_ATTR_ATSC3_TEST_SIGNAL                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 304
#define RSSMXV_ATTR_ATSC3_TEST_IP_PACKET                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 305
#define RSSMXV_ATTR_ATSC3_TEST_TS_PACKET                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 306
#define RSSMXV_ATTR_ATSC3_TEST_PID_TEST_PACKET                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 307
#define RSSMXV_ATTR_ATSC3_TEST_PID                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 308
#define RSSMXV_ATTR_ATSC3_TEST_PAYLOAD                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 309
#define RSSMXV_ATTR_ATSC3_TEST_PRBS_SEQUENCE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 310
#define RSSMXV_ATTR_ATSC3_ENCAPS_ALP_ENCAPSULATION_TYPE                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 311
#define RSSMXV_ATTR_ATSC3_ENCAPS_BB_FRAME_PADDING_ENABLED                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 312
#define RSSMXV_ATTR_ATSC3_ENCAPS_BB_FRAME_COUNTER_ENABLED                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 313
#define RSSMXV_ATTR_ATSC3_BICM_PLP_ID                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 314
#define RSSMXV_ATTR_ATSC3_BICM_PLP_LDM_LAYER                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 315
#define RSSMXV_ATTR_ATSC3_BICM_PLP_LDM_INJECTION_LEVEL                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 316
#define RSSMXV_ATTR_ATSC3_BICM_PLP_SIZE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 317
#define RSSMXV_ATTR_ATSC3_BICM_PLP_SCRAMBLER                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 318
#define RSSMXV_ATTR_ATSC3_BICM_PLP_FEC_TYPE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 319
#define RSSMXV_ATTR_ATSC3_BICM_PLP_CODE_RATE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 320
#define RSSMXV_ATTR_ATSC3_BICM_PLP_CONSTELLATION                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 321
#define RSSMXV_ATTR_ATSC3_BICM_PLP_LOW_LEVEL_SIGNALING                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 322
#define RSSMXV_ATTR_ATSC3_BICM_PLP_INTERLEAVING_TYPE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 323
#define RSSMXV_ATTR_ATSC3_BICM_PLP_INTERLEAVING_NUMBER_OF_SUBSLICES         RS_SPECIFIC_PUBLIC_ATTR_BASE + 324
#define RSSMXV_ATTR_ATSC3_BICM_PLP_INTERLEAVING_SUBSLICE_INTERVAL           RS_SPECIFIC_PUBLIC_ATTR_BASE + 325
#define RSSMXV_ATTR_ATSC3_BICM_PLP_TIME_INTERLEAVER_MODE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 326
#define RSSMXV_ATTR_ATSC3_BICM_PLP_INTERLEAVING_DEPTH_EXTENDED              RS_SPECIFIC_PUBLIC_ATTR_BASE + 327
#define RSSMXV_ATTR_ATSC3_BICM_PLP_INTERLEAVING_DEPTH                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 328
#define RSSMXV_ATTR_ATSC3_BICM_PLP_INTERLEAVING_SUBFRAME_ENABLED            RS_SPECIFIC_PUBLIC_ATTR_BASE + 329
#define RSSMXV_ATTR_ATSC3_BICM_PLP_CELL_INTERLEAVER_ENABLED                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 330
#define RSSMXV_ATTR_ATSC3_BICM_PLP_INTERLEAVING_NUMBER_OF_TI_BLOCKS         RS_SPECIFIC_PUBLIC_ATTR_BASE + 331
#define RSSMXV_ATTR_ATSC3_BICM_PLP_INTERLEAVING_FEC_BLOCKS_MAX              RS_SPECIFIC_PUBLIC_ATTR_BASE + 332
#define RSSMXV_ATTR_ATSC3_BICM_PLP_INTERLEAVING_FEC_BLOCKS                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 333
#define RSSMXV_ATTR_ATSC3_OFDM_NUMBER_OF_SUBFRAMES                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 334
#define RSSMXV_ATTR_ATSC3_OFDM_CHANNEL_BANDWIDTH                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 335
#define RSSMXV_ATTR_ATSC3_OFDM_MIMO_ENABLED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 336
#define RSSMXV_ATTR_ATSC3_OFDM_MISO                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 337
#define RSSMXV_ATTR_ATSC3_OFDM_FFT_SIZE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 338
#define RSSMXV_ATTR_ATSC3_OFDM_REDUCED_CARRIER_MODE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 339
#define RSSMXV_ATTR_ATSC3_OFDM_GUARD_INTERVAL                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 340
#define RSSMXV_ATTR_ATSC3_OFDM_TOTAL_SYMBOLS                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 341
#define RSSMXV_ATTR_ATSC3_OFDM_SUBFRAME_SISO_PILOT_PATTERN                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 342
#define RSSMXV_ATTR_ATSC3_OFDM_SUBFRAME_PILOT_BOOST_MODE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 343
#define RSSMXV_ATTR_ATSC3_OFDM_SUBFRAME_BOUNDARY_SYMBOL_FIRST               RS_SPECIFIC_PUBLIC_ATTR_BASE + 344
#define RSSMXV_ATTR_ATSC3_OFDM_SUBFRAME_BOUNDARY_SYMBOL_LAST                RS_SPECIFIC_PUBLIC_ATTR_BASE + 345
#define RSSMXV_ATTR_ATSC3_OFDM_SUBFRAME_BOUNDARY_NULL_CELLS                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 346
#define RSSMXV_ATTR_ATSC3_AFDM_SUBFRAME_FREQUENCY_INTERLEAVER_ENABLED       RS_SPECIFIC_PUBLIC_ATTR_BASE + 347
#define RSSMXV_ATTR_ATSC3_OFDM_SUBFRAME_DURATION                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 348
#define RSSMXV_ATTR_ATSC3_OFDM_SUBFRAME_USED_BANDWIDTH                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 349
#define RSSMXV_ATTR_ATSC3_OFDM_SUBFRAME_NUMBER_OF_PLPS                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 350
#define RSSMXV_ATTR_ATSC3_OFDM_SUBFRAME_PLP_IDS                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 351
#define RSSMXV_ATTR_ATSC3_NETWORK_MODE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 352
#define RSSMXV_ATTR_ATSC3_NETWORK_LOW_LEVEL_SIGNALLING                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 353
#define RSSMXV_ATTR_ATSC3_NETWORK_TIME                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 354
#define RSSMXV_ATTR_ATSC3_NETWORK_RETURN_CHANNEL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 355
#define RSSMXV_ATTR_ATSC3_NETWORK_MISO_NUMBER_OF_TRANSMITTERS               RS_SPECIFIC_PUBLIC_ATTR_BASE + 356
#define RSSMXV_ATTR_ATSC3_NETWORK_MISO_TRANSMITTER_INDEX                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 357
#define RSSMXV_ATTR_ATSC3_NETWORK_TXID_MODE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 358
#define RSSMXV_ATTR_ATSC3_NETWORK_TXID_ADDRESS                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 359
#define RSSMXV_ATTR_ATSC3_NETWORK_TXID_INJECTION_LEVEL                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 360
#define RSSMXV_ATTR_ATSC3_NETWORK_BROADCAST_STREAM_ID                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 361
#define RSSMXV_ATTR_ATSC3_FRAME_PEAK_TO_AVERAGE_POWER_RATIO                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 362
#define RSSMXV_ATTR_ATSC3_FRAME_MODE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 363
#define RSSMXV_ATTR_ATSC3_FRAME_LENGTH                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 364
#define RSSMXV_ATTR_ATSC3_FRAME_EXCESS_SAMPLES_PER_SYMBOL                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 365
#define RSSMXV_ATTR_ATSC3_FRAME_FINAL_EXCESS_SAMPLES                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 366
#define RSSMXV_ATTR_ATSC3_FRAME_ADDITIONAL_SAMPLES                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 367
#define RSSMXV_ATTR_ATSC3_FRAME_TIME_OFFSET                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 368
#define RSSMXV_ATTR_ATSC3_L1_BASIC_VERSION                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 369
#define RSSMXV_ATTR_ATSC3_L1_BASIC_FEC_TYPE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 370
#define RSSMXV_ATTR_ATSC3_L1_DETAIL_VERSION                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 371
#define RSSMXV_ATTR_ATSC3_L1_DETAIL_FEC_TYPE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 372
#define RSSMXV_ATTR_ATSC3_L1_DETAIL_ADDITIONAL_PARITY_MODE                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 373
#define RSSMXV_ATTR_ATSC3_L1_NUMBER_OF_PREAMBLE_SYMBOLS                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 374
#define RSSMXV_ATTR_ATSC3_L1_REDUCED_CARRIER_MODE_PREAMBLE                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 375
#define RSSMXV_ATTR_ATSC3_L1_PILOT_PATTERN_DX_PREAMBLE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 376
#define RSSMXV_ATTR_ATSC3_L1_NUMBER_OF_RFS                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 377
#define RSSMXV_ATTR_ATSC3_INFO_FRAME_DURATION                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 378
#define RSSMXV_ATTR_ATSC3_INFO_BOOTSTRAP_DURATION                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 379
#define RSSMXV_ATTR_ATSC3_INFO_L1_BASIC_BYTES                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 380
#define RSSMXV_ATTR_ATSC3_INFO_L1_DETAIL_BYTES                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 381
#define RSSMXV_ATTR_ATSC3_INFO_L1_BASIC_CELLS                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 382
#define RSSMXV_ATTR_ATSC3_INFO_L1_DETAIL_CELLS                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 383
#define RSSMXV_ATTR_ATSC3_INFO_BOOTSTRAP_MAJOR_VERSION                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 384
#define RSSMXV_ATTR_ATSC3_INFO_BOOTSTRAP_MINOR_VERSION                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 385
#define RSSMXV_ATTR_ATSC3_INFO_BOOTSTRAP_MIN_TIME_TO_NEXT                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 386
#define RSSMXV_ATTR_ATSC3_INFO_BOOTSTRAP_EMERGENCY_ALERT_SIGNALING          RS_SPECIFIC_PUBLIC_ATTR_BASE + 387
#define RSSMXV_ATTR_ATSC3_INFO_BOOTSTRAP_BANDWIDTH                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 388
#define RSSMXV_ATTR_ATSC3_INFO_BOOTSTRAP_BSR_COEFFICIENT                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 389
#define RSSMXV_ATTR_ATSC3_INFO_STRUCTURE_PREAMBLE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 390
#define RSSMXV_ATTR_ATSC3_INFO_STRUCTURE_FFT_SIZE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 391
#define RSSMXV_ATTR_ATSC3_INFO_STRUCTURE_GUARD_INTERVAL                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 392
#define RSSMXV_ATTR_ATSC3_INFO_STRUCTURE_PILOT_PATTERN_DX                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 393
#define RSSMXV_ATTR_ATSC3_INFO_STRUCTURE_L1_BASIC_FEC_TYPE                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 394
#define RSSMXV_ATTR_ATSC3_SFN_NETWORK_DELAY                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 395
#define RSSMXV_ATTR_ATSC3_SFN_PROCESSING_DELAY                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 396
#define RSSMXV_ATTR_ATSC3_SFN_DYNAMIC_DELAY                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 397
#define RSSMXV_ATTR_ATSC3_SFN_TOTAL_DELAY                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 398
#define RSSMXV_ATTR_ATSC3_SFN_MAX_DEVIATION_TIME                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 399
#define RSSMXV_ATTR_ATSC3_SFN_STATIC_DELAY                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 400
#define RSSMXV_ATTR_ATSC3_SFN_MAXIMUM_DELAY                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 401
#define RSSMXV_ATTR_ATSC3_SFN_DISPATCH_TIME                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 402
#define RSSMXV_ATTR_ATSC3_SFN_MUTE_BOOTSTRAP_OF_FRAME                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 403
#define RSSMXV_ATTR_ATSC3_SFN_MODE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 404
#define RSSMXV_ATTR_ATSC3_SPECIAL_SETTINGS_ENABLED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 405
#define RSSMXV_ATTR_ATSC3_SPECIAL_STL_PREAMBLE_COMPATIBILITY_MODE_ENABLED   RS_SPECIFIC_PUBLIC_ATTR_BASE + 406
#define RSSMXV_ATTR_ATSC3_SPECIAL_STL_TMP_COMPATIBILITY_MODE                RS_SPECIFIC_PUBLIC_ATTR_BASE + 407
#define RSSMXV_ATTR_ATSC3_SPECIAL_AMP_LMT_COMPATIBILITY_MODE_ENABLED        RS_SPECIFIC_PUBLIC_ATTR_BASE + 408
#define RSSMXV_ATTR_ATSC3_SPECIAL_BOOTSTRAP_MINOR_VERSION                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 409
#define RSSMXV_ATTR_ATSC3_SPECIAL_BOOTSTRAP_EMERGENCY_ALERT_SIGNALING       RS_SPECIFIC_PUBLIC_ATTR_BASE + 410
#define RSSMXV_ATTR_DVBT_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 411
#define RSSMXV_ATTR_DVBT_SET_TO_DEFAULT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 412
#define RSSMXV_ATTR_DVBT_RECALL_SETTINGS                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 413
#define RSSMXV_ATTR_DVBT_SAVE_SETTINGS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 415
#define RSSMXV_ATTR_DVBT_SETTINGS_CATALOG                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 417
#define RSSMXV_ATTR_DVBT_DELETE_SETTINGS                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 419
#define RSSMXV_ATTR_DVBT_INPUT_SOURCE_HIGH                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 421
#define RSSMXV_ATTR_DVBT_INPUT_SOURCE_LOW                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 422
#define RSSMXV_ATTR_DVBT_INPUT_HIGH                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 423
#define RSSMXV_ATTR_DVBT_INPUT_LOW                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 424
#define RSSMXV_ATTR_DVBT_INPUT_FORMAT_HIGH                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 425
#define RSSMXV_ATTR_DVBT_INPUT_FORMAT_LOW                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 426
#define RSSMXV_ATTR_DVBT_INPUT_IP_TS_CHANNEL                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 427
#define RSSMXV_ATTR_DVBT_INPUT_STUFFING_HIGH_ENABLED                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 428
#define RSSMXV_ATTR_DVBT_INPUT_STUFFING_LOW_ENABLED                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 429
#define RSSMXV_ATTR_DVBT_INPUT_TEST_SIGNAL_HIGH                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 430
#define RSSMXV_ATTR_DVBT_INPUT_TEST_SIGNAL_LOW                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 431
#define RSSMXV_ATTR_DVBT_INPUT_INFO_PACKET_LENGTH_HIGH                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 432
#define RSSMXV_ATTR_DVBT_INPUT_INFO_PACKET_LENGTH_LOW                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 433
#define RSSMXV_ATTR_DVBT_INPUT_INFO_MAX_USEFUL_DATA_RATE_HIGH               RS_SPECIFIC_PUBLIC_ATTR_BASE + 434
#define RSSMXV_ATTR_DVBT_INPUT_INFO_MAX_USEFUL_DATA_RATE_LOW                RS_SPECIFIC_PUBLIC_ATTR_BASE + 435
#define RSSMXV_ATTR_DVBT_INPUT_INFO_MEAS_DATA_RATE_HIGH                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 436
#define RSSMXV_ATTR_DVBT_INPUT_INFO_MEAS_DATA_RATE_LOW                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 437
#define RSSMXV_ATTR_DVBT_INPUT_INFO_USEFUL_DATA_RATE_LOW                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 438
#define RSSMXV_ATTR_DVBT_INPUT_INFO_USEFUL_DATA_RATE_HIGH                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 439
#define RSSMXV_ATTR_DVBT_INPUT_TEST_SIGNAL_TEST_TS_PACKET                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 440
#define RSSMXV_ATTR_DVBT_INPUT_TEST_SIGNAL_PID_TEST_PACKET                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 441
#define RSSMXV_ATTR_DVBT_INPUT_TEST_SIGNAL_PID                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 442
#define RSSMXV_ATTR_DVBT_INPUT_TEST_SIGNAL_PAYLOAD                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 443
#define RSSMXV_ATTR_DVBT_INPUT_TEST_SIGNAL_PRBS                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 444
#define RSSMXV_ATTR_DVBT_CODING_HIEARCHY                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 445
#define RSSMXV_ATTR_DVBT_CODING_CONSTELLATION                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 446
#define RSSMXV_ATTR_DVBT_CODING_CHANNEL_BANDWIDTH                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 447
#define RSSMXV_ATTR_DVBT_CODING_USED_BW                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 448
#define RSSMXV_ATTR_DVBT_CODING_FFT_MODE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 449
#define RSSMXV_ATTR_DVBT_CODING_GUARD_INTERVAL                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 450
#define RSSMXV_ATTR_DVBT_CODING_DVBH_ENABLED                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 451
#define RSSMXV_ATTR_DVBT_CODING_SYMBOL_INTERLEAVING                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 452
#define RSSMXV_ATTR_DVBT_CODING_CODE_RATE_HIGH                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 453
#define RSSMXV_ATTR_DVBT_CODING_CODE_RATE_LOW                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 454
#define RSSMXV_ATTR_DVBT_TPS_CELL_ID                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 455
#define RSSMXV_ATTR_DVBT_TPS_RESERVED_STATE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 456
#define RSSMXV_ATTR_DVBT_TPS_RESERVED_BITS                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 457
#define RSSMXV_ATTR_DVBT_TPS_TIMESLICE_LOW_ENABLED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 458
#define RSSMXV_ATTR_DVBT_TPS_TIMESLICE_HIGH_ENABLED                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 459
#define RSSMXV_ATTR_DVBT_TPS_MPE_FEC_LOW_ENABLED                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 460
#define RSSMXV_ATTR_DVBT_TPS_MPE_FEC_HIGH_ENABLED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 461
#define RSSMXV_ATTR_DVBT_SPECIAL_SETTINGS_ENABLED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 462
#define RSSMXV_ATTR_DVBT_SPECIAL_SETTINGS_REED_SOLOMON_HIGH_ENABLED         RS_SPECIFIC_PUBLIC_ATTR_BASE + 463
#define RSSMXV_ATTR_DVBT_SPECIAL_SETTINGS_REED_SOLOMON_LOW_ENABLED          RS_SPECIFIC_PUBLIC_ATTR_BASE + 464
#define RSSMXV_ATTR_DVBT2_ENABLED                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 465
#define RSSMXV_ATTR_DVBT2_SET_TO_DEFAULT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 466
#define RSSMXV_ATTR_DVBT2_RECALL_SETTINGS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 467
#define RSSMXV_ATTR_DVBT2_SAVE_SETTINGS                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 469
#define RSSMXV_ATTR_DVBT2_SETTINGS_CATALOG                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 471
#define RSSMXV_ATTR_DVBT2_DELETE_SETTINGS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 473
#define RSSMXV_ATTR_DVBT2_INPUT_T2MI_INTERFACE_ENABLED                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 475
#define RSSMXV_ATTR_DVBT2_INPUT_SOURCE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 476
#define RSSMXV_ATTR_DVBT2_INPUT_NUMBER_OF_PLPS                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 477
#define RSSMXV_ATTR_DVBT2_INPUT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 478
#define RSSMXV_ATTR_DVBT2_INPUT_FORMAT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 479
#define RSSMXV_ATTR_DVBT2_INPUT_IP_TS_CHANNEL                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 480
#define RSSMXV_ATTR_DVBT2_INPUT_T2MI_PID                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 481
#define RSSMXV_ATTR_DVBT2_INPUT_T2MI_SID                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 482
#define RSSMXV_ATTR_DVBT2_INPUT_RESET_LOG_FILE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 483
#define RSSMXV_ATTR_DVBT2_INPUT_STATUS_INFO                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 484
#define RSSMXV_ATTR_DVBT2_INPUT_PLP_TEST_SIGNAL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 485
#define RSSMXV_ATTR_DVBT2_INPUT_PLP_STUFFING_ENABLED                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 486
#define RSSMXV_ATTR_DVBT2_PLP_INPUT_FORMAT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 487
#define RSSMXV_ATTR_DVBT2_INPUT_PACKET_LENGTH                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 488
#define RSSMXV_ATTR_DVBT2_INPUT_MAX_USEFUL_DATA_RATE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 489
#define RSSMXV_ATTR_DVBT2_INPUT_MEAS_DATA_RATE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 490
#define RSSMXV_ATTR_DVBT2_INPUT_USEFUL_DATA_RATE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 491
#define RSSMXV_ATTR_DVBT2_T2MI_MEASUREMENT_MODE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 492
#define RSSMXV_ATTR_DVBT2_INPUT_T2MI_TIME_MAX                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 493
#define RSSMXV_ATTR_DVBT2_INPUT_T2MI_TIME_MIN                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 494
#define RSSMXV_ATTR_DVBT2_INPUT_TEST_SIGNAL_TEST_TS_PACKET                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 495
#define RSSMXV_ATTR_DVBT2_INPUT_TEST_SIGNAL_PID_TEST_PACKET                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 496
#define RSSMXV_ATTR_DVBT2_INPUT_TEST_SIGNAL_PID                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 497
#define RSSMXV_ATTR_DVBT2_INPUT_TEST_SIGNAL_PAYLOAD                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 498
#define RSSMXV_ATTR_DVBT2_INPUT_TEST_SIGNAL_PRBS                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 499
#define RSSMXV_ATTR_DVBT2_MODE_STREAM_PLP_ID                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 500
#define RSSMXV_ATTR_DVBT2_MODE_STREAM_PLP_GROUP_ID                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 501
#define RSSMXV_ATTR_DVBT2_MODE_STREAM_PLP_TYPE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 502
#define RSSMXV_ATTR_DVBT2_MODE_STREAM_PLP_BB_MODE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 503
#define RSSMXV_ATTR_DVBT2_MODE_STREAM_PLP_ISSY                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 504
#define RSSMXV_ATTR_DVBT2_MODE_STREAM_PLP_NULL_PACKET_DELETION_ENABLED      RS_SPECIFIC_PUBLIC_ATTR_BASE + 505
#define RSSMXV_ATTR_DVBT2_MODE_STREAM_PLP_IN_BAND_SIGNALING_ENABLED         RS_SPECIFIC_PUBLIC_ATTR_BASE + 506
#define RSSMXV_ATTR_DVBT2_MODE_STREAM_PLP_IN_BAND_SIGNALING_A_ENABLED       RS_SPECIFIC_PUBLIC_ATTR_BASE + 507
#define RSSMXV_ATTR_DVBT2_MODE_STREAM_PLP_IN_BAND_SIGNALING_B_ENABLED       RS_SPECIFIC_PUBLIC_ATTR_BASE + 508
#define RSSMXV_ATTR_DVBT2_MODE_STREAM_PLP_OTHER_IN_BAND_PLPS                RS_SPECIFIC_PUBLIC_ATTR_BASE + 509
#define RSSMXV_ATTR_DVBT2_MODE_STREAM_PLP_CM_TYPE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 510
#define RSSMXV_ATTR_DVBT2_MODE_STREAM_PLP_STATIC_FLAG_ENABLED               RS_SPECIFIC_PUBLIC_ATTR_BASE + 511
#define RSSMXV_ATTR_DVBT2_MODE_STREAM_PLP_STATIC_PADDING_FLAG_ENABLED       RS_SPECIFIC_PUBLIC_ATTR_BASE + 512
#define RSSMXV_ATTR_DVBT2_BCIM_PLP_FEC_FRAME                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 513
#define RSSMXV_ATTR_DVBT2_BCIM_PLP_CODE_RATE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 514
#define RSSMXV_ATTR_DVBT2_BCIM_PLP_CONSTELLATION                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 515
#define RSSMXV_ATTR_DVBT2_BCIM_PLP_CONSTELLATION_ROTATION_ENABLED           RS_SPECIFIC_PUBLIC_ATTR_BASE + 516
#define RSSMXV_ATTR_DVBT2_BCIM_PLP_TIME_INTERLEAVER                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 517
#define RSSMXV_ATTR_DVBT2_BCIM_PLP_FRAME_INTERLEAVER                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 518
#define RSSMXV_ATTR_DVBT2_BCIM_PLP_TIME_INTERLEAVER_LENGTH                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 519
#define RSSMXV_ATTR_DVBT2_BCIM_PLP_FIRST_FRAME_INDEX                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 520
#define RSSMXV_ATTR_DVBT2_BICM_PLP_FEC_BLOCKS_PER_IF                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 521
#define RSSMXV_ATTR_DVBT2_BICM_PLP_MAX_NUMBER_OF_BLOCKS                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 522
#define RSSMXV_ATTR_DVBT2_OFDM_CHANNEL_BANDWIDTH                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 523
#define RSSMXV_ATTR_DVBT2_OFDM_USED_BANDWIDTH                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 524
#define RSSMXV_ATTR_DVBT2_OFDM_BANDWIDTH_VARIATION                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 525
#define RSSMXV_ATTR_DVBT2_OFDM_FFT_MODE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 526
#define RSSMXV_ATTR_DVBT2_OFDM_GUARD_INTERVAL                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 527
#define RSSMXV_ATTR_DVBT2_OFDM_PILOT_PATTERN                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 528
#define RSSMXV_ATTR_DVBT2_OFDM_T2_FRAMES_PER_SUPER_FRAME                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 529
#define RSSMXV_ATTR_DVBT2_OFDM_SYMBOLS_PER_T2_FRAME                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 530
#define RSSMXV_ATTR_DVBT2_OFDM_DATA_SYMBOLS_PER_T2_FRAME                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 531
#define RSSMXV_ATTR_DVBT2_OFDM_SUBSLICES_PER_T2_FRAME                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 532
#define RSSMXV_ATTR_DVBT2_SYSTEM_NETWORK_MODE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 533
#define RSSMXV_ATTR_DVBT2_SYSTEM_NETWORK_PROFILE_MODE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 534
#define RSSMXV_ATTR_DVBT2_SYSTEM_NETWORK_TRANSMISSION_SYSTEM                RS_SPECIFIC_PUBLIC_ATTR_BASE + 535
#define RSSMXV_ATTR_DVBT2_SYSTEM_NETWORK_MISO_MODE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 536
#define RSSMXV_ATTR_DVBT2_SYSTEM_NETWORK_MISO_GROUP                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 537
#define RSSMXV_ATTR_DVBT2_SYSTEM_NETWORK_PEAK_TO_AVERAGE_POWER_RATIO        RS_SPECIFIC_PUBLIC_ATTR_BASE + 538
#define RSSMXV_ATTR_DVBT2_SYSTEM_NETWORK_FUTURE_EXTENSION_FRAMES_ENABLED    RS_SPECIFIC_PUBLIC_ATTR_BASE + 539
#define RSSMXV_ATTR_DVBT2_SYSTEM_NETWORK_FUTURE_EXTENSION_FRAMES_TYPE       RS_SPECIFIC_PUBLIC_ATTR_BASE + 540
#define RSSMXV_ATTR_DVBT2_SYSTEM_NETWORK_FUTURE_EXTENSION_FRAMES_LENGTH     RS_SPECIFIC_PUBLIC_ATTR_BASE + 541
#define RSSMXV_ATTR_DVBT2_SYSTEM_NETWORK_FUTURE_EXTENSION_FRAMES_INTERVAL   RS_SPECIFIC_PUBLIC_ATTR_BASE + 542
#define RSSMXV_ATTR_DVBT2_SYSTEM_NETWORK_FUTURE_EXTENSION_FRAMES_PAYLOAD    RS_SPECIFIC_PUBLIC_ATTR_BASE + 543
#define RSSMXV_ATTR_DVBT2_SYSTEM_NETWORK_TIME_FREQUENCY_SLICING             RS_SPECIFIC_PUBLIC_ATTR_BASE + 544
#define RSSMXV_ATTR_DVBT2_SYSTEM_L1_T2_VERSION                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 545
#define RSSMXV_ATTR_DVBT2_SYSTEM_L1_POST_MODULATION                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 546
#define RSSMXV_ATTR_DVBT2_SYSTEM_L1_REPETITION_ENABLED                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 547
#define RSSMXV_ATTR_DVBT2_SYSTEM_L1_POST_EXTENSION                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 548
#define RSSMXV_ATTR_DVBT2_SYSTEM_L1_POST_SCRAMBLING_ENABLED                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 549
#define RSSMXV_ATTR_DVBT2_SYSTEM_L1_BASE_LITE_ENABLED                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 550
#define RSSMXV_ATTR_DVBT2_SYSTEM_L1_NUMBER_OF_AUX_STREAMS                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 551
#define RSSMXV_ATTR_DVBT2_SYSTEM_L1_RF_SIGNALING_ENABLED                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 552
#define RSSMXV_ATTR_DVBT2_SYSTEM_L1_RF_SIGNALING_FREQUENCY                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 553
#define RSSMXV_ATTR_DVBT2_SYSTEM_CELL_ID                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 554
#define RSSMXV_ATTR_DVBT2_SYSTEM_NETWORK_ID                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 555
#define RSSMXV_ATTR_DVBT2_SYSTEM_T2_ID                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 556
#define RSSMXV_ATTR_DVBT2_SYSTEM_TX_ID_AVAILABILITY                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 557
#define RSSMXV_ATTR_DVBT2_SFN_PROCESSING_DELAY                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 558
#define RSSMXV_ATTR_DVBT2_SFN_STATIC_DELAY                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 559
#define RSSMXV_ATTR_DVBT2_SFN_DYNAMIC_DELAY                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 560
#define RSSMXV_ATTR_DVBT2_SFN_TOTAL_DELAY                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 561
#define RSSMXV_ATTR_DVBT2_SFN_DELAY_MAX_DEVIATION_TIME                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 562
#define RSSMXV_ATTR_DVBT2_SFN_DELAY_MUTE_1ST_SUPER_FRAME_P1_ENABLED         RS_SPECIFIC_PUBLIC_ATTR_BASE + 563
#define RSSMXV_ATTR_DVBT2_SFN_TIMESTAMP_MODE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 564
#define RSSMXV_ATTR_DVBT2_SFN_UPDATE_TIMESTAMP                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 565
#define RSSMXV_ATTR_DVBT2_SFN_UTC_DATE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 566
#define RSSMXV_ATTR_DVBT2_SFN_UTC_TIME                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 567
#define RSSMXV_ATTR_DVBT2_SFN_OFFSET                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 568
#define RSSMXV_ATTR_DVBT2_SFN_SECONDS_SINCE_2000                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 569
#define RSSMXV_ATTR_DVBT2_FRAME_INFO_SUPER_FRAME_DURATION                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 570
#define RSSMXV_ATTR_DVBT2_FRAME_INFO_T2_FRAME_DURATION                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 571
#define RSSMXV_ATTR_DVBT2_FRAME_INFO_FUTURE_EXTENSION_FRAME_DURATION        RS_SPECIFIC_PUBLIC_ATTR_BASE + 572
#define RSSMXV_ATTR_DVBT2_FRAME_INFO_P1_SYMBOL_DURATION                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 573
#define RSSMXV_ATTR_DVBT2_FRAME_INFO_OFDM_SYMBOL_DURATION_TS                RS_SPECIFIC_PUBLIC_ATTR_BASE + 574
#define RSSMXV_ATTR_DVBT2_FRAME_INFO_OFDM_SYMBOL_DURATION_TP2               RS_SPECIFIC_PUBLIC_ATTR_BASE + 575
#define RSSMXV_ATTR_DVBT2_FRAME_INFO_L1_PRE_SIGNALING_BITS                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 576
#define RSSMXV_ATTR_DVBT2_FRAME_INFO_L1_PRE_SIGNALING_CELLS                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 577
#define RSSMXV_ATTR_DVBT2_FRAME_INFO_L1_POST_SIGNALING_BITS                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 578
#define RSSMXV_ATTR_DVBT2_FRAME_INFO_L1_POST_SIGNALING_CELLS                RS_SPECIFIC_PUBLIC_ATTR_BASE + 579
#define RSSMXV_ATTR_DVBT2_FRAME_INFO_PLP_DATA_CELLS_PER_T2_FRAME            RS_SPECIFIC_PUBLIC_ATTR_BASE + 580
#define RSSMXV_ATTR_DVBT2_FRAME_INFO_PLP_DATA_CELLS_IN_CURRENT_T2_FRAME     RS_SPECIFIC_PUBLIC_ATTR_BASE + 581
#define RSSMXV_ATTR_DVB_C_ENABLED                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1556
#define RSSMXV_ATTR_DVB_C_SET_TO_DEFAULT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1557
#define RSSMXV_ATTR_DVB_C_RECALL_SETTINGS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1558
#define RSSMXV_ATTR_DVB_C_SAVE_SETTINGS                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1559
#define RSSMXV_ATTR_DVB_C_SETTINGS_CATALOG                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1560
#define RSSMXV_ATTR_DVB_C_DELETE_SETTINGS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1561
#define RSSMXV_ATTR_DVB_C_INPUT_SOURCE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1562
#define RSSMXV_ATTR_DVB_C_INPUT_STUFFING_STATE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1563
#define RSSMXV_ATTR_DVB_C_INPUT_INTERFACE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1564
#define RSSMXV_ATTR_DVB_C_INPUT_FORMAT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1565
#define RSSMXV_ATTR_DVB_C_INPUT_TS_CHANNEL                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1566
#define RSSMXV_ATTR_DVB_C_INPUT_INFO_PACKET_LENGTH                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1567
#define RSSMXV_ATTR_DVB_C_INPUT_INFO_MAXIMUM_USEFUL_DATA_RATE               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1568
#define RSSMXV_ATTR_DVB_C_INPUT_INFO_USEFUL_DATA_RATE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1569
#define RSSMXV_ATTR_DVB_C_INPUT_INFO_MEASURED_DATA_RATE                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1570
#define RSSMXV_ATTR_DVB_C_INPUT_TEST_SIGNAL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1571
#define RSSMXV_ATTR_DVB_C_INPUT_TEST_SIGNAL_TEST_TS_PACKET                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1572
#define RSSMXV_ATTR_DVB_C_INPUT_TEST_SIGNAL_PID_TEST_PACKET                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1573
#define RSSMXV_ATTR_DVB_C_INPUT_TEST_SIGNAL_PID                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1574
#define RSSMXV_ATTR_DVB_C_INPUT_TEST_SIGNAL_PAYLOAD                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1575
#define RSSMXV_ATTR_DVB_C_INPUT_TEST_SIGNAL_PRBS                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1576
#define RSSMXV_ATTR_DVB_C_CODING_SYMBOL_RATE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1577
#define RSSMXV_ATTR_DVB_C_CODING_CONSTELLATION                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1578
#define RSSMXV_ATTR_DVB_C_CODING_ROLL_OFF_FACTOR                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1579
#define RSSMXV_ATTR_DVB_C_SPECIAL_SETTINGS_ENABLED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1580
#define RSSMXV_ATTR_DVB_C_SPECIAL_SETTINGS_REED_SOLOMON_ENABLED             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1581
#define RSSMXV_ATTR_ISDBT_ENABLED                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 582
#define RSSMXV_ATTR_ISDBT_SET_TO_DEFAULT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 583
#define RSSMXV_ATTR_ISDBT_RECALL_SETTINGS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 584
#define RSSMXV_ATTR_ISDBT_SAVE_SETTINGS                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 586
#define RSSMXV_ATTR_ISDBT_DELETE_SETTINGS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 588
#define RSSMXV_ATTR_ISDBT_SETTINGS_CATALOG                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 590
#define RSSMXV_ATTR_ISDBT_CONTROL                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 592
#define RSSMXV_ATTR_ISDBT_SYSTEM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 593
#define RSSMXV_ATTR_ISDBT_INPUT_SOURCE_LAYER_A                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 594
#define RSSMXV_ATTR_ISDBT_INPUT_SOURCE_LAYER_B                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 595
#define RSSMXV_ATTR_ISDBT_INPUT_SOURCE_LAYER_C                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 596
#define RSSMXV_ATTR_ISDBT_INPUT_EXTERNAL                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 597
#define RSSMXV_ATTR_ISDBT_INPUT_FORMAT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 598
#define RSSMXV_ATTR_ISDBT_INPUT_IP_TS_CHANNEL                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 599
#define RSSMXV_ATTR_ISDBT_INPUT_REMUX_ENABLED                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 600
#define RSSMXV_ATTR_ISDBT_INPUT_INFO_PACKET_LENGTH                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 601
#define RSSMXV_ATTR_ISDBT_INPUT_INFO_STUFFING_ENABLED                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 602
#define RSSMXV_ATTR_ISDBT_INPUT_INFO_USEFUL_DATA_RATE_A                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 603
#define RSSMXV_ATTR_ISDBT_INPUT_INFO_USEFUL_DATA_RATE_B                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 604
#define RSSMXV_ATTR_ISDBT_INPUT_INFO_USEFUL_DATA_RATE_C                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 605
#define RSSMXV_ATTR_ISDBT_INPUT_INFO_USEFUL_DATA_RATE_MAXIMUM_A             RS_SPECIFIC_PUBLIC_ATTR_BASE + 606
#define RSSMXV_ATTR_ISDBT_INPUT_INFO_USEFUL_DATA_RATE_MAXIMUM_B             RS_SPECIFIC_PUBLIC_ATTR_BASE + 607
#define RSSMXV_ATTR_ISDBT_INPUT_INFO_USEFUL_DATA_RATE_MAXIMUM_C             RS_SPECIFIC_PUBLIC_ATTR_BASE + 608
#define RSSMXV_ATTR_ISDBT_INPUT_INFO_MEASURED_DATA_RATE                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 609
#define RSSMXV_ATTR_ISDBT_TEST_SIGNAL_TS_PACKET                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 610
#define RSSMXV_ATTR_ISDBT_TEST_SIGNAL_PID_PACKET                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 611
#define RSSMXV_ATTR_ISDBT_TEST_SIGNAL_PID                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 612
#define RSSMXV_ATTR_ISDBT_TEST_SIGNAL_PAYLOAD                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 613
#define RSSMXV_ATTR_ISDBT_TEST_SIGNAL_PRBS                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 614
#define RSSMXV_ATTR_ISDBT_TEST_SIGNAL_DATA_A                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 615
#define RSSMXV_ATTR_ISDBT_TEST_SIGNAL_DATA_B                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 616
#define RSSMXV_ATTR_ISDBT_TEST_SIGNAL_DATA_C                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 617
#define RSSMXV_ATTR_ISDBT_CODING_IIP_PID                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 618
#define RSSMXV_ATTR_ISDBT_CODING_MODE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 619
#define RSSMXV_ATTR_ISDBT_CODING_CHANNEL_BANDWIDTH                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 620
#define RSSMXV_ATTR_ISDBT_CODING_GUARD_INTERVAL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 621
#define RSSMXV_ATTR_ISDBT_CODING_SUBCHANNEL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 622
#define RSSMXV_ATTR_ISDBT_USED_BANDWIDTH                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 623
#define RSSMXV_ATTR_ISDBT_CODING_PORTION_A                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 624
#define RSSMXV_ATTR_ISDBT_CODING_CONSTELLATION_A                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 625
#define RSSMXV_ATTR_ISDBT_CODING_SEGMENTS_A                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 626
#define RSSMXV_ATTR_ISDBT_CODING_RATE_A                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 627
#define RSSMXV_ATTR_ISDBT_CODING_TIME_INTERLEAVING_A                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 628
#define RSSMXV_ATTR_ISDBT_CODING_PORTION_B                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 629
#define RSSMXV_ATTR_ISDBT_CODING_CONSTELLATION_B                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 630
#define RSSMXV_ATTR_ISDBT_CODING_RATE_B                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 631
#define RSSMXV_ATTR_ISDBT_CODING_TIME_INTERLEAVING_B                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 632
#define RSSMXV_ATTR_ISDBT_CODING_SEGMENTS_B                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 633
#define RSSMXV_ATTR_ISDBT_CODING_PORTION_C                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 634
#define RSSMXV_ATTR_ISDBT_CODING_CONSTELLATION_C                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 635
#define RSSMXV_ATTR_ISDBT_CODING_SEGMENTS_C                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 636
#define RSSMXV_ATTR_ISDBT_CODING_RATE_C                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 637
#define RSSMXV_ATTR_ISDBT_CODING_TIME_INTERLEAVING_C                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 638
#define RSSMXV_ATTR_ISDBT_EEW_ENABLED                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 639
#define RSSMXV_ATTR_ISDBT_EEW_NUMBER_OF_EPICENTER                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 640
#define RSSMXV_ATTR_ISDBT_EEW_SIGNAL_TYPE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 641
#define RSSMXV_ATTR_ISDBT_EEW_AREA_INFORMATION                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 642
#define RSSMXV_ATTR_ISDBT_EEW_APPLY_SIGNAL_SETTINGS                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 643
#define RSSMXV_ATTR_ISDBT_EEW_INFORMATION_TYPE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 644
#define RSSMXV_ATTR_ISDBT_EEW_WARNING_ID                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 645
#define RSSMXV_ATTR_ISDBT_EEW_LATITUDE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 646
#define RSSMXV_ATTR_ISDBT_EEW_LONGITUDE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 647
#define RSSMXV_ATTR_ISDBT_EEW_DEPTH                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 648
#define RSSMXV_ATTR_ISDBT_EEW_OCCURENCE_TIME                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 649
#define RSSMXV_ATTR_ISDBT_EEW_APPLY_EPICENTER_1                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 650
#define RSSMXV_ATTR_ISDBT_EEW_APPLY_EPICENTER_2                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 651
#define RSSMXV_ATTR_ISDBT_SPECIAL_ENABLED                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 652
#define RSSMXV_ATTR_ISDBT_REED_SOLOMON_ENABLED                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 653
#define RSSMXV_ATTR_ISDBT_SPECIAL_ALERT_BROADCASTING_FLAG_ENABLED           RS_SPECIFIC_PUBLIC_ATTR_BASE + 654
#define RSSMXV_ATTR_ISDBT_SPECIAL_AC_DATA                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 655
#define RSSMXV_ATTR_ISDBT_SPECIAL_TX_PARAMETER_SWITCHING_INDICATOR          RS_SPECIFIC_PUBLIC_ATTR_BASE + 656
#define RSSMXV_ATTR_ISDBT_SPECIAL_TMCC_NEXT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 657
#define RSSMXV_ATTR_DTMB_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 658
#define RSSMXV_ATTR_DTMB_SET_TO_DEFAULT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 659
#define RSSMXV_ATTR_DTMB_RECALL_SETTINGS                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 660
#define RSSMXV_ATTR_DTMB_SAVE_SETTINGS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 662
#define RSSMXV_ATTR_DTMB_DELETE_SETTINGS                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 664
#define RSSMXV_ATTR_DTMB_SETTINGS_CATALOG                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 666
#define RSSMXV_ATTR_DTMB_SINGLE_CARRIER_MODE_ENABLED                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 668
#define RSSMXV_ATTR_DTMB_DUAL_PILOT_TONE_ENABLED                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 669
#define RSSMXV_ATTR_DTMB_INPUT_SOURCE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 670
#define RSSMXV_ATTR_DTMB_INPUT_EXTERNAL_INTERFACE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 671
#define RSSMXV_ATTR_DTMB_INPUT_FORMAT                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 672
#define RSSMXV_ATTR_DTMB_INPUT_IP_TS_CHANNEL                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 673
#define RSSMXV_ATTR_DTMB_INPUT_STUFFING_ENABLED                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 674
#define RSSMXV_ATTR_DTMB_INFO_PACKET_LENGTH                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 675
#define RSSMXV_ATTR_DTMB_INFO_USEFUL_DATA_RATE_MAX                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 676
#define RSSMXV_ATTR_DTMB_INFO_MEASURED_DATA_RATE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 677
#define RSSMXV_ATTR_DTMB_INFO_USEFUL_DATA_RATE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 678
#define RSSMXV_ATTR_DTMB_TEST_TS_PACKET                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 679
#define RSSMXV_ATTR_DTMB_TEST_SIGNAL                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 680
#define RSSMXV_ATTR_DTMB_TEST_PID_PACKET                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 681
#define RSSMXV_ATTR_DTMB_TEST_PID_HEX                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 682
#define RSSMXV_ATTR_DTMB_TEST_PAYLOAD                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 683
#define RSSMXV_ATTR_DTMB_TEST_SIGNAL_PRBS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 684
#define RSSMXV_ATTR_DTMB_CODING_CHANNEL_BANDWIDTH                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 685
#define RSSMXV_ATTR_DTMB_CODING_CONTROL_FRAMES                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 686
#define RSSMXV_ATTR_DTMB_CODING_CONSTELLATION                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 687
#define RSSMXV_ATTR_DTMB_CODING_CODE_RATE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 688
#define RSSMXV_ATTR_DTMB_CODING_GUARD_INTERVAL                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 689
#define RSSMXV_ATTR_DTMB_CODING_GUARD_INTERVAL_PN                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 690
#define RSSMXV_ATTR_DTMB_CODING_TIME_INTERLEAVER                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 691
#define RSSMXV_ATTR_DTMB_SPECIAL_SETTINGS_ENABLED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 692
#define RSSMXV_ATTR_DTMB_SPECIAL_SETTINGS_SI_POWER_NORMALIZATION_ENABLED    RS_SPECIFIC_PUBLIC_ATTR_BASE + 693
#define RSSMXV_ATTR_DVBS_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 694
#define RSSMXV_ATTR_DVBS_SET_TO_DEFAULT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 695
#define RSSMXV_ATTR_DVBS_RECALL_SETTINGS                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 696
#define RSSMXV_ATTR_DVBS_SAVE_SETTINGS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 698
#define RSSMXV_ATTR_DVBS_SETTINGS_CATALOG                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 700
#define RSSMXV_ATTR_DVBS_DELETE_SETTINGS                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 702
#define RSSMXV_ATTR_DVBS_SOURCE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 704
#define RSSMXV_ATTR_DVBS_INPUT                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 705
#define RSSMXV_ATTR_DVBS_INPUT_FORMAT                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 706
#define RSSMXV_ATTR_DVBS_IP_TS_CHANNEL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 707
#define RSSMXV_ATTR_DVBS_STUFFING_ENABLED                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 708
#define RSSMXV_ATTR_DVBS_PACKET_LENGTH                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 709
#define RSSMXV_ATTR_DVBS_MAX_USEFUL_DATA_RATE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 710
#define RSSMXV_ATTR_DVBS_MEASURED_DATA_RATE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 711
#define RSSMXV_ATTR_DVBS_USEFUL_DATA_RATE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 712
#define RSSMXV_ATTR_DVBS_TEST_SIGNAL                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 713
#define RSSMXV_ATTR_DVBS_TEST_TS_PACKET                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 714
#define RSSMXV_ATTR_DVBS_PID_TEST_PACKET                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 715
#define RSSMXV_ATTR_DVBS_PID                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 716
#define RSSMXV_ATTR_DVBS_PAYLOAD_TEST                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 717
#define RSSMXV_ATTR_DVBS_PRBS                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 718
#define RSSMXV_ATTR_DVBS_CODING_SYMBOL_RATE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 719
#define RSSMXV_ATTR_DVBS_CODING_CONSTELLATION                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 720
#define RSSMXV_ATTR_DVBS_CODING_ROLL_OFF                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 721
#define RSSMXV_ATTR_DVBS_CODING_CODE_RATE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 722
#define RSSMXV_ATTR_DVBS_SPECIAL_SETTINGS_ENABLED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 723
#define RSSMXV_ATTR_DVBS_SPECIAL_REED_SOLOMON_ENABLED                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 724
#define RSSMXV_ATTR_DVBS2_ENABLED                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 725
#define RSSMXV_ATTR_DVBS2_SET_TO_DEFAULT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 726
#define RSSMXV_ATTR_DVBS2_CM_MODE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 727
#define RSSMXV_ATTR_DVBS2_ANNEX_M_ENABLED                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 728
#define RSSMXV_ATTR_DVBS2_S2X_ENABLED                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 729
#define RSSMXV_ATTR_DVBS2_RECALL_SETTINGS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 730
#define RSSMXV_ATTR_DVBS2_SAVE_SETTINGS                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 732
#define RSSMXV_ATTR_DVBS2_SETTINGS_CATALOG                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 734
#define RSSMXV_ATTR_DVBS2_DELETE_SETTINGS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 736
#define RSSMXV_ATTR_DVBS2_NUMBER_OF_INPUT_STREAMS                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 738
#define RSSMXV_ATTR_DVBS2_SOURCE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 739
#define RSSMXV_ATTR_DVBS2_INPUT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 740
#define RSSMXV_ATTR_DVBS2_INPUT_FORMAT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 741
#define RSSMXV_ATTR_DVBS2_IP_TS_CHANNEL                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 742
#define RSSMXV_ATTR_DVBS2_STUFFING_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 743
#define RSSMXV_ATTR_DVBS2_PACKET_LENGTH                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 744
#define RSSMXV_ATTR_DVBS2_MAX_USEFUL_DATA_RATE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 745
#define RSSMXV_ATTR_DVBS2_MEASURED_DATA_RATE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 746
#define RSSMXV_ATTR_DVBS2_USEFUL_DATA_RATE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 747
#define RSSMXV_ATTR_DVBS2_TEST_SIGNAL                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 748
#define RSSMXV_ATTR_DVBS2_TEST_TS_PACKET                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 749
#define RSSMXV_ATTR_DVBS2_PID_TEST_PACKET                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 750
#define RSSMXV_ATTR_DVBS2_PID                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 751
#define RSSMXV_ATTR_DVBS2_PAYLOAD_TEST                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 752
#define RSSMXV_ATTR_DVBS2_PRBS                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 753
#define RSSMXV_ATTR_DVBS2_CODING_SYMBOL_RATE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 754
#define RSSMXV_ATTR_DVBS2_CODING_ROLL_OFF                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 755
#define RSSMXV_ATTR_DVBS2_CODING_NUMBER_OF_TIME_SLICES                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 756
#define RSSMXV_ATTR_DVBS2_CODING_SUPER_FRAME_ENABLED                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 757
#define RSSMXV_ATTR_DVBS2_CODING_CHANNEL_BONDING                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 758
#define RSSMXV_ATTR_DVBS2_CODING_TSN                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 759
#define RSSMXV_ATTR_DVBS2_CODING_ISI                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 760
#define RSSMXV_ATTR_DVBS2_CODING_NUMBER_OF_SYMBOLS                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 761
#define RSSMXV_ATTR_DVBS2_CODING_MODULATION_CODING                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 762
#define RSSMXV_ATTR_DVBS2_CODING_FEC_FRAME                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 763
#define RSSMXV_ATTR_DVBS2_CODING_PILOTS_ENABLED                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 764
#define RSSMXV_ATTR_DVBS2_SPECIAL_SETTINGS_ENABLED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 765
#define RSSMXV_ATTR_DVBS2_SPECIAL_PL_GOLD_CODE_INDEX                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 766
#define RSSMXV_ATTR_DVBS2_SPECIAL_PL_SCRAMBLING_SEQUENCE_ID                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 767
#define RSSMXV_ATTR_DVBS2_SPECIAL_PL_SCRAMBLER_ENABLED                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 768
#define RSSMXV_ATTR_DVBS2_SPECIAL_PL_DATASLICES_PRBS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 769
#define RSSMXV_ATTR_J83B_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1582
#define RSSMXV_ATTR_J83B_SET_TO_DEFAULT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1583
#define RSSMXV_ATTR_J83B_RECALL_SETTINGS                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1584
#define RSSMXV_ATTR_J83B_SAVE_SETTINGS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1585
#define RSSMXV_ATTR_J83B_SETTINGS_CATALOG                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1586
#define RSSMXV_ATTR_J83B_DELETE_SETTINGS                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1587
#define RSSMXV_ATTR_J83B_INPUT_SIGNAL_SOURCE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1588
#define RSSMXV_ATTR_J83B_INPUT_SIGNAL_STUFFING_STATE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1589
#define RSSMXV_ATTR_J83B_INPUT_SIGNAL_INTERFACE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1590
#define RSSMXV_ATTR_J83B_INPUT_SIGNAL_FORMAT                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1591
#define RSSMXV_ATTR_J83B_INPUT_SIGNAL_TS_CHANNEL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1592
#define RSSMXV_ATTR_J83B_INPUT_INFO_PACKET_LENGTH                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1593
#define RSSMXV_ATTR_J83B_INPUT_INFO_MAXIMUM_USEFUL_DATA_RATE                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1594
#define RSSMXV_ATTR_J83B_INPUT_INFO_USEFUL_DATA_RATE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1595
#define RSSMXV_ATTR_J83B_INPUT_INFO_MEASURED_DATA_RATE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1596
#define RSSMXV_ATTR_J83B_INPUT_TEST_SIGNAL                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1597
#define RSSMXV_ATTR_J83B_INPUT_TEST_SIGNAL_TS_TEST_PACKET                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1598
#define RSSMXV_ATTR_J83B_INPUT_TEST_SIGNAL_PID_TEST_PACKET                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1599
#define RSSMXV_ATTR_J83B_INPUT_TEST_SIGNAL_PID                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1600
#define RSSMXV_ATTR_J83B_INPUT_TEST_SIGNAL_PAYLOAD                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1601
#define RSSMXV_ATTR_J83B_INPUT_TEST_SIGNAL_PRBS                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1602
#define RSSMXV_ATTR_J83B_CODING_SYMBOL_RATE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1603
#define RSSMXV_ATTR_J83B_CODING_CONSTELLATION                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1604
#define RSSMXV_ATTR_J83B_CODING_ROLL_OFF_FACTOR                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1605
#define RSSMXV_ATTR_J83B_CODING_INTERLEAVER_MODE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1606
#define RSSMXV_ATTR_J83B_SPECIAL_SETTINGS_ENABLED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1607
#define RSSMXV_ATTR_J83B_SPECIAL_SETTINGS_REED_SOLOMON_ENABLED              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1608
#define RSSMXV_ATTR_IP_CHANNEL_INPUT_IP_ENABLED                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 770
#define RSSMXV_ATTR_IP_CHANNEL_ALIAS                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 771
#define RSSMXV_ATTR_IP_CHANNEL_TYPE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 772
#define RSSMXV_ATTR_IP_CHANNEL_MULTICAST_ADDRESS                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 773
#define RSSMXV_ATTR_IP_CHANNEL_PORT                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 774
#define RSSMXV_ATTR_IP_CHANNEL_IGMPV3_SOURCE_ADDRESS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 775
#define RSSMXV_ATTR_IP_CHANNEL_IGMPV3_PING_SOURCE_ADDRESS                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 776
#define RSSMXV_ATTR_IP_CHANNEL_IGMPV3_PING_RESULT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 777
#define RSSMXV_ATTR_AM_ENABLED                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 778
#define RSSMXV_ATTR_AM_DEPTH                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 779
#define RSSMXV_ATTR_AM_FREQUENCY                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 780
#define RSSMXV_ATTR_AM_PERIOD                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 781
#define RSSMXV_ATTR_AM_SHAPE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 782
#define RSSMXV_ATTR_FM_ENABLED                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 783
#define RSSMXV_ATTR_FM_DEVIATION                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 784
#define RSSMXV_ATTR_FM_FREQUENCY                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 785
#define RSSMXV_ATTR_FM_PERIOD                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 786
#define RSSMXV_ATTR_FM_SHAPE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 787
#define RSSMXV_ATTR_PM_ENABLED                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 788
#define RSSMXV_ATTR_PM_DEVIATION                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 789
#define RSSMXV_ATTR_PM_FREQUENCY                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 790
#define RSSMXV_ATTR_PM_PERIOD                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 791
#define RSSMXV_ATTR_PM_SHAPE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 792
#define RSSMXV_ATTR_PULM_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 793
#define RSSMXV_ATTR_PULM_MODE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 794
#define RSSMXV_ATTR_PULM_DELAY                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 795
#define RSSMXV_ATTR_PULM_DOUBLE_DELAY                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 796
#define RSSMXV_ATTR_PULM_WIDTH                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 797
#define RSSMXV_ATTR_PULM_DOUBLE_WIDTH                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 798
#define RSSMXV_ATTR_PULM_PERIOD                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 799
#define RSSMXV_ATTR_PULM_TRANSITION_TYPE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 800
#define RSSMXV_ATTR_PULM_VIDEO_POLARITY                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 801
#define RSSMXV_ATTR_PROGRESS_MULTI_CODER                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 802
#define RSSMXV_ATTR_PROGRESS_MULTI_CODER_CARRIER                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 803
#define RSSMXV_ATTR_PROGRESS_MULTI_CODER_SEGMENT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 804
#define RSSMXV_ATTR_ARB_ENABLED                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 805
#define RSSMXV_ATTR_ARB_SET_TO_DEFAULT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 806
#define RSSMXV_ATTR_ARB_LOAD_WAVEFORM                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 807
#define RSSMXV_ATTR_ARB_CLOCK_FREQUENCY                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 809
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_CLOCK                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 810
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_SAMPLES                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 811
#define RSSMXV_ATTR_ARB_TEST_SIGNAL_FORM                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 812
#define RSSMXV_ATTR_ARB_TRIGGER_STATUS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 814
#define RSSMXV_ATTR_ARB_EXECUTE_TRIGGER                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 815
#define RSSMXV_ATTR_ARB_TRIGGER_ARM                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 816
#define RSSMXV_ATTR_ARB_TRIGGER_MODE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 817
#define RSSMXV_ATTR_ARB_EXTENDED_TRIGGER_MODE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 818
#define RSSMXV_ATTR_ARB_TRIGGER_SOURCE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 819
#define RSSMXV_ATTR_ARB_TRIGGER_SINGLE_SIGNAL_DURATION_UNIT                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 820
#define RSSMXV_ATTR_ARB_TRIGGER_SINGLE_SEQUENCE_LENGTH                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 821
#define RSSMXV_ATTR_ARB_TRIGGER_PROCESSING_TIME                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1609
#define RSSMXV_ATTR_ARB_EXTERNAL_TRIGGER_DELAY                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 822
#define RSSMXV_ATTR_ARB_EXTERNAL_TRIGGER_INHIBIT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 823
#define RSSMXV_ATTR_ARB_EXTERNAL_TRIGGER_DELAY_UNIT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 824
#define RSSMXV_ATTR_ARB_EXTERNAL_TRIGGER_EVENT_DELAY                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 825
#define RSSMXV_ATTR_ARB_EXTERNAL_TRIGGER_DELAY_SECONDS                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 826
#define RSSMXV_ATTR_ARB_MARKER_MODE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 827
#define RSSMXV_ATTR_ARB_MARKER_PULSE_DIVIDER                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 828
#define RSSMXV_ATTR_ARB_MARKER_DELAY                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 829
#define RSSMXV_ATTR_ARB_MARKER_DELAY_IN_SECONDS                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1610
#define RSSMXV_ATTR_ARB_MARKER_PULSE_FREQUENCY                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 830
#define RSSMXV_ATTR_ARB_MARKER_PATTERN                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 831
#define RSSMXV_ATTR_ARB_MARKER_ON_TIME                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 832
#define RSSMXV_ATTR_ARB_MARKER_OFF_TIME                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 833
#define RSSMXV_ATTR_ARB_CLOCK_SOURCE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 834
#define RSSMXV_ATTR_ARB_CLOCK_SYNCHRONIZATION_OUTPUT_TO_EXTERNAL_TRIGGER    RS_SPECIFIC_PUBLIC_ATTR_BASE + 835
#define RSSMXV_ATTR_ARB_WAVEFORM_CATALOG                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 836
#define RSSMXV_ATTR_ARB_WAVEFORM_CATALOG_LENGTH                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 837
#define RSSMXV_ATTR_ARB_WAVEFORM_DELETE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 840
#define RSSMXV_ATTR_ARB_WAVEFORM_POINTS                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 842
#define RSSMXV_ATTR_ARB_WAVEFORM_FREE_DISK_SPACE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 843
#define RSSMXV_ATTR_ARB_WAVEFORM_TAG                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 845
#define RSSMXV_ATTR_ARB_WAVEFORM_HDD_STREAMING_ENABLED                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1538
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_OUTPUT_FILE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 846
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_SAVE_FILE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 848
#define RSSMXV_ATTR_ARB_CREATE_MULTI_CARRIER                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 849
#define RSSMXV_ATTR_ARB_RESET_MULTI_CARRIER                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 850
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_MODE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 851
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_SPACING                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 852
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_COUNT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 853
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_CREST_FACTOR_MODE                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 854
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_CLIPPING_ENABLED                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 855
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_CLIPPING_TARGET_CREST_FACTOR          RS_SPECIFIC_PUBLIC_ATTR_BASE + 856
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_CLIPPING_FILTER_CUT_OFF_FREQUENCY     RS_SPECIFIC_PUBLIC_ATTR_BASE + 857
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_POWER_REFERENCE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 858
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_SIGNAL_PERIOD_MODE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 859
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_SIGNAL_PERIOD                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 860
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_ENABLED                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 861
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_FREQUENCY                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 862
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_GAIN                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 863
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_PHASE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 864
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_DELAY                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 865
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_DATA_FILE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 866
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_CONFLICT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 867
#define RSSMXV_ATTR_ARB_EXECUTE_MULTI_CARRIER_EDITION                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 868
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_EDIT_DELAY                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 869
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_EDIT_DELAY_STEP                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 870
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_INPUT_IQ_FILE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 871
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_EDIT_PHASE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 872
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_EDIT_PHASE_STEP                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 873
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_EDIT_POWER                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 874
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_EDIT_POWER_STEP                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 875
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_START                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 876
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_STOP                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 877
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_EDIT_ENABLED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 878
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_CATALOG                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 879
#define RSSMXV_ATTR_ARB_LOAD_MULTI_CARRIER_SETTING_FILE                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 880
#define RSSMXV_ATTR_ARB_STORE_MULTI_CARRIER_SETTING_FILE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 882
#define RSSMXV_ATTR_ARB_MULTI_CARRIER_FAST_STORE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 884
#define RSSMXV_ATTR_ARB_TSIGNAL_GENERATE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 885
#define RSSMXV_ATTR_ARB_TSIGNAL_CREATE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 887
#define RSSMXV_ATTR_ARB_TSIGNAL_FREQUENCY                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 888
#define RSSMXV_ATTR_ARB_TSIGNAL_PHASE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 889
#define RSSMXV_ATTR_ARB_TSIGNAL_SAMPLES                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 890
#define RSSMXV_ATTR_ARB_TSIGNAL_RECTANGLE_NAME                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 891
#define RSSMXV_ATTR_ARB_TSIGNAL_RECTANGLE_CREATE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 893
#define RSSMXV_ATTR_ARB_TSIGNAL_RECTANGLE_FREQUENCY                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 894
#define RSSMXV_ATTR_ARB_TSIGNAL_RECTANGLE_AMPLITUDE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 895
#define RSSMXV_ATTR_ARB_TSIGNAL_RECTANGLE_OFFSET_DC                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 896
#define RSSMXV_ATTR_ARB_TSIGNAL_RECTANGLE_SAMPLES                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 897
#define RSSMXV_ATTR_ARB_TSIGNAL_CIQ_NAME                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 898
#define RSSMXV_ATTR_ARB_TSIGNAL_CIQ_CREATE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 900
#define RSSMXV_ATTR_ARB_TSIGNAL_CIQ_I                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 901
#define RSSMXV_ATTR_ARB_TSIGNAL_CIQ_Q                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 902
#define RSSMXV_ATTR_ARB_MULTISEGMENT_OUTPUT_SEGMENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 903
#define RSSMXV_ATTR_ARB_MULTISEGMENT_CURRENT_OUTPUT_SEGMENT                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 904
#define RSSMXV_ATTR_ARB_MULTISEGMENT_LEVEL_MODE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 905
#define RSSMXV_ATTR_ARB_MULTISEGMENT_SEQUENCE_RESTART                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 906
#define RSSMXV_ATTR_ARB_MULTISEGMENT_SEGMENT_RESTART                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 907
#define RSSMXV_ATTR_ARB_MULTISEGMENT_SAVE_FILE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 908
#define RSSMXV_ATTR_ARB_MULTISEGMENT_APPEND_FILE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 910
#define RSSMXV_ATTR_ARB_MULTISEGMENT_NAME                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 912
#define RSSMXV_ATTR_ARB_MULTISEGMENT_EXECUTE_NEXT_SEGMENT                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 913
#define RSSMXV_ATTR_ARB_MULTISEGMENT_NEXT_SEGMENT_SOURCE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 914
#define RSSMXV_ATTR_ARB_MULTISEGMENT_NEXT_SEGMENT_LEVEL_MODE                RS_SPECIFIC_PUBLIC_ATTR_BASE + 915
#define RSSMXV_ATTR_ARB_MULTISEGMENT_SEQUENCING_LIST                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 916
#define RSSMXV_ATTR_ARB_MULTISEGMENT_MARKER_MODE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 919
#define RSSMXV_ATTR_ARB_MULTISEGMENT_CONFIGURATION_FILE                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 920
#define RSSMXV_ATTR_ARB_MULTISEGMENT_CREATE_FROM_CONFIGURATION_FILE         RS_SPECIFIC_PUBLIC_ATTR_BASE + 921
#define RSSMXV_ATTR_ARB_MULTISEGMENT_LOAD_FROM_CONFIGURATION_FILE           RS_SPECIFIC_PUBLIC_ATTR_BASE + 923
#define RSSMXV_ATTR_ARB_MULTISEGMENT_CATALOG                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 925
#define RSSMXV_ATTR_ARB_MULTISEGMENT_SEGMENT_CATALOG                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 926
#define RSSMXV_ATTR_ARB_MULTISEGMENT_DELETE_CONFIGURATION_FILE              RS_SPECIFIC_PUBLIC_ATTR_BASE + 927
#define RSSMXV_ATTR_ARB_MULTISEGMENT_COMMENT                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 929
#define RSSMXV_ATTR_ARB_MULTISEGMENT_CLOCK                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 930
#define RSSMXV_ATTR_ARB_MULTISEGMENT_CLOCK_MODE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 931
#define RSSMXV_ATTR_ARB_CFR_ENABLED                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 932
#define RSSMXV_ATTR_ARB_CFR_ALGORITHM                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 933
#define RSSMXV_ATTR_ARB_CFR_DCF_DELTA                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 934
#define RSSMXV_ATTR_ARB_CFR_OCF                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 935
#define RSSMXV_ATTR_ARB_CFR_FILTERING_MODE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 936
#define RSSMXV_ATTR_ARB_CFR_MAXIMUM_FILTER_ORDER                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 937
#define RSSMXV_ATTR_ARB_CFR_CHANNEL_SPACING                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 938
#define RSSMXV_ATTR_ARB_CFR_STOP_BAND_FREQUENCY                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 939
#define RSSMXV_ATTR_ARB_CFR_MAX_ITERATIONS                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 940
#define RSSMXV_ATTR_ARB_CFR_RCF                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 941
#define RSSMXV_ATTR_ARB_CFR_SIGNAL_BANDWITH                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 942
#define RSSMXV_ATTR_ARB_CFR_PF_REQ                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 943
#define RSSMXV_ATTR_ARB_CFR_MEASURE_ENABLED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 944
#define RSSMXV_ATTR_ARB_CFR_WAVEFORM_CREATE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 945
#define RSSMXV_ATTR_AWGN_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 946
#define RSSMXV_ATTR_AWGN_MODE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 947
#define RSSMXV_ATTR_AWGN_SYSTEM_BANDWIDTH                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 948
#define RSSMXV_ATTR_AWGN_SYSTEM_BANDWIDTH_RATIO                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 949
#define RSSMXV_ATTR_AWGN_SYSTEM_BANDWIDTH_NOISE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 950
#define RSSMXV_ATTR_AWGN_SYSTEM_BANDWIDTH_COUPLING                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1539
#define RSSMXV_ATTR_AWGN_POWERS_FOR_OUTPUT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 951
#define RSSMXV_ATTR_AWGN_RESULTING_CW_FREQUENCY_OFFSET                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 952
#define RSSMXV_ATTR_AWGN_TARGET_CW_FREQUENCY_OFFSET                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 953
#define RSSMXV_ATTR_AWGN_NOISE_LEVEL_VIA                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 954
#define RSSMXV_ATTR_AWGN_CARRIER_TO_NOISE_RATIO                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 955
#define RSSMXV_ATTR_AWGN_REFERENCE_MODE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 956
#define RSSMXV_ATTR_AWGN_BIT_RATE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 957
#define RSSMXV_ATTR_AWGN_CARRIER_POWER                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 958
#define RSSMXV_ATTR_AWGN_CARRIER_AND_NOISE_POWER                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 959
#define RSSMXV_ATTR_AWGN_CARRIER_AND_NOISE_PEP                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 960
#define RSSMXV_ATTR_AWGN_NOISE_POWER_TOTAL                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 961
#define RSSMXV_ATTR_AWGN_NOISE_LEVEL                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 962
#define RSSMXV_ATTR_AWGN_EB_TO_N0                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 963
#define RSSMXV_ATTR_IMP_ENABLED                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 964
#define RSSMXV_ATTR_IMP_QUADRATURE_OFFSET                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 965
#define RSSMXV_ATTR_IMP_QUADRATURE_ANGLE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 966
#define RSSMXV_ATTR_IMP_OPTIMIZATION_MODE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 967
#define RSSMXV_ATTR_IMP_I_OFFSET                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 968
#define RSSMXV_ATTR_IMP_Q_OFFSET                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 969
#define RSSMXV_ATTR_IMP_GAIN_IMBALANCE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 970
#define RSSMXV_ATTR_IMP_CREST_FACTOR                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 971
#define RSSMXV_ATTR_IMP_PEAK_LEVEL                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 972
#define RSSMXV_ATTR_IMP_SIGNAL_LEVEL                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 973
#define RSSMXV_ATTR_IQ_ENABLED                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 974
#define RSSMXV_ATTR_IQ_SWAP_ENABLED                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 975
#define RSSMXV_ATTR_IQ_OUTPUT_DIGITAL_ENABLED                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 976
#define RSSMXV_ATTR_IQ_OUTPUT_DIGITAL_LEVEL_PEP                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 977
#define RSSMXV_ATTR_IQ_OUTPUT_DIGITAL_LEVEL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 978
#define RSSMXV_ATTR_IQ_OUTPUT_DIGITAL_POWER_VARIATION_STEP_MODE             RS_SPECIFIC_PUBLIC_ATTR_BASE + 979
#define RSSMXV_ATTR_IQ_OUTPUT_DIGITAL_POWER_VARIATION_STEP                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 980
#define RSSMXV_ATTR_IQ_OUTPUT_DIGITAL_POWER_ON_STATE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 981
#define RSSMXV_ATTR_IQ_OUTPUT_DIGITAL_CONNECTED_DEVICE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 982
#define RSSMXV_ATTR_IQ_OUTPUT_DIGITAL_SAMPLE_RATE_ENABLED                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 983
#define RSSMXV_ATTR_IQ_OUTPUT_DIGITAL_SAMPLE_RATE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 984
#define RSSMXV_ATTR_IQ_OUTPUT_DIGITAL_SAMPLE_RATE_MAX                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 985
#define RSSMXV_ATTR_IQ_OUTPUT_DIGITAL_SAMPLE_RATE_SUM                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 986
#define RSSMXV_ATTR_IQ_OUTPUT_DIGITAL_INTERFACE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 987
#define RSSMXV_ATTR_IQ_OUTPUT_DIGITAL_POWER_VIA                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 988
#define RSSMXV_ATTR_IQ_OUTPUT_DIGITAL_GROUP_DELAY_COMPENSATION_ENABLED      RS_SPECIFIC_PUBLIC_ATTR_BASE + 989
#define RSSMXV_ATTR_IQ_OUTPUT_DIGITAL_CHANNEL_ENABLED                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 990
#define RSSMXV_ATTR_IQ_OUTPUT_DIGITAL_CHANNEL_NAME                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 991
#define RSSMXV_ATTR_IQ_OUTPUT_DIGITAL_CHANNEL_SAMPLE_RATE                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 992
#define RSSMXV_ATTR_IQ_OUTPUT_DIGITAL_CHANNEL_LEVEL                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1611
#define RSSMXV_ATTR_IQ_OUTPUT_DIGITAL_CHANNEL_LEVEL_PEP                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1612
#define RSSMXV_ATTR_DM_ENABLED                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 993
#define RSSMXV_ATTR_DM_SET_TO_DEFAULT                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 994
#define RSSMXV_ATTR_DM_ACC_TO_STANDARD                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 995
#define RSSMXV_ATTR_DM_SYMBOL_RATE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 996
#define RSSMXV_ATTR_DM_CODING                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 998
#define RSSMXV_ATTR_DM_SWITCHING_ENABLED                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 999
#define RSSMXV_ATTR_DM_SETTINGS_CATALOG                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1000
#define RSSMXV_ATTR_DM_DELETE_SETTINGS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1001
#define RSSMXV_ATTR_DM_RECALL_SETTINGS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1003
#define RSSMXV_ATTR_DM_SAVE_SETTINGS                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1005
#define RSSMXV_ATTR_DM_USER_STANDARDS_CATALOG                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1007
#define RSSMXV_ATTR_DM_DELETE_USER_STANDARDS                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1008
#define RSSMXV_ATTR_DM_LOAD_USER_STANDARDS                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1010
#define RSSMXV_ATTR_DM_STORE_USER_STANDARDS                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1012
#define RSSMXV_ATTR_DM_DATA_SOURCE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1014
#define RSSMXV_ATTR_DM_DATA_PATTERN                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1015
#define RSSMXV_ATTR_DM_PRBS_TYPE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1016
#define RSSMXV_ATTR_DM_CONTROL_LIST_CATALOG                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1017
#define RSSMXV_ATTR_DM_CONTROL_LIST_COPY                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1018
#define RSSMXV_ATTR_DM_CONTROL_LIST_DELETE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1020
#define RSSMXV_ATTR_DM_CONTROL_LIST_SELECT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1022
#define RSSMXV_ATTR_DM_CONTROL_LIST_DATA_WRITE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1023
#define RSSMXV_ATTR_DM_CONTROL_LIST_NUMBER_OF_LINES                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1026
#define RSSMXV_ATTR_DM_CONTROL_LIST_FREE_MEMORY                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1027
#define RSSMXV_ATTR_DM_DATA_LIST_CATALOG                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1028
#define RSSMXV_ATTR_DM_DATA_LIST_COPY                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1029
#define RSSMXV_ATTR_DM_DATA_LIST_DELETE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1031
#define RSSMXV_ATTR_DM_DATA_LIST_SELECT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1033
#define RSSMXV_ATTR_DM_DATA_LIST_DATA_WRITE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1035
#define RSSMXV_ATTR_DM_DATA_LIST_DATA_READ_ALL                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1537
#define RSSMXV_ATTR_DM_DATA_LIST_DATA_APPEND                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1037
#define RSSMXV_ATTR_DM_DATA_LIST_NUMBER_OF_BITS                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1039
#define RSSMXV_ATTR_DM_DATA_LIST_FREE_MEMORY                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1040
#define RSSMXV_ATTR_DM_MAPPING_LIST_CATALOG                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1041
#define RSSMXV_ATTR_DM_MAPPING_LIST_DELETE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1042
#define RSSMXV_ATTR_DM_MAPPING_LIST_FREE_MEMORY                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1044
#define RSSMXV_ATTR_DM_MAPPING_LIST_LENGTH                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1045
#define RSSMXV_ATTR_DM_MAPPING_LIST_SELECT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1046
#define RSSMXV_ATTR_DM_MODULATION_TYPE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1047
#define RSSMXV_ATTR_DM_ASK_DEPTH                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1048
#define RSSMXV_ATTR_DM_FSK_DEVIATION                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1049
#define RSSMXV_ATTR_DM_VARIABLE_FSK_TYPE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1050
#define RSSMXV_ATTR_DM_AQPSK_ANGLE_ALPHA                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1051
#define RSSMXV_ATTR_DM_16APSK_GAMMA                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1052
#define RSSMXV_ATTR_DM_32APSK_GAMMA                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1053
#define RSSMXV_ATTR_DM_VARIABLE_FSK_DEVIATION                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1054
#define RSSMXV_ATTR_DM_FILTER_TYPE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1055
#define RSSMXV_ATTR_DM_FILTER_TYPE_PARAMETER                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1056
#define RSSMXV_ATTR_DM_FILTER_PARAMETER_APCO25                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1057
#define RSSMXV_ATTR_DM_FILTER_PARAMETER_APCO25_LSM_GAUSS                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1058
#define RSSMXV_ATTR_DM_FILTER_PARAMETER_APCO25_LSM_LOWPASS                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1059
#define RSSMXV_ATTR_DM_FILTER_PARAMETER_COSINE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1060
#define RSSMXV_ATTR_DM_FILTER_PARAMETER_COSINE_BANDWIDTH                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1061
#define RSSMXV_ATTR_DM_FILTER_PARAMETER_GAUSS                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1062
#define RSSMXV_ATTR_DM_FILTER_PARAMETER_PURE_GAUSS                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1063
#define RSSMXV_ATTR_DM_FILTER_PARAMETER_ROOT_COSINE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1064
#define RSSMXV_ATTR_DM_FILTER_PARAMETER_SPLIT_PHASE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1065
#define RSSMXV_ATTR_DM_FILTER_PARAMETER_LOW_PASS                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1066
#define RSSMXV_ATTR_DM_FILTER_PARAMETER_LOW_PASS_EVM                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1067
#define RSSMXV_ATTR_DM_FILTER_CATALOG                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1068
#define RSSMXV_ATTR_DM_FILTER_LIST_DELETE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1069
#define RSSMXV_ATTR_DM_FILTER_LIST_SELECT                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1071
#define RSSMXV_ATTR_DM_FILTER_LIST_FREE_MEMORY                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1072
#define RSSMXV_ATTR_DM_FILTER_LIST_LENGTH                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1073
#define RSSMXV_ATTR_DM_POWER_RAMPING_ENABLED                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1074
#define RSSMXV_ATTR_DM_POWER_RAMP_SOURCE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1075
#define RSSMXV_ATTR_DM_POWER_RAMP_FUNCTION                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1076
#define RSSMXV_ATTR_DM_POWER_RAMP_TIME                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1077
#define RSSMXV_ATTR_DM_POWER_RAMP_RISE_DELAY                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1078
#define RSSMXV_ATTR_DM_POWER_RAMP_FALL_DELAY                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1079
#define RSSMXV_ATTR_DM_POWER_RAMP_ATTENUATION                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1080
#define RSSMXV_ATTR_DM_POWER_RAMP_IN_BASEBAND_ONLY                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1081
#define RSSMXV_ATTR_DM_TRIGGER_ARM_EXECUTE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1082
#define RSSMXV_ATTR_DM_TRIGGER_EXECUTE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1083
#define RSSMXV_ATTR_DM_TRIGGER_MODE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1084
#define RSSMXV_ATTR_DM_TRIGGER_SOURCE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1085
#define RSSMXV_ATTR_DM_TRIGGER_SIGNAL_DURATION                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1086
#define RSSMXV_ATTR_DM_TRIGGER_STATUS                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1087
#define RSSMXV_ATTR_DM_EXTERNAL_TRIGGER_DELAY                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1088
#define RSSMXV_ATTR_DM_EXTERNAL_TRIGGER_INHIBIT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1089
#define RSSMXV_ATTR_DM_EXTERNAL_TRIGGER_DELAY_UNIT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1090
#define RSSMXV_ATTR_DM_EXTERNAL_TRIGGER_ACTUAL_DELAY                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1091
#define RSSMXV_ATTR_DM_EXTERNAL_TRIGGER_SPECIFIED_DELAY                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1092
#define RSSMXV_ATTR_DM_MARKER_MODE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1093
#define RSSMXV_ATTR_DM_MARKER_PULSE_DIVIDER                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1094
#define RSSMXV_ATTR_DM_MARKER_PULSE_FREQUENCY                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1095
#define RSSMXV_ATTR_DM_MARKER_PATTERN                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1096
#define RSSMXV_ATTR_DM_MARKER_ON_TIME                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1097
#define RSSMXV_ATTR_DM_MARKER_OFF_TIME                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1098
#define RSSMXV_ATTR_DM_MARKER_DELAY                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1099
#define RSSMXV_ATTR_DM_CLOCK_SOURCE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1100
#define RSSMXV_ATTR_DM_CLOCK_SYNCHRONIZATION_OUTPUT_TO_EXTERNAL_TRIGGER     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1101
#define RSSMXV_ATTR_BBIN_FREQUENCY_OFFSET                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1102
#define RSSMXV_ATTR_BBIN_PHASE_OFFSET                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1103
#define RSSMXV_ATTR_BBIN_SIGNAL_ROUTING                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1104
#define RSSMXV_ATTR_BBIN_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1105
#define RSSMXV_ATTR_BBIN_MODE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1106
#define RSSMXV_ATTR_BBIN_CREST_FACTOR                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1107
#define RSSMXV_ATTR_BBIN_PEAK_LEVEL                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1108
#define RSSMXV_ATTR_BBIN_MEASUREMENT_PERIOD                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1109
#define RSSMXV_ATTR_BBIN_AUTO_LEVEL_SET_ENABLED                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1110
#define RSSMXV_ATTR_BBIN_AUTO_LEVEL_SET                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1111
#define RSSMXV_ATTR_BBIN_RMS_POWER                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1112
#define RSSMXV_ATTR_BBIN_SAMPLE_RATE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1113
#define RSSMXV_ATTR_BBIN_SAMPLE_RATE_SOURCE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1114
#define RSSMXV_ATTR_BBIN_EXTERNALLY_CONNECTED_ID                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1115
#define RSSMXV_ATTR_BBIN_DIGITAL_INTERFACE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1116
#define RSSMXV_ATTR_BBIN_MAX_SAMPLE_RATE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1117
#define RSSMXV_ATTR_BBIN_SUM_SAMPLE_RATE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1118
#define RSSMXV_ATTR_BBIN_ACTIVATE_CHANNEL                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1119
#define RSSMXV_ATTR_BBIN_CHANNEL_NAME                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1121
#define RSSMXV_ATTR_BBIN_CHANNEL_CREST_FACTOR                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1122
#define RSSMXV_ATTR_BBIN_CHANNEL_PEAK_LEVEL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1123
#define RSSMXV_ATTR_BBIN_CHANNEL_SAMPLE_RATE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1124
#define RSSMXV_ATTR_BBIN_CHANNEL_RMS_LEVEL                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1125
#define RSSMXV_ATTR_REFERENCE_OSCILLATOR_PRESET                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1126
#define RSSMXV_ATTR_REFERENCE_OSCILLATOR_SOURCE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1127
#define RSSMXV_ATTR_REFERENCE_OSCILLATOR_ADJUSTMENT_FREQUENCY               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1128
#define RSSMXV_ATTR_REFERENCE_OSCILLATOR_ADJUSTMENT_ACTIVE                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1129
#define RSSMXV_ATTR_REFERENCE_OSCILLATOR_EXTERNAL_FREQUENCY                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1130
#define RSSMXV_ATTR_REFERENCE_OSCILLATOR_OUTPUT_FREQUENCY_MODE              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1131
#define RSSMXV_ATTR_REFERENCE_OSCILLATOR_USER_DEFINED_FREQUENCY             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1132
#define RSSMXV_ATTR_REFERENCE_OSCILLATOR_RF_SWITCH_OFF_ACTIVE               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1133
#define RSSMXV_ATTR_REFERENCE_OSCILLATOR_SYNCHRONIZATION_BANDWIDTH          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1134
#define RSSMXV_ATTR_REFERENCE_OSCILLATOR_MINIMUM_LOCKING_RANGE              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1135
#define RSSMXV_ATTR_REFERENCE_OSCILLATOR_NOMINAL_SYNCHRONIZATION_BANDWIDTH  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1136
#define RSSMXV_ATTR_ALL_RF_OUTPUTS_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1137
#define RSSMXV_ATTR_ALL_MODULATIONS_ENABLED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1138
#define RSSMXV_ATTR_TRIGGER_SOURCE_FOR_ALL_SWEEPS                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1139
#define RSSMXV_ATTR_RESET_ALL_SWEEPS                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1140
#define RSSMXV_ATTR_OUTPUT_ENABLED                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1253
#define RSSMXV_ATTR_OUTPUT_IMPEDANCE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1141
#define RSSMXV_ATTR_RF_FREQUENCY_ON_RECALL                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1142
#define RSSMXV_ATTR_RF_LEVEL_ON_RECALL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1143
#define RSSMXV_ATTR_OVERVOLTAGE_PROTECTION_CLEAR                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1144
#define RSSMXV_ATTR_OVERVOLTAGE_PROTECTION_TRIPPED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1145
#define RSSMXV_ATTR_ALC_SEARCH_ONCE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1146
#define RSSMXV_ATTR_ALC_MODE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1540
#define RSSMXV_ATTR_TRIGGER_ALL_SWEEPS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1147
#define RSSMXV_ATTR_RF_FREQUENCY                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1148
#define RSSMXV_ATTR_RF_FREQUENCY_MODE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1149
#define RSSMXV_ATTR_RF_FREQUENCY_OFFSET                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1150
#define RSSMXV_ATTR_RF_FREQUENCY_MULTIPLIER                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1151
#define RSSMXV_ATTR_RF_VARIATION_STEP_MODE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1152
#define RSSMXV_ATTR_RF_VARIATION_STEP                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1153
#define RSSMXV_ATTR_FREQUENCY_SWEEP_EXECUTE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1154
#define RSSMXV_ATTR_FREQUENCY_SWEEP_MODE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1155
#define RSSMXV_ATTR_FREQUENCY_SWEEP_TRIGGER_SOURCE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1156
#define RSSMXV_ATTR_FREQUENCY_SWEEP_START_FREQUENCY                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1157
#define RSSMXV_ATTR_FREQUENCY_SWEEP_STOP_FREQUENCY                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1158
#define RSSMXV_ATTR_FREQUENCY_SWEEP_CENTER_FREQUENCY                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1159
#define RSSMXV_ATTR_FREQUENCY_SWEEP_SPAN                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1160
#define RSSMXV_ATTR_FREQUENCY_SWEEP_SPACING                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1161
#define RSSMXV_ATTR_FREQUENCY_SWEEP_STEP_LINEAR                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1162
#define RSSMXV_ATTR_FREQUENCY_SWEEP_STEP_LOG                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1163
#define RSSMXV_ATTR_FREQUENCY_SWEEP_DWELL_TIME                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1164
#define RSSMXV_ATTR_FREQUENCY_SWEEP_MANUAL_STEP                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1165
#define RSSMXV_ATTR_FREQUENCY_SWEEP_POINTS                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1166
#define RSSMXV_ATTR_FREQUENCY_SWEEP_SHAPE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1167
#define RSSMXV_ATTR_FREQUENCY_SWEEP_STATUS                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1168
#define RSSMXV_ATTR_RF_DELTA_PHASE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1169
#define RSSMXV_ATTR_RESET_RF_DELTA_PHASE_DISPLAY                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1170
#define RSSMXV_ATTR_RF_AMPLITUDE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1171
#define RSSMXV_ATTR_RF_LEVEL_LOWER_RANGE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1172
#define RSSMXV_ATTR_RF_LEVEL_UPPER_RANGE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1173
#define RSSMXV_ATTR_RF_POWER_MODE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1174
#define RSSMXV_ATTR_RF_LEVEL_PEAK_ENVELOPE_POWER                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1613
#define RSSMXV_ATTR_RF_LEVEL_LIMIT                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1175
#define RSSMXV_ATTR_RF_LEVEL_OFFSET                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1176
#define RSSMXV_ATTR_RF_LEVEL_POWER                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1177
#define RSSMXV_ATTR_RF_LEVEL_SETTING_CHARACTERISTICS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1179
#define RSSMXV_ATTR_RF_LEVEL_EMF_ENABLED                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1180
#define RSSMXV_ATTR_RF_LEVEL_REFERENCE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1622
#define RSSMXV_ATTR_RF_ATTENUATOR_LOWER_RANGE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1183
#define RSSMXV_ATTR_RF_ATTENUATOR_UPPER_RANGE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1184
#define RSSMXV_ATTR_RF_ATTENUATOR_RF_OFF_MODE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1185
#define RSSMXV_ATTR_RF_LEVEL_VARIATION_ACTIVE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1186
#define RSSMXV_ATTR_RF_LEVEL_VARIATION_STEP                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1187
#define RSSMXV_ATTR_LEVEL_SWEEP_EXECUTE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1188
#define RSSMXV_ATTR_LEVEL_SWEEP_MODE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1189
#define RSSMXV_ATTR_LEVEL_SWEEP_TRIGGER_SOURCE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1190
#define RSSMXV_ATTR_LEVEL_SWEEP_TRIGGER_START                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1191
#define RSSMXV_ATTR_LEVEL_SWEEP_START                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1181
#define RSSMXV_ATTR_LEVEL_SWEEP_STOP                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1182
#define RSSMXV_ATTR_LEVEL_SWEEP_STEP                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1192
#define RSSMXV_ATTR_LEVEL_SWEEP_DWELL_TIME                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1193
#define RSSMXV_ATTR_LEVEL_SWEEP_MANUAL_STEP                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1194
#define RSSMXV_ATTR_LEVEL_SWEEP_POINTS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1195
#define RSSMXV_ATTR_LEVEL_SWEEP_SHAPE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1196
#define RSSMXV_ATTR_LEVEL_SWEEP_RETRACE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1197
#define RSSMXV_ATTR_SWEEP_FREQUENCY_RETRACE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1198
#define RSSMXV_ATTR_LEVEL_SWEEP_SPACING_MODE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1199
#define RSSMXV_ATTR_LEVEL_SWEEP_STATUS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1200
#define RSSMXV_ATTR_LEVEL_SWEEP_ATTENUATOR_MODE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1201
#define RSSMXV_ATTR_USER_CORRECTION_DATA_ENABLED                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1202
#define RSSMXV_ATTR_USER_CORRECTION_DATA_VALUE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1203
#define RSSMXV_ATTR_USER_CORRECTION_DATA_CATALOG                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1204
#define RSSMXV_ATTR_USER_CORRECTION_DATA_DELETE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1205
#define RSSMXV_ATTR_USER_CORRECTION_DATA_TABLE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1206
#define RSSMXV_ATTR_USER_CORRECTION_DATA_NUMBER_OF_FREQUENCY_VALUES         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1665
#define RSSMXV_ATTR_USER_CORRECTION_DATA_NUMBER_OF_LEVEL_VALUES             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1210
#define RSSMXV_ATTR_CORRECTION_LIST_MODE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1212
#define RSSMXV_ATTR_CORRECTION_LIST_FILE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1213
#define RSSMXV_ATTR_CORRECTION_EXECUTE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1214
#define RSSMXV_ATTR_CORRECTION_ASCII_FILE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1215
#define RSSMXV_ATTR_CORRECTION_ASCII_EXTENSION                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1216
#define RSSMXV_ATTR_CORRECTION_ASCII_COLUMN_SEPARATOR                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1217
#define RSSMXV_ATTR_CORRECTION_ASCII_COLUMN_DECIMAL_SEPARATOR               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1218
#define RSSMXV_ATTR_CORRECTION_IMPORT_EXPORT_CATALOG                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1219
#define RSSMXV_ATTR_CORRECTION_DATA_WITH_SENSOR                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1220
#define RSSMXV_ATTR_LIST_CATALOG                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1221
#define RSSMXV_ATTR_LIST_DELETE_LIST                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1222
#define RSSMXV_ATTR_LIST_DELETE_ALL                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1224
#define RSSMXV_ATTR_LIST_LEARN                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1225
#define RSSMXV_ATTR_LIST_FREQUENCY_POINTS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1226
#define RSSMXV_ATTR_LIST_POWER_POINTS                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1227
#define RSSMXV_ATTR_LIST_START_INDEX                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1228
#define RSSMXV_ATTR_LIST_STOP_INDEX                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1229
#define RSSMXV_ATTR_LIST_FREE_MEMORY                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1230
#define RSSMXV_ATTR_LIST_RESET_LIST                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1231
#define RSSMXV_ATTR_LIST_TRIGGER_EXECUTE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1232
#define RSSMXV_ATTR_LIST_MODE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1233
#define RSSMXV_ATTR_LIST_RUN_MODE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1234
#define RSSMXV_ATTR_LIST_DWELL_TIME                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1235
#define RSSMXV_ATTR_LIST_DWELL_TIME_MODE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1236
#define RSSMXV_ATTR_LIST_SELECT_LIST                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1237
#define RSSMXV_ATTR_LIST_DWELL_TIME_POINTS                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1239
#define RSSMXV_ATTR_LIST_TRIGGER_SOURCE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1242
#define RSSMXV_ATTR_LIST_INDEX                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1243
#define RSSMXV_ATTR_LIST_CURRENT_STATE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1244
#define RSSMXV_ATTR_LIST_IMPORT_EXPORT_LIST_MODE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1245
#define RSSMXV_ATTR_LIST_IMPORT_EXPORT_LIST_FILE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1246
#define RSSMXV_ATTR_LIST_IMPORT_EXPORT_EXECUTE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1247
#define RSSMXV_ATTR_LIST_IMPORT_EXPORT_ASCII_FILE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1248
#define RSSMXV_ATTR_LIST_IMPORT_EXPORT_ASCII_EXTENSION                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1249
#define RSSMXV_ATTR_LIST_IMPORT_EXPORT_ASCII_COLUMN_SEPARATOR               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1250
#define RSSMXV_ATTR_LIST_IMPORT_EXPORT_ASCII_COLUMN_DECIMAL_SEPARATOR       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1251
#define RSSMXV_ATTR_LIST_IMPORT_EXPORT_CATALOG                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1252
#define RSSMXV_ATTR_POWER_ON_STATE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1254
#define RSSMXV_ATTR_POWER_UNIT                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1255
#define RSSMXV_ATTR_POWER_FREQUENCY                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1256
#define RSSMXV_ATTR_POWER_RECORDER_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1257
#define RSSMXV_ATTR_POWER_OFFSET                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1258
#define RSSMXV_ATTR_POWER_OFFSET_ENABLED                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1259
#define RSSMXV_ATTR_POWER_SOURCE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1260
#define RSSMXV_ATTR_POWER_ZERO                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1261
#define RSSMXV_ATTR_POWER_CONTINUOUS                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1262
#define RSSMXV_ATTR_POWER_USE_SPARAMETERS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1263
#define RSSMXV_ATTR_POWER_SPARAMETER_LIST                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1264
#define RSSMXV_ATTR_POWER_SPARAMETER_SELECT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1265
#define RSSMXV_ATTR_POWER_DISPLAY_PERMANENT_STATE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1614
#define RSSMXV_ATTR_POWER_DISPLAY_PERMANENT_PRIORITY                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1615
#define RSSMXV_ATTR_POWER_FILTER_LENGTH                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1266
#define RSSMXV_ATTR_POWER_FILTER_MODE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1267
#define RSSMXV_ATTR_POWER_FILTER_NS_RATIO                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1268
#define RSSMXV_ATTR_POWER_FILTER_NS_RATIO_TIMEOUT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1269
#define RSSMXV_ATTR_POWER_FILTER_LENGTH_AUTO                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1270
#define RSSMXV_ATTR_POWER_FILTER_SEARCH_ONCE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1271
#define RSSMXV_ATTR_POWER_SENSOR_DEVICE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1272
#define RSSMXV_ATTR_POWER_SENSOR_TYPE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1273
#define RSSMXV_ATTR_POWER_READ_POWER                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1274
#define RSSMXV_ATTR_POWER_SENSOR_SERIAL_NUMBER                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1275
#define RSSMXV_ATTR_POWER_DEFAULT_APERTURE_TIME_ENABLED                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1276
#define RSSMXV_ATTR_POWER_APERTURE_TIME                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1277
#define RSSMXV_ATTR_POWER_SENSOR_CONTROL_SELECT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1623
#define RSSMXV_ATTR_POWER_SENSOR_CONTROL_ENABLED                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1624
#define RSSMXV_ATTR_POWER_SENSOR_CONTROL_MODE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1625
#define RSSMXV_ATTR_POWER_SENSOR_CONTROL_EXECUTE_SINGLE                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1626
#define RSSMXV_ATTR_POWER_SENSOR_CONTROL_CATCH_RANGE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1627
#define RSSMXV_ATTR_POWER_SENSOR_CONTROL_DELAY                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1628
#define RSSMXV_ATTR_POWER_SENSOR_CONTROL_PEAK_ENABLED                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1629
#define RSSMXV_ATTR_POWER_SENSOR_CONTROL_TARGET_LEVEL                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1630
#define RSSMXV_ATTR_POWER_SENSOR_CONTROL_MEASURE_AS_TARGET_LEVEL            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1631
#define RSSMXV_ATTR_POWER_SENSOR_MAPPING_SCAN_ENABLED                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1278
#define RSSMXV_ATTR_POWER_SENSOR_MAPPING_LIST                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1279
#define RSSMXV_ATTR_POWER_SENSOR_REMOVES_ALL_LAN_SENSORS                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1541
#define RSSMXV_ATTR_POWER_SENSOR_REMOVES_ALL_USB_SENSORS                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1542
#define RSSMXV_ATTR_POWER_SENSOR_REMOVES_ALL_SENSORS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1543
#define RSSMXV_ATTR_POWER_SENSOR_SCAN_LAN                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1544
#define RSSMXV_ATTR_CALIBRATION_ALL                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1282
#define RSSMXV_ATTR_REMOTE_CONTROL_COMMAND_SET                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1283
#define RSSMXV_ATTR_KEYBOARD_LAYOUT                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1284
#define RSSMXV_ATTR_TS_PLAYER_SELECT_FILE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1285
#define RSSMXV_ATTR_TS_PLAYER_SELECTED_FILE_SIZE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1286
#define RSSMXV_ATTR_TS_PLAYER_SELECTED_FILE_LOOP_TIME                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1287
#define RSSMXV_ATTR_TS_PLAYER_START_TIME                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1288
#define RSSMXV_ATTR_TS_PLAYER_STOP_TIME                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1289
#define RSSMXV_ATTR_TS_PLAYER_POSITION                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1290
#define RSSMXV_ATTR_TS_PLAYER_RESET_WINDOW                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1291
#define RSSMXV_ATTR_TS_PLAYER_PLAY                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1292
#define RSSMXV_ATTR_TS_PLAYER_PAUSE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1293
#define RSSMXV_ATTR_TS_PLAYER_STOP                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1294
#define RSSMXV_ATTR_TS_PLAYER_RESET                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1616
#define RSSMXV_ATTR_TS_PLAYER_DATA_RATE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1295
#define RSSMXV_ATTR_TS_PLAYER_ORIGINAL_DATA_RATE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1296
#define RSSMXV_ATTR_TS_PLAYER_PACKET_LENGTH                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1297
#define RSSMXV_ATTR_TS_PLAYER_NULL_PACKET_STUFFING                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1298
#define RSSMXV_ATTR_TS_PLAYER_STOP_DATA                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1299
#define RSSMXV_ATTR_TS_PLAYER_TEST_PACKET                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1300
#define RSSMXV_ATTR_TS_PLAYER_PID_TEST_PACKET                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1301
#define RSSMXV_ATTR_TS_PLAYER_PID                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1302
#define RSSMXV_ATTR_TS_PLAYER_PAYLOAD_TEST_STUFF                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1303
#define RSSMXV_ATTR_TS_PLAYER_PRBS                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1304
#define RSSMXV_ATTR_TS_PLAYER_CONTINUITY_COUNTER                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1305
#define RSSMXV_ATTR_TS_PLAYER_PCR_DTS_PTS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1306
#define RSSMXV_ATTR_TS_PLAYER_TDT_TOT                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1307
#define RSSMXV_ATTR_IP_DATA_NETWORK_CONNECTED                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1308
#define RSSMXV_ATTR_IP_DATA_NETWORK_RESTART                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1309
#define RSSMXV_ATTR_IP_DATA_NETWORK_HOSTNAME                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1310
#define RSSMXV_ATTR_IP_DATA_NETWORK_ADDRESS_MODE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1311
#define RSSMXV_ATTR_IP_DATA_NETWORK_IP_ADDRESS                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1312
#define RSSMXV_ATTR_IP_DATA_NETWORK_SUBNET_MASK                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1313
#define RSSMXV_ATTR_IP_DATA_NETWORK_PROTOCOL                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1314
#define RSSMXV_ATTR_IP_DATA_NETWORK_MAC_ADDRESS                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1315
#define RSSMXV_ATTR_DIAGNOSTIC_OPERATING_HOUR                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1316
#define RSSMXV_ATTR_DIAGNOSTIC_POWER_ON_COUNT                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1317
#define RSSMXV_ATTR_DIAGNOSTIC_AVAILABLE_ASSEMBLIES_CATALOG                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1318
#define RSSMXV_ATTR_DIAGNOSTIC_TEST_POINTS_CATALOG                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1320
#define RSSMXV_ATTR_HUMS_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1632
#define RSSMXV_ATTR_HUMS_DATA_FORMAT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1633
#define RSSMXV_ATTR_HUMS_DELETE_ALL_DATA                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1634
#define RSSMXV_ATTR_HUMS_EXPORT_HISTORY                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1635
#define RSSMXV_ATTR_HUMS_EXPORT_HISTORY_TO_FILE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1636
#define RSSMXV_ATTR_HUMS_DELETE_HISTORY                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1637
#define RSSMXV_ATTR_HUMS_DEVICE_TAGS_CATALOG                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1638
#define RSSMXV_ATTR_HUMS_DEVICE_TAGS_DELETE_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1639
#define RSSMXV_ATTR_HUMS_DEVICE_TAG_DELETE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1641
#define RSSMXV_ATTR_HUMS_SNMP_REST_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1642
#define RSSMXV_ATTR_HUMS_SNMP_VERSION                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1643
#define RSSMXV_ATTR_HUMS_SNMP_USER_CATALOG                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1644
#define RSSMXV_ATTR_HUMS_SNMP_USER_DELETE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1646
#define RSSMXV_ATTR_HUMS_SNMP_USER_DELETE_ALL                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1647
#define RSSMXV_ATTR_HUMS_SNMP_READ_ONLY_ACCESS                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1648
#define RSSMXV_ATTR_HUMS_SNMP_READ_WRITE_ACCESS                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1649
#define RSSMXV_ATTR_HUMS_SNMP_CONTACT                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1650
#define RSSMXV_ATTR_HUMS_SNMP_LOCATION                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1651
#define RSSMXV_ATTR_CALIBRATION_CONTINUE_ON_ERROR                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1322
#define RSSMXV_ATTR_CALIBRATION_LAST_FULL_ADJUSTMENT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1323
#define RSSMXV_ATTR_CALIBRATION_LAST_FULL_ADJUSTMENT_TIME                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1324
#define RSSMXV_ATTR_CALIBRATION_TEMPERATURE_OFFSET_TO_LAST_FULL_ADJUSTMENT  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1325
#define RSSMXV_ATTR_CALIBRATION_INTERNAL_ADJUSTMENT_STATE                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1326
#define RSSMXV_ATTR_LAST_FACTORY_CALIBRATION                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1327
#define RSSMXV_ATTR_CALIBRATION_REFERENCE_FREQUENCY                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1617
#define RSSMXV_ATTR_CALIBRATION_SHUTDOWN_AFTER_ADJUSTMENT_ENABLED           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1652
#define RSSMXV_ATTR_CALIBRATION_WARM_UP_TIME                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1653
#define RSSMXV_ATTR_CALIBRATION_ADJUST_ALL_DELAYED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1654
#define RSSMXV_ATTR_HARD_COPY_EXECUTE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1329
#define RSSMXV_ATTR_HARD_COPY_LANGUAGE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1330
#define RSSMXV_ATTR_HARD_COPY_IMAGE_FORMAT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1331
#define RSSMXV_ATTR_HARD_COPY_REGION                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1332
#define RSSMXV_ATTR_HARD_COPY_FILE_NAME                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1333
#define RSSMXV_ATTR_HARD_COPY_AUTO_FILE_DIRECTORY                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1334
#define RSSMXV_ATTR_CLEAR_HARD_COPY_FILE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1335
#define RSSMXV_ATTR_HARD_COPY_AUTO_FILE_NAME                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1336
#define RSSMXV_ATTR_HARD_COPY_AUTO_NAMING_ENABLED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1337
#define RSSMXV_ATTR_HARD_COPY_DAY_TO_FILE_NAME                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1338
#define RSSMXV_ATTR_HARD_COPY_MONTH_TO_FILE_NAME                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1339
#define RSSMXV_ATTR_HARD_COPY_YEAR_TO_FILE_NAME                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1340
#define RSSMXV_ATTR_HARD_COPY_CURRENT_NUMBER                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1341
#define RSSMXV_ATTR_HARD_COPY_AUTO_PREFIX_ENABLED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1342
#define RSSMXV_ATTR_HARD_COPY_FILE_NAME_PREFIX                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1343
#define RSSMXV_ATTR_HARD_COPY_GENERATED_AUTO_NAME                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1344
#define RSSMXV_ATTR_DISPLAY_UPDATE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1345
#define RSSMXV_ATTR_DISPLAY_ANNOTATION_AMPLITUDE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1346
#define RSSMXV_ATTR_DISPLAY_ANNOTATION_FREQUENCY                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1347
#define RSSMXV_ATTR_DISPLAY_ANNOTATION_ALL                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1348
#define RSSMXV_ATTR_DISPLAY_DIALOG_ID                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1349
#define RSSMXV_ATTR_DISPLAY_DIALOG_OPEN                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1350
#define RSSMXV_ATTR_DISLPAY_DIALOG_CLOSE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1351
#define RSSMXV_ATTR_DISPLAY_DIALOG_CLOSE_ALL                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1352
#define RSSMXV_ATTR_DISPLAY_BRIGHTNESS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1655
#define RSSMXV_ATTR_DISPLAY_BUTTON_BRIGHTNESS                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1656
#define RSSMXV_ATTR_DISPLAY_POWER_SAVE_ENABLED                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1353
#define RSSMXV_ATTR_DISPLAY_POWER_SAVE_HOLDOFF                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1354
#define RSSMXV_ATTR_MMEMORY_CHANGE_DIRECTORY                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1357
#define RSSMXV_ATTR_MMEMORY_DELETE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1364
#define RSSMXV_ATTR_MMEMORY_MAKE_DIRECTORY                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1367
#define RSSMXV_ATTR_MMEMORY_DRIVE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1370
#define RSSMXV_ATTR_MMEMORY_REMOVE_DIRECTORY                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1371
#define RSSMXV_ATTR_MMEMORY_USER_PATH                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1374
#define RSSMXV_ATTR_MMEMORY_USED_AND_AVAILABLE_MEMORY                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1375
#define RSSMXV_ATTR_SYSTEM_REBOOT                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1376
#define RSSMXV_ATTR_SYSTEM_RESTART                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1377
#define RSSMXV_ATTR_SYSTEM_SHUTDOWN                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1378
#define RSSMXV_ATTR_SYSTEM_FACTORY_PRESET                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1379
#define RSSMXV_ATTR_SYSTEM_ERROR                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1380
#define RSSMXV_ATTR_SYSTEM_STATIC_ERRORS                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1381
#define RSSMXV_ATTR_SYSTEM_ERROR_QUEUE_ALL                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1382
#define RSSMXV_ATTR_SYSTEM_ERROR_QUEUE_CODE_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1383
#define RSSMXV_ATTR_SYSTEM_ERROR_QUEUE_CODE_NEXT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1384
#define RSSMXV_ATTR_SYSTEM_ERROR_QUEUE_COUNT                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1385
#define RSSMXV_ATTR_SYSTEM_ERROR_GLOBAL                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1386
#define RSSMXV_ATTR_SYSTEM_USB_RESOURCE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1387
#define RSSMXV_ATTR_SYSTEM_VISA_RESOURCE_STRING                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1388
#define RSSMXV_ATTR_SYSTEM_OPTION_USER_DEFINED                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1389
#define RSSMXV_ATTR_SYSTEM_KEYBOARD_LOCK                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1390
#define RSSMXV_ATTR_SYSTEM_USER_INTERFACE_LOCK                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1391
#define RSSMXV_ATTR_SYSTEM_DISPLAY_LOCK                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1392
#define RSSMXV_ATTR_SYSTEM_VERSION                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1394
#define RSSMXV_ATTR_SYSTEM_BIOS_VERSION                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1395
#define RSSMXV_ATTR_SYSTEM_STARTUP_COMPLETE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1396
#define RSSMXV_ATTR_SYSTEM_ALL_DATA_SHEETS                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1397
#define RSSMXV_ATTR_SYSTEM_DATA_SHEET_FACTORY_VERSION                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1398
#define RSSMXV_ATTR_SYSTEM_DATA_SHEET_VERSION                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1399
#define RSSMXV_ATTR_SYSTEM_DATA_SHEET_IDS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1400
#define RSSMXV_ATTR_SYSTEM_SCPI_INFORMATION                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1403
#define RSSMXV_ATTR_SYSTEM_TIME_ZONE_CATALOG                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1405
#define RSSMXV_ATTR_SYSTEM_TIME_ZONE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1406
#define RSSMXV_ATTR_SYSTEM_TIME_PROTOCOL                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1657
#define RSSMXV_ATTR_SYSTEM_DATE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1407
#define RSSMXV_ATTR_SYSTEM_UPTIME                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1408
#define RSSMXV_ATTR_SYSTEM_CLEAR_HISTORY                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1409
#define RSSMXV_ATTR_SYSTEM_WAIT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1410
#define RSSMXV_ATTR_SYSTEM_DEVICE_FOOTPRINT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1411
#define RSSMXV_ATTR_SYSTEM_OPERATING_SYSTEM                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1412
#define RSSMXV_ATTR_SYSTEM_SCPI_RECORDING_DATA                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1413
#define RSSMXV_ATTR_SYSTEM_SAVE_INSTRUMENT_SETTINGS                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1414
#define RSSMXV_ATTR_SYSTEM_LOAD_INSTRUMENT_SETTINGS                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1416
#define RSSMXV_ATTR_SYSTEM_HELP_EXPORT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1418
#define RSSMXV_ATTR_SYSTEM_IDENTIFICATION                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1419
#define RSSMXV_ATTR_SYSTEM_IDENTIFICATION_PRESET                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1420
#define RSSMXV_ATTR_SYSTEM_IDN_RESPONSE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1421
#define RSSMXV_ATTR_SYSTEM_LOCK_OWNER                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1422
#define RSSMXV_ATTR_SYSTEM_LOCK_RELEASE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1423
#define RSSMXV_ATTR_SYSTEM_LOCK_EXKLUSIVE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1424
#define RSSMXV_ATTR_SYSTEM_OLDEST_INFORMATION_MESSAGE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1425
#define RSSMXV_ATTR_SYSTEM_NETWORK_DOMAIN                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1426
#define RSSMXV_ATTR_SYSTEM_NETWORK_GATEWAY                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1427
#define RSSMXV_ATTR_SYSTEM_NETWORK_RESOURCE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1428
#define RSSMXV_ATTR_SYSTEM_NETWORK_SUBNET_MASK                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1429
#define RSSMXV_ATTR_SYSTEM_NETWORK_WORKGROUP                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1430
#define RSSMXV_ATTR_SYSTEM_NETWORK_MAC_ADDRESS                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1431
#define RSSMXV_ATTR_SYSTEM_NETWORK_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1432
#define RSSMXV_ATTR_SYSTEM_NETWORK_HOSTNAME                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1433
#define RSSMXV_ATTR_SYSTEM_NETWORK_RESTART                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1434
#define RSSMXV_ATTR_SYSTEM_NETWORK_DNS_SERVER                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1435
#define RSSMXV_ATTR_SYSTEM_NETWORK_IP_ADDRESS                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1436
#define RSSMXV_ATTR_SYSTEM_NETWORK_IP_ADDRESS_SUBNET_MASK                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1437
#define RSSMXV_ATTR_SYSTEM_NETWORK_IP_ADDRESS_GATEWAY                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1438
#define RSSMXV_ATTR_SYSTEM_NETWORK_IP_ADDRESS_MODE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1439
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_FREQUENCY_OFFSET                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1440
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_PHASE_OFFSET                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1441
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_EXTERNAL_CLEAN                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1443
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_EXTERNAL_CONNECT_ALL               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1444
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_EXTERNAL_DISCONNECT_ALL            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1445
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_EXTERNAL_DISPLAY_MODE              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1446
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_EXTERNAL_REMOTE_SCAN               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1447
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_EXTERNAL_REMOTE_SCAN_STATE         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1448
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_EXTERNAL_REMOTE_LIST               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1449
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_EXTERNAL_REMOVE_UNUSED             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1450
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_EXTERNAL_AUTO_CONNECT              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1451
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_PRESET_BEHAVIOR_ENABLED            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1453
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_EXTERNAL_CATALOG                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1454
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_HSD_REMOTE_CONNECTION_STATE        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1658
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_HSD_DISCONNECT                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1457
#define RSSMXV_ATTR_SYSTEM_CONFIG_HSD_DIRECTION                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1659
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_HSD_REMOTE_SEND_COMMAND            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1459
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_HSD_RF_COUPLING                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1460
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_HSD_REMOTE_CONNECT                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1464
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_HSD_REMOTE_INIT_FILE               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1466
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_HSD_RF_FREQ                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1660
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_HSD_RF_FREQUENCY_OFFSET            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1661
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_HSD_RF_POWER                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1471
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_CONNECTOR_HSD_NAME                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1662
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_HSD_REMOTE_INFO                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1458
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_HSD_RF_POWER_OFFSET                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1461
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_HSD_IQ_CONNECTION_ENABLED          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1463
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_HSD_REMOTE_DETECT                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1465
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_HSD_RF_ENABLED                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1468
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_MAPPING_RF_MODE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1472
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_MAPPING_BBMM_CHANNEL_ENABLED       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1473
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_MULTI_INSTRUMENT_MODE              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1619
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_MULTI_INSTRUMENT_TRIGGER           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1620
#define RSSMXV_ATTR_SYSTEM_CONFIGURATION_MULTI_INSTRUMENT_TRIGGER_SYNC      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1621
#define RSSMXV_ATTR_STATUS_PRESET                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1476
#define RSSMXV_ATTR_UNIT_ANGLE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1477
#define RSSMXV_ATTR_UNIT_POWER                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1478
#define RSSMXV_ATTR_GRAPHICS_MODE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1479
#define RSSMXV_ATTR_GRAPHICS_TRIGGER_SOURCE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1480
#define RSSMXV_ATTR_GRAPHICS_SAMPLE_RATE_MODE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1481
#define RSSMXV_ATTR_GRAPHICS_SAMPLE_RATE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1482
#define RSSMXV_ATTR_GRAPHICS_ADD                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1483
#define RSSMXV_ATTR_GRAPHICS_CLOSE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1484
#define RSSMXV_ATTR_GRAPHICS_SOURCE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1485
#define RSSMXV_ATTR_GRAPHICS_FFT_SCALE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1486
#define RSSMXV_ATTR_GRAPHICS_FFT_SIZE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1487
#define RSSMXV_ATTR_NTP_SERVER_ENABLED                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1489
#define RSSMXV_ATTR_NTP_SERVER_HOSTNAME                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1490
#define RSSMXV_ATTR_TEST_START                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1546
#define RSSMXV_ATTR_TEST_RESULT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1547
#define RSSMXV_ATTR_PRESET_DEVICE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1491
#define RSSMXV_ATTR_ID_QUERY_RESPONSE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1507
#define RSSMXV_ATTR_LOGGING                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1508
#define RSSMXV_ATTR_PROCESS_ALL_PREVIOUS_COMMANDS                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1512
#define RSSMXV_ATTR_QUEUE_NEXT                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1517
#define RSSMXV_ATTR_TRIGGER_INPUT_SLOPE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1523
#define RSSMXV_ATTR_CONTROL_INPUT_TRIGGER_IMPEDANCE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1524
#define RSSMXV_ATTR_CONTROL_INPUT_TRIGGER_LEVEL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1525
#define RSSMXV_ATTR_CONTROL_INPUT_TRIGGER_SLOPE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1526
#define RSSMXV_ATTR_CONTROL_INPUT_CLOCK_IMPEDANCE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1527
#define RSSMXV_ATTR_CONTROL_INPUT_CLOCK_LEVEL                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1528
#define RSSMXV_ATTR_GLOBAL_CONTROL_INPUT_CLOCK_SLOPE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1529
#define RSSMXV_ATTR_OUTPUT_CONNECTOR_USER_SIGNAL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1530
#define RSSMXV_ATTR_OUTPUT_CONNECTOR_DIRECTION                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1531
#define RSSMXV_ATTR_INPUT_CONNECTOR_DIRECTION                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1532
#define RSSMXV_ATTR_INPUT_CONNECTOR_SIGNAL                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1533
#define RSSMXV_ATTR_TRIGGER_RF_FREQUENCY_SWEEP                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1534
#define RSSMXV_ATTR_INIT_SW_TRIGGER                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1535

// RsSMxV_rngReferenceOscillatorSource
#define RSSMXV_VAL_ROSC_SOURCE_INTERNAL                                     0
#define RSSMXV_VAL_ROSC_SOURCE_EXTERNAL                                     1

// RsSMxV_rngReferenceOscillatorExternalFrequency
#define RSSMXV_VAL_ROSC_EXTERNAL_FREQUENCY_5MHZ                             0
#define RSSMXV_VAL_ROSC_EXTERNAL_FREQUENCY_10MHZ                            1
#define RSSMXV_VAL_ROSC_EXTERNAL_FREQUENCY_13MHZ                            2

// RsSMxV_rngDMDigitalStandard
#define RSSMXV_VAL_DM_DIGITAL_STANDARD_BLUETOOTH                            0
#define RSSMXV_VAL_DM_DIGITAL_STANDARD_COFORWARD                            1
#define RSSMXV_VAL_DM_DIGITAL_STANDARD_COREVERSE                            2
#define RSSMXV_VAL_DM_DIGITAL_STANDARD_DECT                                 3
#define RSSMXV_VAL_DM_DIGITAL_STANDARD_ETC                                  4
#define RSSMXV_VAL_DM_DIGITAL_STANDARD_GSM                                  5
#define RSSMXV_VAL_DM_DIGITAL_STANDARD_GSMEDGE                              6
#define RSSMXV_VAL_DM_DIGITAL_STANDARD_NADC                                 7
#define RSSMXV_VAL_DM_DIGITAL_STANDARD_PDC                                  8
#define RSSMXV_VAL_DM_DIGITAL_STANDARD_PHS                                  9
#define RSSMXV_VAL_DM_DIGITAL_STANDARD_TDSCDMA                              10
#define RSSMXV_VAL_DM_DIGITAL_STANDARD_TETRA                                11
#define RSSMXV_VAL_DM_DIGITAL_STANDARD_TFTS                                 12
#define RSSMXV_VAL_DM_DIGITAL_STANDARD_W3GPP                                13
#define RSSMXV_VAL_DM_DIGITAL_STANDARD_WORLDSPACE                           14
#define RSSMXV_VAL_DM_DIGITAL_STANDARD_APCO_PHASE1_C4FM                     15
#define RSSMXV_VAL_DM_DIGITAL_STANDARD_APCO_PHASE1_CQPSK                    16
#define RSSMXV_VAL_DM_DIGITAL_STANDARD_APCO_PHASE2_H_CPM                    17
#define RSSMXV_VAL_DM_DIGITAL_STANDARD_APCO_PHASE2_H_DQPSK                  18
#define RSSMXV_VAL_DM_DIGITAL_STANDARD_APCO_PHASE2_H_D8PSK_WIDE             19
#define RSSMXV_VAL_DM_DIGITAL_STANDARD_APCO_PHASE2_H_D8PSK_NARROW           20
#define RSSMXV_VAL_DM_DIGITAL_STANDARD_APCO_PHASE1_LSM                      21
#define RSSMXV_VAL_DM_DIGITAL_STANDARD_APCO_PHASE1_WCQPSK                   22
#define RSSMXV_VAL_DM_DIGITAL_STANDARD_CWB_PSK                              23
#define RSSMXV_VAL_DM_DIGITAL_STANDARD_SOQPSKTG                             24

// RsSMxV_rngTrgTriggerMode
#define RSSMXV_VAL_TRIGGER_MODE_AUTO                                        0
#define RSSMXV_VAL_TRIGGER_MODE_RETRIGGER                                   1
#define RSSMXV_VAL_TRIGGER_MODE_AAUTO                                       2
#define RSSMXV_VAL_TRIGGER_MODE_ARETRIGGER                                  3
#define RSSMXV_VAL_TRIGGER_MODE_SINGLE                                      4

// RsSMxV_rngDMCoding
#define RSSMXV_VAL_DM_CODING_OFF                                            0
#define RSSMXV_VAL_DM_CODING_APCO25                                         1
#define RSSMXV_VAL_DM_CODING_APCO25FSK                                      2
#define RSSMXV_VAL_DM_CODING_CDMA2000                                       3
#define RSSMXV_VAL_DM_CODING_DIFF                                           4
#define RSSMXV_VAL_DM_CODING_DPHS                                           5
#define RSSMXV_VAL_DM_CODING_DGRAY                                          6
#define RSSMXV_VAL_DM_CODING_EDGE                                           7
#define RSSMXV_VAL_DM_CODING_GRAY                                           8
#define RSSMXV_VAL_DM_CODING_GSM                                            9
#define RSSMXV_VAL_DM_CODING_ICO                                            10
#define RSSMXV_VAL_DM_CODING_NADC                                           11
#define RSSMXV_VAL_DM_CODING_PDC                                            12
#define RSSMXV_VAL_DM_CODING_PHS                                            13
#define RSSMXV_VAL_DM_CODING_TETRA                                          14
#define RSSMXV_VAL_DM_CODING_PWT                                            15
#define RSSMXV_VAL_DM_CODING_TFTS                                           16
#define RSSMXV_VAL_DM_CODING_INMARSAT                                       17
#define RSSMXV_VAL_DM_CODING_VDL                                            18
#define RSSMXV_VAL_DM_CODING_WCDMA                                          19
#define RSSMXV_VAL_DM_CODING_APCO258PSK                                     20

// RsSMxV_rngDMDataSource
#define RSSMXV_VAL_DM_DATA_SOURCE_ZERO                                      0
#define RSSMXV_VAL_DM_DATA_SOURCE_ONE                                       1
#define RSSMXV_VAL_DM_DATA_SOURCE_PRBS                                      2
#define RSSMXV_VAL_DM_DATA_SOURCE_PATTERN                                   3
#define RSSMXV_VAL_DM_DATA_SOURCE_DLIST                                     4

// RsSMxV_rngDMPRBSType
#define RSSMXV_VAL_DM_PRBS_LENGTH_9                                         0
#define RSSMXV_VAL_DM_PRBS_LENGTH_11                                        1
#define RSSMXV_VAL_DM_PRBS_LENGTH_15                                        2
#define RSSMXV_VAL_DM_PRBS_LENGTH_16                                        3
#define RSSMXV_VAL_DM_PRBS_LENGTH_20                                        4
#define RSSMXV_VAL_DM_PRBS_LENGTH_21                                        5
#define RSSMXV_VAL_DM_PRBS_LENGTH_23                                        6

// RsSMxV_rngDMModulationType
#define RSSMXV_VAL_DM_FORMAT_ASK                                            0
#define RSSMXV_VAL_DM_FORMAT_BPSK                                           1
#define RSSMXV_VAL_DM_FORMAT_QPSK                                           2
#define RSSMXV_VAL_DM_FORMAT_QPSK45                                         3
#define RSSMXV_VAL_DM_FORMAT_OQPSK                                          4
#define RSSMXV_VAL_DM_FORMAT_P4QPSK                                         5
#define RSSMXV_VAL_DM_FORMAT_P2DBPSK                                        6
#define RSSMXV_VAL_DM_FORMAT_P4DQPSK                                        7
#define RSSMXV_VAL_DM_FORMAT_P8D8PSK                                        8
#define RSSMXV_VAL_DM_FORMAT_PSK8                                           9
#define RSSMXV_VAL_DM_FORMAT_P8EDGE                                         10
#define RSSMXV_VAL_DM_FORMAT_QAM16                                          11
#define RSSMXV_VAL_DM_FORMAT_QAM32                                          12
#define RSSMXV_VAL_DM_FORMAT_QAM64                                          13
#define RSSMXV_VAL_DM_FORMAT_QAM256                                         14
#define RSSMXV_VAL_DM_FORMAT_QAM1024                                        15
#define RSSMXV_VAL_DM_FORMAT_MSK                                            16
#define RSSMXV_VAL_DM_FORMAT_FSK2                                           17
#define RSSMXV_VAL_DM_FORMAT_FSK4                                           18
#define RSSMXV_VAL_DM_FORMAT_USER                                           19
#define RSSMXV_VAL_DM_FORMAT_FSKVAR                                         20
#define RSSMXV_VAL_DM_FORMAT_QEDGE                                          21
#define RSSMXV_VAL_DM_FORMAT_AQPS                                           22
#define RSSMXV_VAL_DM_FORMAT_QAM128                                         23
#define RSSMXV_VAL_DM_FORMAT_QAM16ED                                        24
#define RSSMXV_VAL_DM_FORMAT_QAM32ED                                        25
#define RSSMXV_VAL_DM_FORMAT_QAM4096                                        26
#define RSSMXV_VAL_DM_FORMAT_APSK16                                         27
#define RSSMXV_VAL_DM_FORMAT_APSK32                                         28
#define RSSMXV_VAL_DM_FORMAT_FSK32                                          29
#define RSSMXV_VAL_DM_FORMAT_FSK64                                          30
#define RSSMXV_VAL_DM_FORMAT_FSK8                                           31
#define RSSMXV_VAL_DM_FORMAT_FSK16                                          32
#define RSSMXV_VAL_DM_FORMAT_QAM512                                         33
#define RSSMXV_VAL_DM_FORMAT_QAM2048                                        34

// RsSMxV_rngDMVariableFSKType
#define RSSMXV_VAL_FSK_TYPE_FSK4                                            0
#define RSSMXV_VAL_FSK_TYPE_FSK8                                            1
#define RSSMXV_VAL_FSK_TYPE_FSK16                                           2

// RsSMxV_rngDMFilterType
#define RSSMXV_VAL_DM_FILTER_TYPE_RCOSINE                                   0
#define RSSMXV_VAL_DM_FILTER_TYPE_COSINE                                    1
#define RSSMXV_VAL_DM_FILTER_TYPE_GAUSS                                     2
#define RSSMXV_VAL_DM_FILTER_TYPE_LGAUSS                                    3
#define RSSMXV_VAL_DM_FILTER_TYPE_CONE                                      4
#define RSSMXV_VAL_DM_FILTER_TYPE_COF705                                    5
#define RSSMXV_VAL_DM_FILTER_TYPE_COEQ                                      6
#define RSSMXV_VAL_DM_FILTER_TYPE_COF                                       7
#define RSSMXV_VAL_DM_FILTER_TYPE_C2K3X                                     8
#define RSSMXV_VAL_DM_FILTER_TYPE_APCO25                                    9
#define RSSMXV_VAL_DM_FILTER_TYPE_RECTANGLE                                 10
#define RSSMXV_VAL_DM_FILTER_TYPE_SPHASE                                    11
#define RSSMXV_VAL_DM_FILTER_TYPE_LPASS                                     12
#define RSSMXV_VAL_DM_FILTER_TYPE_USER                                      13
#define RSSMXV_VAL_DM_FILTER_TYPE_PGAUSS                                    14
#define RSSMXV_VAL_DM_FILTER_TYPE_DIRAC                                     15
#define RSSMXV_VAL_DM_FILTER_TYPE_ENPS                                      16
#define RSSMXV_VAL_DM_FILTER_TYPE_EWPS                                      17
#define RSSMXV_VAL_DM_FILTER_TYPE_LPASSEVM                                  18
#define RSSMXV_VAL_DM_FILTER_TYPE_LTEF                                      19
#define RSSMXV_VAL_DM_FILTER_TYPE_APCO25H                                   20
#define RSSMXV_VAL_DM_FILTER_TYPE_APCO25L                                   21
#define RSSMXV_VAL_DM_FILTER_TYPE_HRP                                       22
#define RSSMXV_VAL_DM_FILTER_TYPE_SOQPSK                                    23

// RsSMxV_rngDMPowerRampSource
#define RSSMXV_VAL_DM_POWER_RAMP_SOURCE_INTERNAL                            0
#define RSSMXV_VAL_DM_POWER_RAMP_SOURCE_EXTERNAL                            1

// RsSMxV_rngDMPowerRampFunction
#define RSSMXV_VAL_DM_POWER_RAMP_SHAPE_LINEAR                               0
#define RSSMXV_VAL_DM_POWER_RAMP_SHAPE_COSINE                               1

// RsSMxV_rngDMMarkerMode
#define RSSMXV_VAL_DM_MARKER_MODE_CLIST                                     0
#define RSSMXV_VAL_DM_MARKER_MODE_PULSE                                     1
#define RSSMXV_VAL_DM_MARKER_MODE_PATTERN                                   2
#define RSSMXV_VAL_DM_MARKER_MODE_RATIO                                     3

// RsSMxV_rngARBExtendedTriggerMode
#define RSSMXV_VAL_ARB_EXTENDED_TRIGGER_MODE_SAME                           0
#define RSSMXV_VAL_ARB_EXTENDED_TRIGGER_MODE_NEXT                           1
#define RSSMXV_VAL_ARB_EXTENDED_TRIGGER_MODE_NSEAM                          2
#define RSSMXV_VAL_ARB_EXTENDED_TRIGGER_MODE_SEQ                            3

// RsSMxV_rngARBSignalDurationUnit
#define RSSMXV_VAL_ARB_TRIGGER_UNIT_SAMPLE                                  0
#define RSSMXV_VAL_ARB_TRIGGER_UNIT_SEQUENCE                                1

// RsSMxV_rngARBMarkerMode
#define RSSMXV_VAL_ARB_MARKER_MODE_UNCH                                     0
#define RSSMXV_VAL_ARB_MARKER_MODE_RESTART                                  1
#define RSSMXV_VAL_ARB_MARKER_MODE_PULSE                                    2
#define RSSMXV_VAL_ARB_MARKER_MODE_PATTERN                                  3
#define RSSMXV_VAL_ARB_MARKER_MODE_RATIO                                    4

// RsSMxV_rngARBMultisegmentLevelMode
#define RSSMXV_VAL_ARB_LEVEL_MODE_UNCH                                      0
#define RSSMXV_VAL_ARB_LEVEL_MODE_ERMS                                      1

// RsSMxV_rngARBSequenceRestart
#define RSSMXV_VAL_ARB_SRESTART_DISABLED                                    0
#define RSSMXV_VAL_ARB_SRESTART_MRK1                                        1
#define RSSMXV_VAL_ARB_SRESTART_MRK2                                        2
#define RSSMXV_VAL_ARB_SRESTART_MRK3                                        3
#define RSSMXV_VAL_ARB_SRESTART_MRK4                                        4

// RsSMxV_rngARBMultisegmentClockMode
#define RSSMXV_VAL_ARB_CLOCK_MODE_UNCH                                      0
#define RSSMXV_VAL_ARB_CLOCK_MODE_HIGH                                      1
#define RSSMXV_VAL_ARB_CLOCK_MODE_USER                                      2

// RsSMxV_rngARBMultiCarrierMode
#define RSSMXV_VAL_ARB_MULTIC_MODE_EQU                                      0
#define RSSMXV_VAL_ARB_MULTIC_MODE_ARB                                      1

// RsSMxV_rngARBMultiCarrierCrestFactorMode
#define RSSMXV_VAL_ARB_SRC_CFACTOR_MODE_OFF                                 0
#define RSSMXV_VAL_ARB_SRC_CFACTOR_MODE_MIN                                 1
#define RSSMXV_VAL_ARB_SRC_CFACTOR_MODE_MAX                                 2

// RsSMxV_rngARBSignalPeriodMode
#define RSSMXV_VAL_ARB_SRC_CALC_MODE_LONG                                   0
#define RSSMXV_VAL_ARB_SRC_CALC_MODE_SHRT                                   1
#define RSSMXV_VAL_ARB_SRC_CALC_MODE_USER                                   2
#define RSSMXV_VAL_ARB_SRC_CALC_MODE_LCM                                    3

// RsSMxV_rngSlope
#define RSSMXV_VAL_SLOPE_NEGATIVE                                           0
#define RSSMXV_VAL_SLOPE_POSITIVE                                           1

// RsSMxV_rngIMPOptimizationMode
#define RSSMXV_VAL_OPTIM_MODE_FAST                                          0
#define RSSMXV_VAL_OPTIM_MODE_QHIGH                                         1
#define RSSMXV_VAL_OPTIM_MODE_UCOR                                          2
#define RSSMXV_VAL_OPTIM_MODE_QHT                                           3

// RsSMxV_rngAWGNMode
#define RSSMXV_VAL_AWGN_MODE_ADD                                            0
#define RSSMXV_VAL_AWGN_MODE_ONLY                                           1
#define RSSMXV_VAL_AWGN_MODE_CW                                             2

// RsSMxV_rngAWGNNoiseMode
#define RSSMXV_VAL_AWGN_POWER_MODE_CN                                       0
#define RSSMXV_VAL_AWGN_POWER_MODE_EN                                       1
#define RSSMXV_VAL_AWGN_POWER_MODE_SN                                       2

// RsSMxV_rngKeyboardLayout
#define RSSMXV_VAL_SYSTEM_KEYBOARD_US                                       0
#define RSSMXV_VAL_SYSTEM_KEYBOARD_DE                                       1
#define RSSMXV_VAL_SYSTEM_KEYBOARD_ENGUK                                    2
#define RSSMXV_VAL_SYSTEM_KEYBOARD_CHIN                                     3
#define RSSMXV_VAL_SYSTEM_KEYBOARD_DAN                                      4
#define RSSMXV_VAL_SYSTEM_KEYBOARD_DUTC                                     5
#define RSSMXV_VAL_SYSTEM_KEYBOARD_FINN                                     6
#define RSSMXV_VAL_SYSTEM_KEYBOARD_FREN                                     7
#define RSSMXV_VAL_SYSTEM_KEYBOARD_FREB                                     8
#define RSSMXV_VAL_SYSTEM_KEYBOARD_FREC                                     9
#define RSSMXV_VAL_SYSTEM_KEYBOARD_ITAL                                     10
#define RSSMXV_VAL_SYSTEM_KEYBOARD_JAP                                      11
#define RSSMXV_VAL_SYSTEM_KEYBOARD_KOR                                      12
#define RSSMXV_VAL_SYSTEM_KEYBOARD_NORW                                     13
#define RSSMXV_VAL_SYSTEM_KEYBOARD_PORT                                     14
#define RSSMXV_VAL_SYSTEM_KEYBOARD_RUSS                                     15
#define RSSMXV_VAL_SYSTEM_KEYBOARD_SPAN                                     16
#define RSSMXV_VAL_SYSTEM_KEYBOARD_SWED                                     17

// RsSMxV_rngGraphicsMode
#define RSSMXV_VAL_GRAPHICS_MODE_IQ                                         0
#define RSSMXV_VAL_GRAPHICS_MODE_VECT                                       1
#define RSSMXV_VAL_GRAPHICS_MODE_CCDF                                       2
#define RSSMXV_VAL_GRAPHICS_MODE_PSP                                        3
#define RSSMXV_VAL_GRAPHICS_MODE_CONST                                      4
#define RSSMXV_VAL_GRAPHICS_MODE_EYEI                                       5
#define RSSMXV_VAL_GRAPHICS_MODE_EYEQ                                       6

// RsSMxV_rngGraphicsTriggerSource
#define RSSMXV_VAL_GRAPHICS_TRIGGER_SOURCE_SOFTWARE                         0
#define RSSMXV_VAL_GRAPHICS_TRIGGER_SOURCE_MARKER                           1

// RsSMxV_rngGraphicsSampleRateMode
#define RSSMXV_VAL_GRAPHICS_SAMPLE_RATE_AUTO                                0
#define RSSMXV_VAL_GRAPHICS_SAMPLE_RATE_FULL                                1
#define RSSMXV_VAL_GRAPHICS_SAMPLE_RATE_USER                                2

// RsSMxV_rngUnitAngle
#define RSSMXV_VAL_UNIT_ANGLE_DEG                                           0
#define RSSMXV_VAL_UNIT_ANGLE_RAD                                           1

// RsSMxV_rngUnitPower
#define RSSMXV_VAL_UNIT_POWER_V                                             0
#define RSSMXV_VAL_UNIT_POWER_DBM                                           1
#define RSSMXV_VAL_UNIT_POWER_DBUV                                          2

// RsSMxV_rngHardCopyLanguage
#define RSSMXV_VAL_HCOPY_LANG_BMP                                           0
#define RSSMXV_VAL_HCOPY_LANG_JPG                                           1
#define RSSMXV_VAL_HCOPY_LANG_XPM                                           2
#define RSSMXV_VAL_HCOPY_LANG_PNG                                           3

// RsSMxV_rngTriggerSourceForAllSweeps
#define RSSMXV_VAL_TRIG_SOURCE_SWEEP_AUTO                                   0
#define RSSMXV_VAL_TRIG_SOURCE_SWEEP_IMMEDIATE                              1
#define RSSMXV_VAL_TRIG_SOURCE_SWEEP_SINGLE                                 2
#define RSSMXV_VAL_TRIG_SOURCE_SWEEP_EXTERNAL                               3
#define RSSMXV_VAL_TRIG_SOURCE_SWEEP_BUS                                    4
#define RSSMXV_VAL_TRIG_SOURCE_SWEEP_EAUT                                   5

// RsSMxV_rngRFFrequencyonRecall
#define RSSMXV_VAL_FREQUENCY_RCL_INCLUDE                                    0
#define RSSMXV_VAL_FREQUENCY_RCL_EXCLUDE                                    1

// RsSMxV_rngROSCSynchronizationBandwidth
#define RSSMXV_VAL_ROSC_SBAN_WIDE                                           0
#define RSSMXV_VAL_ROSC_SBAN_NARROW                                         1

// RsSMxV_rngTriggerStatus
#define RSSMXV_VAL_TRIGGER_STATUS_STOP                                      0
#define RSSMXV_VAL_TRIGGER_STATUS_RUN                                       1

// RsSMxV_rngIQOutputPowerOnState
#define RSSMXV_VAL_OUTPUT_STATE_PON_OFF                                     0
#define RSSMXV_VAL_OUTPUT_STATE_PON_UNCH                                    1

// RsSMxV_rngRFLevelVariationActive
#define RSSMXV_VAL_RF_VARIATION_ACTIVE_DECIMAL                              0
#define RSSMXV_VAL_RF_VARIATION_ACTIVE_USER                                 1

// RsSMxV_rngTriggerSour
#define RSSMXV_VAL_TRIGGER_SOURCE_INTERNAL                                  0
#define RSSMXV_VAL_TRIGGER_SOURCE_EXT                                       1

// RsSMxV_rngClockSource
#define RSSMXV_VAL_CLOCK_SOURCE_INTERNAL                                    0
#define RSSMXV_VAL_CLOCK_SOURCE_EGC1                                        1
#define RSSMXV_VAL_CLOCK_SOURCE_EGC2                                        2
#define RSSMXV_VAL_CLOCK_SOURCE_ELC                                         3

// RsSMxV_rngPULMTransitionType
#define RSSMXV_VAL_PULM_TRANSITION_TYPE_SMOOTHED                            0
#define RSSMXV_VAL_PULM_TRANSITION_TYPE_FAST                                1

// RsSMxV_rngGraphicsSource
#define RSSMXV_VAL_GRAPHICS_SOURCE_BBA                                      0
#define RSSMXV_VAL_GRAPHICS_SOURCE_STRA                                     1
#define RSSMXV_VAL_GRAPHICS_SOURCE_RFA                                      2
#define RSSMXV_VAL_GRAPHICS_SOURCE_BBIA                                     3
#define RSSMXV_VAL_GRAPHICS_SOURCE_IQO1                                     4
#define RSSMXV_VAL_GRAPHICS_SOURCE_DO1                                      5

// RsSMxV_rngRFAttRFOFFMode
#define RSSMXV_VAL_RF_OFF_MODE_UNCH                                         0
#define RSSMXV_VAL_RF_OFF_MODE_FULL                                         1

// RsSMxV_rngRFLevelSettingCharacteristics
#define RSSMXV_VAL_RF_LEVEL_SETTING_AUTO                                    0
#define RSSMXV_VAL_RF_LEVEL_SETTING_UNINT                                   1

// RsSMxV_rngExternalDisplayMode
#define RSSMXV_VAL_EXTERNAL_DISPLAY_ALL                                     0
#define RSSMXV_VAL_EXTERNAL_DISPLAY_MAPP                                    1
#define RSSMXV_VAL_EXTERNAL_DISPLAY_INP                                     2
#define RSSMXV_VAL_EXTERNAL_DISPLAY_OUTP                                    3

// RsSMxV_rngRefOscOutputFreqMode
#define RSSMXV_VAL_REFERENCE_OSCILLATOR_FREQ_MODE_DER10M                    0
#define RSSMXV_VAL_REFERENCE_OSCILLATOR_FREQ_MODE_OFF                       1
#define RSSMXV_VAL_REFERENCE_OSCILLATOR_FREQ_MODE_LOOP                      2

// RsSMxV_rngARBPowerReference
#define RSSMXV_VAL_ARB_POWER_REFERENCE_RMS                                  0
#define RSSMXV_VAL_ARB_POWER_REFERENCE_PEAK                                 1

// RsSMxV_rngSystemConfigurationMappingMode
#define RSSMXV_VAL_SYST_CONF_MAPPING_MODE_SINGLE                            0
#define RSSMXV_VAL_SYST_CONF_MAPPING_MODE_ADD                               1
#define RSSMXV_VAL_SYST_CONF_MAPPING_MODE_MULTIPLEX                         2

// RsSMxV_rngARBMultisegmentMarkerMode
#define RSSMXV_VAL_ARB_MARKER_MODE_IGNORE                                   0
#define RSSMXV_VAL_ARB_MARKER_MODE_TAKE                                     1

// RsSMxV_rngDMExternalTriggerDelayUnit
#define RSSMXV_VAL_EXTERNAL_TRIGGER_DELAY_UNIT_SAMPLE                       0
#define RSSMXV_VAL_EXTERNAL_TRIGGER_DELAY_UNIT_TIME                         1

// RsSMxV_rngARBMultisegmentSegmentLevelMode
#define RSSMXV_VAL_MULTI_SEGMENTS_ARB_LEVEL_MODE_UNCHANGED                  0
#define RSSMXV_VAL_MULTI_SEGMENTS_ARB_LEVEL_MODE_HIGHEST                    1

// RsSMxV_rngHardCopyRegion
#define RSSMXV_VAL_HCOPY_SNAPSHOT_ALL                                       0
#define RSSMXV_VAL_HCOPY_SNAPSHOT_DIALOG                                    1

// RsSMxV_rngRFFrequencyModeNew
#define RSSMXV_VAL_RF_FREQUENCY_CW                                          0
#define RSSMXV_VAL_RF_FREQUENCY_FIXED                                       1
#define RSSMXV_VAL_RF_FREQUENCY_SWEEP                                       2
#define RSSMXV_VAL_RF_FREQUENCY_LIST                                        3

// RsSMxV_rngARBTriggerStatus
#define RSSMXV_VAL_ARB_TRIGGER_STATUS_STOP                                  0
#define RSSMXV_VAL_ARB_TRIGGER_STATUS_RUN                                   1

// RsSMxV_rngRFPowerMode
#define RSSMXV_VAL_RF_POWER_CW                                              0
#define RSSMXV_VAL_RF_POWER_FIXED                                           1
#define RSSMXV_VAL_RF_POWER_SWEEP                                           2

// RsSMxV_rngOutputImpedance
#define RSSMXV_VAL_IMPEDANCE_G50                                            0
#define RSSMXV_VAL_IMPEDANCE_G1K                                            1
#define RSSMXV_VAL_IMPEDANCE_G10K                                           2

// RsSMxV_rngGraphicsFFTSize
#define RSSMXV_VAL_GRAPHICS_FFT_LEN256                                      0
#define RSSMXV_VAL_GRAPHICS_FFT_LEN512                                      1
#define RSSMXV_VAL_GRAPHICS_FFT_LEN1024                                     2
#define RSSMXV_VAL_GRAPHICS_FFT_LEN2048                                     3
#define RSSMXV_VAL_GRAPHICS_FFT_LEN4096                                     4
#define RSSMXV_VAL_GRAPHICS_FFT_LEN8192                                     5

// RsSMxV_rngSystemUserInterfaceLock
#define RSSMXV_VAL_SYSTEM_ULOCK_TOFF                                        0
#define RSSMXV_VAL_SYSTEM_ULOCK_ENABLED                                     1
#define RSSMXV_VAL_SYSTEM_ULOCK_DISABLED                                    2
#define RSSMXV_VAL_SYSTEM_ULOCK_DONLY                                       3
#define RSSMXV_VAL_SYSTEM_ULOCK_VNCONLY                                     4

// RsSMxV_rngAWGNPowersForOutput
#define RSSMXV_VAL_AWGN_OUTPUT_RF                                           0
#define RSSMXV_VAL_AWGN_OUTPUT_BB                                           1
#define RSSMXV_VAL_AWGN_OUTPUT_RFA                                          2
#define RSSMXV_VAL_AWGN_OUTPUT_IQOUT1                                       3

// RsSMxV_rngAWGNReferenceMode
#define RSSMXV_VAL_AWGN_REF_MODE_CARRIER                                    0
#define RSSMXV_VAL_AWGN_REF_MODE_NOISE                                      1

// RsSMxV_rngVariationStepMode
#define RSSMXV_VAL_STEP_MODE_DECIMAL                                        0
#define RSSMXV_VAL_STEP_MODE_USER                                           1

// RsSMxV_rngARBTestSignalForm
#define RSSMXV_VAL_ARB_TESTSIGNAL_SINE                                      0
#define RSSMXV_VAL_ARB_TESTSIGNAL_RECT                                      1
#define RSSMXV_VAL_ARB_TESTSIGNAL_CIQ                                       2

// RsSMxV_rngDMAPSK16Gamma
#define RSSMXV_VAL_DM_APSK16_G2D3                                           0
#define RSSMXV_VAL_DM_APSK16_G3D4                                           1
#define RSSMXV_VAL_DM_APSK16_G4D5                                           2
#define RSSMXV_VAL_DM_APSK16_G5D6                                           3
#define RSSMXV_VAL_DM_APSK16_G8D9                                           4
#define RSSMXV_VAL_DM_APSK16_G9D10                                          5

// RsSMxV_rngDM32APSKGamma
#define RSSMXV_VAL_DM_APSK32_G3D4                                           0
#define RSSMXV_VAL_DM_APSK32_G4D5                                           1
#define RSSMXV_VAL_DM_APSK32_G5D6                                           2
#define RSSMXV_VAL_DM_APSK32_G8D9                                           3
#define RSSMXV_VAL_DM_APSK32_G9D10                                          4

// RsSMxV_rngARBCFRFilteringMode
#define RSSMXV_VAL_ARB_CFR_FILTERING_MODE_SIMP                              0
#define RSSMXV_VAL_ARB_CFR_FILTERING_MODE_ENH                               1

// RsSMxV_rngARBCFRAlgorithm
#define RSRFSIGEN_VAL_ARB_CFR_ALGORITHM_CLIPPING_FILTERING                  0

// RsSMxV_rngARBMultisegmentNextSegmentSource
#define RSSMXV_VAL_ARB_NEXT_SEGMENT_SOURCE_INT                              0
#define RSSMXV_VAL_ARB_NEXT_SEGMENT_SOURCE_NSEGM1                           1

// RsSMxV_rngIQOutputDigitalInterface
#define RSSMXV_VAL_IQ_DIGITAL_INTERFACE_DIGITAL                             0
#define RSSMXV_VAL_IQ_DIGITAL_INTERFACE_HSDIN                               1

// RsSMxV_rngSystemIdentification
#define RSSMXV_VAL_SYSTEM_ID_AUTO                                           0
#define RSSMXV_VAL_SYSTEM_ID_USER                                           1

// RsSMxV_rngAMShape
#define RSSMXV_VAL_AM_SHAPE_SINE                                            0

// RsSMxV_rngPULMMode
#define RSSMXV_VAL_PULM_MODE_SINGLE                                         0
#define RSSMXV_VAL_PULM_MODE_DOUBLE                                         1

// RsSMxV_rngPULMVideoPolarity
#define RSSMXV_VAL_PULM_VIDEO_POLARITY_NORMAL                               0
#define RSSMXV_VAL_PULM_VIDEO_POLARITY_INVERTED                             1

// RsSMxV_rngHSDDirection
#define RSSMXV_VAL_HSD_DIRECTION_NONE                                       0
#define RSSMXV_VAL_HSD_DIRECTION_IN                                         1
#define RSSMXV_VAL_HSD_DIRECTION_OUT                                        2

// RsSMxV_rngAttenMode
#define RSSMXV_VAL_ATTEN_MODE_AUTO                                          0
#define RSSMXV_VAL_ATTEN_MODE_FIXED                                         1

// RsSMxV_rngPowerFilterMode
#define RSSMXV_VAL_SENSE_POWER_FILTER_TYPE_AUTO                             0
#define RSSMXV_VAL_SENSE_POWER_FILTER_TYPE_USER                             1
#define RSSMXV_VAL_SENSE_POWER_FILTER_TYPE_NSRATIO                          2

// RsSMxV_rngSweepTriggerSource
#define RSSMXV_VAL_SWEEP_TRIGGER_SOURCE_AUTO                                0
#define RSSMXV_VAL_SWEEP_TRIGGER_SOURCE_IMMEDIATE                           1
#define RSSMXV_VAL_SWEEP_TRIGGER_SOURCE_SINGLE                              2
#define RSSMXV_VAL_SWEEP_TRIGGER_SOURCE_BUS                                 3
#define RSSMXV_VAL_SWEEP_TRIGGER_SOURCE_EXTERNAL                            4
#define RSSMXV_VAL_SWEEP_TRIGGER_SOURCE_EXTERNAL_AUTO                       5

// RsSMxV_rngInputConnectorSignal
#define RSSMXV_VAL_INPUT_SIGNAL_TRIG1                                       0
#define RSSMXV_VAL_INPUT_SIGNAL_TS                                          1
#define RSSMXV_VAL_INPUT_SIGNAL_SPDIF                                       2
#define RSSMXV_VAL_INPUT_SIGNAL_PPS                                         3
#define RSSMXV_VAL_INPUT_SIGNAL_ETI                                         4
#define RSSMXV_VAL_INPUT_SIGNAL_NSEGM1                                      5
#define RSSMXV_VAL_INPUT_SIGNAL_INST                                        6

// RsSMxV_rngBBINDigitalInterface
#define RSSMXV_VAL_BBIN_DIGITAL_INTERFACE_DIGITAL                           0
#define RSSMXV_VAL_BBIN_DIGITAL_INTERFACE_HSDIN                             1

// RsSMxV_rngBBINSampleSource
#define RSSMXV_VAL_BBIN_SAMPLE_SOURCE_HSDIN                                 0

// RsSMxV_rngListDwellTimeMode
#define RSSMXV_VAL_DWELL_MODE_LIST                                          0
#define RSSMXV_VAL_DWELL_MODE_GLOBAL                                        1

// RsSMxV_rngListRunMode
#define RSSMXV_VAL_LIST_RUN_LEARNED                                         0
#define RSSMXV_VAL_LIST_RUN_LIVE                                            1

// RsSMxV_rngListTriggerSource
#define RSSMXV_VAL_LIST_TRIGGER_SOURCE_AUTO                                 0
#define RSSMXV_VAL_LIST_TRIGGER_SOURCE_SINGLE                               1
#define RSSMXV_VAL_LIST_TRIGGER_SOURCE_EXTERNAL                             2
#define RSSMXV_VAL_LIST_TRIGGER_SOURCE_IMMEDIATE                            3
#define RSSMXV_VAL_LIST_TRIGGER_SOURCE_BUS                                  4

// RsSMxV_rngSweepMode
#define RSSMXV_VAL_SWEEP_MODE_AUTO                                          0
#define RSSMXV_VAL_SWEEP_MODE_MANUAL                                        1
#define RSSMXV_VAL_SWEEP_MODE_STEP                                          2

// RsSMxV_rngFrequencySweepSpacing
#define RSSMXV_VAL_FREQ_SWEEP_SPACING_LINEAR                                0
#define RSSMXV_VAL_FREQ_SWEEP_SPACING_LOGARITHMIC                           1

// RsSMxV_rngSystemIPAddressMode
#define RSSMXV_VAL_SYSTEM_COMMUNICATE_IP_MODE_STATIC                        0
#define RSSMXV_VAL_SYSTEM_COMMUNICATE_IP_MODE_AUTO                          1

// RsSMxV_rngBBINSignalRouting
#define RSSMXV_VAL_BB_SIGNAL_ROUTING_A                                      0

// RsSMxV_rngBBINMode
#define RSSMXV_VAL_BBIN_MODE_DIGITAL                                        0

// RsSMxV_rngCorrectionListMode
#define RSSMXV_VAL_DEXC_MODE_IMPORT                                         0
#define RSSMXV_VAL_DEXC_MODE_EXPORT                                         1

// RsSMxV_rngCorrectionASCIIExtension
#define RSSMXV_VAL_DEXC_ASCII_SOURCE_FILE_EXT_TXT                           0
#define RSSMXV_VAL_DEXC_ASCII_SOURCE_FILE_EXT_CSV                           1

// RsSMxV_rngCorrectionASCIISeparator
#define RSSMXV_VAL_DEXC_ASCII_SOURCE_FILE_COLUMN_SEP_TAB                    0
#define RSSMXV_VAL_DEXC_ASCII_SOURCE_FILE_COLUMN_SEP_SEM                    1
#define RSSMXV_VAL_DEXC_ASCII_SOURCE_FILE_COLUMN_SEP_COMMA                  2
#define RSSMXV_VAL_DEXC_ASCII_SOURCE_FILE_COLUMN_SEP_SPACE                  3

// RsSMxV_rngCorrectionASCIIColumnDecimalSeparator
#define RSSMXV_VAL_DEXC_ASCII_SOURCE_FILE_COLUMN_DEC_SEP_DOT                0
#define RSSMXV_VAL_DEXC_ASCII_SOURCE_FILE_COLUMN_DEC_SEP_COMMA              1

// RsSMxV_rngFrequencySweepShape
#define RSSMXV_VAL_SWEEP_SHAPE_SAW                                          0
#define RSSMXV_VAL_SWEEP_SHAPE_TRI                                          1

// RsSMxV_rngPowerUnit
#define RSSMXV_VAL_SENSE_POWER_UNIT_DBM                                     0
#define RSSMXV_VAL_SENSE_POWER_UNIT_DBUV                                    1
#define RSSMXV_VAL_SENSE_POWER_UNIT_WATT                                    2

// RsSMxV_rngPowerSource
#define RSSMXV_VAL_SENSE_POWER_SOURCE_A                                     0
#define RSSMXV_VAL_SENSE_POWER_SOURCE_USER                                  1
#define RSSMXV_VAL_SENSE_POWER_SOURCE_RF                                    2

// RsSMxV_rngPowerSensorMapping
#define RSSMXV_VAL_POWER_SENSOR_MAPPING_SENSOR1                             0
#define RSSMXV_VAL_POWER_SENSOR_MAPPING_SENSOR2                             1
#define RSSMXV_VAL_POWER_SENSOR_MAPPING_SENSOR3                             2
#define RSSMXV_VAL_POWER_SENSOR_MAPPING_SENSOR4                             3
#define RSSMXV_VAL_POWER_SENSOR_MAPPING_UNMAPPED                            4

// RsSMxV_rngOutputConnectorUserSignal
#define RSSMXV_VAL_OUTPUT_MARKER_A1                                         0
#define RSSMXV_VAL_OUTPUT_NONE                                              1

// RsSMxV_rngOutputConnectorDirection
#define RSSMXV_VAL_OUTPUT_DIRECTION_INPUT                                   0
#define RSSMXV_VAL_OUTPUT_DIRECTION_OUTPUT                                  1
#define RSSMXV_VAL_OUTPUT_DIRECTION_UNUSED                                  2

// RsSMxV_rngControlImpedance
#define RSSMXV_VAL_INPUT_IMP_G1K                                            0
#define RSSMXV_VAL_INPUT_IMP_G50                                            1

// RsSMxV_rngIQOutputDigitalPowerVia
#define RSSMXV_VAL_IQ_OUTPUT_DIGITAL_POWER_PEP                              0
#define RSSMXV_VAL_IQ_OUTPUT_DIGITAL_POWER_LEVEL                            1

// RsSMxV_rngRegisterGroup
#define RSSMXV_VAL_REGISTER_OPERATION                                       0
#define RSSMXV_VAL_REGISTER_QUESTIONABLE                                    1

// RsSMxV_rngRegisterType
#define RSSMXV_VAL_REGISTER_CONDITION                                       0
#define RSSMXV_VAL_REGISTER_EVENT                                           1
#define RSSMXV_VAL_REGISTER_ENABLE                                          2
#define RSSMXV_VAL_REGISTER_NTR                                             3
#define RSSMXV_VAL_REGISTER_PTR                                             4

// RsSMxV_rngAudioSignalSource
#define RSSMXV_VAL_AUDIO_SOURCE_EXTERNAL                                    0
#define RSSMXV_VAL_AUDIO_SOURCE_GENERATOR                                   1
#define RSSMXV_VAL_AUDIO_SOURCE_PLAYER                                      2
#define RSSMXV_VAL_AUDIO_SOURCE_OFF                                         3

// RsSMxV_rngAMAudioExternalSource
#define RSSMXV_VAL_AUDIO_SOURCE_EXTERNAL_SPDIF                              0

// RsSMxV_rngStereoAudioMode
#define RSSMXV_VAL_STEREO_AUDIO_MODE_LEFT                                   0
#define RSSMXV_VAL_STEREO_AUDIO_MODE_RIGHT                                  1
#define RSSMXV_VAL_STEREO_AUDIO_MODE_ST_LR                                  2
#define RSSMXV_VAL_STEREO_AUDIO_MODE_ST_LNR                                 3
#define RSSMXV_VAL_STEREO_AUDIO_MODE_ST_TRUE                                4

// RsSMxV_rngFMAudioModulationMode
#define RSSMXV_VAL_FM_MODULATION_MODE_MONO                                  0
#define RSSMXV_VAL_FM_MODULATION_MODE_STEREO                                1

// RsSMxV_rngFMAudioModulationPreemphasis
#define RSSMXV_VAL_FM_AUDIO_PREEMPHASIS_OFF                                 0
#define RSSMXV_VAL_FM_AUDIO_PREEMPHASIS_D50                                 1
#define RSSMXV_VAL_FM_AUDIO_PREEMPHASIS_D75                                 2

// RsSMxV_rngRDSProgramType
#define RSSMXV_VAL_RDS_PROGRAM_NONE                                         0
#define RSSMXV_VAL_RDS_PROGRAM_NEWS                                         1
#define RSSMXV_VAL_RDS_PROGRAM_CURRENT_AFFAIRS                              2
#define RSSMXV_VAL_RDS_PROGRAM_INFORMATION                                  3
#define RSSMXV_VAL_RDS_PROGRAM_SPORT                                        4
#define RSSMXV_VAL_RDS_PROGRAM_EDUCATION                                    5
#define RSSMXV_VAL_RDS_PROGRAM_DRAMA                                        6
#define RSSMXV_VAL_RDS_PROGRAM_CULTURE                                      7
#define RSSMXV_VAL_RDS_PROGRAM_SCIENCE                                      8
#define RSSMXV_VAL_RDS_PROGRAM_VARIED                                       9
#define RSSMXV_VAL_RDS_PROGRAM_POP_MUSIC                                    10
#define RSSMXV_VAL_RDS_PROGRAM_ROCK_MUSIC                                   11
#define RSSMXV_VAL_RDS_PROGRAM_EASY_LISTENING                               12
#define RSSMXV_VAL_RDS_PROGRAM_LIGHT_CLASSICAL                              13
#define RSSMXV_VAL_RDS_PROGRAM_SERIOUS_CLASSICAL                            14
#define RSSMXV_VAL_RDS_PROGRAM_OTHER_MUSIC                                  15
#define RSSMXV_VAL_RDS_PROGRAM_WEATHER                                      16
#define RSSMXV_VAL_RDS_PROGRAM_FINANCE                                      17
#define RSSMXV_VAL_RDS_PROGRAM_CHILDREN                                     18
#define RSSMXV_VAL_RDS_PROGRAM_SOCIAL                                       19
#define RSSMXV_VAL_RDS_PROGRAM_RELIGION                                     20
#define RSSMXV_VAL_RDS_PROGRAM_PHONE_IN                                     21
#define RSSMXV_VAL_RDS_PROGRAM_TRAVEL                                       22
#define RSSMXV_VAL_RDS_PROGRAM_LEISURE                                      23
#define RSSMXV_VAL_RDS_PROGRAM_JAZZ_MUSIC                                   24
#define RSSMXV_VAL_RDS_PROGRAM_COUNTRY_MUSIC                                25
#define RSSMXV_VAL_RDS_PROGRAM_NATIONAL_MUSIC                               26
#define RSSMXV_VAL_RDS_PROGRAM_OLDIES_MUSIC                                 27
#define RSSMXV_VAL_RDS_PROGRAM_FOLK_MUSIC                                   28
#define RSSMXV_VAL_RDS_PROGRAM_DOCUMENTARY                                  29
#define RSSMXV_VAL_RDS_PROGRAM_ALARM_TEST                                   30
#define RSSMXV_VAL_RDS_PROGRAM_ALARM                                        31

// RsSMxV_rngRDSMSFlag
#define RSSMXV_VAL_FM_AUDIO_RDS_MUSIC                                       0
#define RSSMXV_VAL_FM_AUDIO_RDS_SPEECH                                      1

// RsSMxV_rngDIStereoDecoder
#define RSSMXV_VAL_RDS_DI_DECODER_MONO                                      0
#define RSSMXV_VAL_RDS_DI_DECODER_STEREO                                    1

// RsSMxV_rngRDSAFMethod
#define RSSMXV_VAL_RDS_AF_METHOD_A                                          0
#define RSSMXV_VAL_RDS_AF_METHOD_B                                          1

// RsSMxV_rngAFListEntryOrder
#define RSSMXV_VAL_RDS_AF_ORDER_ASCENDING                                   0
#define RSSMXV_VAL_RDS_AF_ORDER_DESCENDING                                  1

// RsSMxV_rngEONAFMethod
#define RSSMXV_VAL_EON_AF_METHOD_A                                          0
#define RSSMXV_VAL_EON_AF_METHOD_MAPF                                       1

// RsSMxV_rngDARCInformation
#define RSSMXV_VAL_DARC_INFORMATION_OFF                                     0
#define RSSMXV_VAL_DARC_INFORMATION_PRBS                                    1
#define RSSMXV_VAL_DARC_INFORMATION_DATA                                    2

// RsSMxV_rngBroadcastInputSource
#define RSSMXV_VAL_INPUT_SOURCE_EXTERNAL                                    0
#define RSSMXV_VAL_INPUT_SOURCE_TS_PLAYER                                   1
#define RSSMXV_VAL_INPUT_SOURCE_TEST_SIGNAL                                 2

// RsSMxV_rngTDMBInput
#define RSSMXV_VAL_PROTOCOL_TS                                              0
#define RSSMXV_VAL_PROTOCOL_IP                                              1

// RsSMxV_rngTDMBInputFormat
#define RSSMXV_VAL_INPUT_FORMAT_ETI                                         0

// RsSMxV_rngTDMBETISignalType
#define RSSMXV_VAL_TDMB_ETI_SIGNAL_INVALID                                  0
#define RSSMXV_VAL_TDMB_ETI_SIGNAL_ENI                                      1
#define RSSMXV_VAL_TDMB_ETI_SIGNAL_E559                                     2
#define RSSMXV_VAL_TDMB_ETI_SIGNAL_E537                                     3

// RsSMxV_rngTDMBModeIdentity
#define RSSMXV_VAL_TDMB_TRANSMISSION_MODE_INVALID                           0
#define RSSMXV_VAL_TDMB_TRANSMISSION_MODE_ONE                               1
#define RSSMXV_VAL_TDMB_TRANSMISSION_MODE_TWO                               2
#define RSSMXV_VAL_TDMB_TRANSMISSION_MODE_THREE                             3
#define RSSMXV_VAL_TDMB_TRANSMISSION_MODE_FOUR                              4

// RsSMxV_rngETIProtectionProfile
#define RSSMXV_VAL_ETI_PROTECTION_PROFILE_UNEQUAL                           0
#define RSSMXV_VAL_ETI_PROTECTION_PROFILE_EQUAL                             1

// RsSMxV_rngETIProtectionLevel
#define RSSMXV_VAL_ETI_PROTECTION_LEVEL_UNDEFINED                           0
#define RSSMXV_VAL_ETI_PROTECTION_LEVEL_EP1A                                1
#define TSSMXV_VAL_ETI_PROTECTION_LEVEL_EP2A                                2
#define TSSMXV_VAL_ETI_PROTECTION_LEVEL_EP3A                                3
#define TSSMXV_VAL_ETI_PROTECTION_LEVEL_EP4A                                4
#define RSSMXV_VAL_ETI_PROTECTION_LEVEL_EP1B                                5
#define TSSMXV_VAL_ETI_PROTECTION_LEVEL_EP2B                                6
#define TSSMXV_VAL_ETI_PROTECTION_LEVEL_EP3B                                7
#define TSSMXV_VAL_ETI_PROTECTION_LEVEL_EP4B                                8
#define TSSMXV_VAL_ETI_PROTECTION_LEVEL_UP1                                 9
#define TSSMXV_VAL_ETI_PROTECTION_LEVEL_UP2                                 10
#define TSSMXV_VAL_ETI_PROTECTION_LEVEL_UP3                                 11
#define TSSMXV_VAL_ETI_PROTECTION_LEVEL_UP4                                 12
#define TSSMXV_VAL_ETI_PROTECTION_LEVEL_UP5                                 13

// RsSMxV_rngNetworkFrequencyMode
#define RSSMXV_VAL_NETWORK_MODE_MULTI_FREQ                                  0
#define RSSMXV_VAL_NETWORK_MODE_SINGLE_FREQ                                 1

// RsSMxV_rngTDMBSpecialTransmissionMode
#define RSSMXV_VAL_TDMB_SPECIAL_TRANSMISSION_MODE_MID                       0
#define RSSMXV_VAL_TDMB_SPECIAL_TRANSMISSION_MODE_MANUAL                    1

// RsSMxV_rngDRMInputSource
#define RSSMXV_VAL_DRM_INPUT_SOURCE_EXTERNAL                                0
#define RSSMXV_VAL_DRM_INPUT_SOURCE_FILE                                    1

// RsSMxV_rngDRMInputAudioServicesNumberDFA
#define RSSMXV_VAL_DRM_INPUT_AUDIO_NUMER_ZERO                               0
#define RSSMXV_VAL_DRM_INPUT_AUDIO_NUMER_ONE                                1
#define RSSMXV_VAL_DRM_INPUT_AUDIO_NUMER_TWO                                2
#define RSSMXV_VAL_DRM_INPUT_AUDIO_NUMER_THREE                              3
#define RSSMXV_VAL_DRM_INPUT_AUDIO_NUMER_FOUR                               4
#define RSSMXV_VAL_DRM_INPUT_AUDIO_NUMER_INVALID                            5

// RsSMxV_rngDRMInputDataServicesNumber
#define RSSMXV_VAL_DRM_INPUT_DATA_NUMER_ZERO                                0
#define RSSMXV_VAL_DRM_INPUT_DATA_NUMER_ONE                                 1
#define RSSMXV_VAL_DRM_INPUT_DATA_NUMER_TWO                                 2
#define RSSMXV_VAL_DRM_INPUT_DATA_NUMER_THREE                               3
#define RSSMXV_VAL_DRM_INPUT_DATA_NUMER_FOUR                                4
#define RSSMXV_VAL_DRM_INPUT_DATA_NUMER_INVALID                             5

// RsSMxV_rngDRMInputLayerType
#define RSSMXV_VAL_DRM_INPUT_LAYER_TYPE_INVALID                             0
#define RSSMXV_VAL_DRM_INPUT_LAYER_TYPE_BASE                                1
#define RSSMXV_VAL_DRM_INPUT_LAYER_TYPE_ENHANCEMENT                         2

// RsSMxV_rngDRMCodingRobustnessMode
#define RSSMXV_VAL_DRM_CODING_ROBUSTNESS_MODE_INVALID                       0
#define RSSMXV_VAL_DRM_CODING_ROBUSTNESS_MODE_A                             1
#define RSSMXV_VAL_DRM_CODING_ROBUSTNESS_MODE_B                             2
#define RSSMXV_VAL_DRM_CODING_ROBUSTNESS_MODE_C                             3
#define RSSMXV_VAL_DRM_CODING_ROBUSTNESS_MODE_D                             4
#define RSSMXV_VAL_DRM_CODING_ROBUSTNESS_MODE_E                             5

// RsSMxV_rngDRMCodingConstellationMSC
#define RSSMXV_VAL_DRM_CODING_CONSTELLATION_MSC_INVALID                     0
#define RSSMXV_VAL_DRM_CODING_CONSTELLATION_MSC_Q64N                        1
#define RSSMXV_VAL_DRM_CODING_CONSTELLATION_MSC_Q64I                        2
#define RSSMXV_VAL_DRM_CODING_CONSTELLATION_MSC_Q64Q                        3
#define RSSMXV_VAL_DRM_CODING_CONSTELLATION_MSC_Q16                         4
#define RSSMXV_VAL_DRM_CODING_CONSTELLATION_MSC_Q4                          5

// RsSMxV_rngDRMCodingConstellationSDC
#define RSSMXV_VAL_DRM_CODING_CONSTELLATION_SDC_INVALID                     0
#define RSSMXV_VAL_DRM_CODING_CONSTELLATION_SDC_Q16                         1
#define RSSMXV_VAL_DRM_CODING_CONSTELLATION_SDC_Q4                          2

// RsSMxV_rngDRMCodingInterleaverDepth
#define RSSMXV_VAL_DRM_CODING_INTERLEAVER_DEPTH_INVALID                     0
#define RSSMXV_VAL_DRM_CODING_INTERLEAVER_DEPTH_MS4                         1
#define RSSMXV_VAL_DRM_CODING_INTERLEAVER_DEPTH_MS6                         2
#define RSSMXV_VAL_DRM_CODING_INTERLEAVER_DEPTH_S2                          3

// RsSMxV_rngDRMCodingChannelBandwidth
#define RSSMXV_VAL_DRM_CODING_CHANNEL_BANDWIDTH_INVALID                     0
#define RSSMXV_VAL_DRM_CODING_CHANNEL_BANDWIDTH_K045                        1
#define RSSMXV_VAL_DRM_CODING_CHANNEL_BANDWIDTH_K05                         2
#define RSSMXV_VAL_DRM_CODING_CHANNEL_BANDWIDTH_K09                         3
#define RSSMXV_VAL_DRM_CODING_CHANNEL_BANDWIDTH_K10                         4
#define RSSMXV_VAL_DRM_CODING_CHANNEL_BANDWIDTH_K18                         5
#define RSSMXV_VAL_DRM_CODING_CHANNEL_BANDWIDTH_K20                         6
#define RSSMXV_VAL_DRM_CODING_CHANNEL_BANDWIDTH_K100                        7

// RsSMxV_rngDRMCodeRateSDCProtectionProfile
#define RSSMXV_VAL_DRM_CODE_RATE_PROTECTION_PROFILE_SDC_INV                 0
#define RSSMXV_VAL_DRM_CODE_RATE_PROTECTION_PROFILE_SDC_EEP                 1

// RsSMxV_rngDRMCodeRateSDCProtectionLevel
#define RSSMXV_VAL_DRM_CODE_RATE_PROTECTION_LEVEL_SDC_ZERO                  0
#define RSSMXV_VAL_DRM_CODE_RATE_PROTECTION_LEVEL_SDC_ONE                   1
#define RSSMXV_VAL_DRM_CODE_RATE_PROTECTION_LEVEL_SDC_INVALID               2

// RsSMxV_rngDRMSDCCodeRate
#define RSSMXV_VAL_DRM_CODE_RATE_INVALID                                    0
#define RSSMXV_VAL_DRM_CODE_RATE_R025                                       1
#define RSSMXV_VAL_DRM_CODE_RATE_R033                                       2
#define RSSMXV_VAL_DRM_CODE_RATE_R040                                       3
#define RSSMXV_VAL_DRM_CODE_RATE_R041                                       4
#define RSSMXV_VAL_DRM_CODE_RATE_R045                                       5
#define RSSMXV_VAL_DRM_CODE_RATE_R048                                       6
#define RSSMXV_VAL_DRM_CODE_RATE_R050                                       7
#define RSSMXV_VAL_DRM_CODE_RATE_R055                                       8
#define RSSMXV_VAL_DRM_CODE_RATE_R057                                       9
#define RSSMXV_VAL_DRM_CODE_RATE_R058                                       10
#define RSSMXV_VAL_DRM_CODE_RATE_R060                                       11
#define RSSMXV_VAL_DRM_CODE_RATE_R062                                       12
#define RSSMXV_VAL_DRM_CODE_RATE_R066                                       13
#define RSSMXV_VAL_DRM_CODE_RATE_R071                                       14
#define RSSMXV_VAL_DRM_CODE_RATE_R072                                       15
#define RSSMXV_VAL_DRM_CODE_RATE_R078                                       16

// RsSMxV_rngDRMCodeRateMSCProtectionProfile
#define RSSMXV_DRM_CODING_MSC_PROTECTION_PROFILE_INVALID                    0
#define RSSMXV_DRM_CODING_MSC_PROTECTION_PROFILE_HPP                        1
#define RSSMXV_DRM_CODING_MSC_PROTECTION_PROFILE_LPP                        2
#define RSSMXV_DRM_CODING_MSC_PROTECTION_PROFILE_VSPP                       3

// RsSMxV_rngDRMCodeRateMSCProtectionLevel
#define RSSMXV_VAL_DRM_CODING_MSC_PROTECTION_LEVELS_ZERO                    0
#define RSSMXV_VAL_DRM_CODING_MSC_PROTECTION_LEVELS_ONE                     1
#define RSSMXV_VAL_DRM_CODING_MSC_PROTECTION_LEVELS_TWO                     2
#define RSSMXV_VAL_DRM_CODING_MSC_PROTECTION_LEVELS_THREE                   3
#define RSSMXV_VAL_DRM_CODING_MSC_PROTECTION_LEVELS_INVALID                 4

// RsSMxV_rngATSCVSBFrequency
#define RSSMXV_VAL_ATSCMH_VSB_FREQUENCY_PILOT                               0
#define RSSMXV_VAL_ATSCMH_VSB_FREQUENCY_CENTER                              1

// RsSMxV_rngATSCMHInputFormat
#define RSSMXV_VAL_ATSCMH_INPUT_FORMAT_ASI                                  0
#define RSSMXV_VAL_ATSCMH_INPUT_FORMAT_SMPTE                                1

// RsSMxV_rngATSCMHPacketLength
#define RSSMXV_VAL_PACKET_LENGTH_INVALID                                    0
#define RSSMXV_VAL_PACKET_LENGTH_188                                        1
#define RSSMXV_VAL_PACKET_LENGTH_208                                        2

// RsSMxV_rngATSCMHTestSignal
#define RSSMXV_VAL_ATSCMH_TEST_SIGNAL_TEST_TS_PACKET                        0
#define RSSMXV_VAL_ATSCMH_TEST_SIGNAL_PRBS_INTERLEAVER                      1
#define RSSMXV_VAL_ATSCMH_TEST_SIGNAL_PRBS_TRELLIS                          2
#define RSSMXV_VAL_ATSCMH_TEST_SIGNAL_PRBS_MAPPER                           3

// RsSMxV_rngTestTSPacketStructure
#define RSSMXV_VAL_TS_PACKET_H184                                           0
#define RSSMXV_VAL_TS_PACKET_S187                                           1

// RsSMxV_rngTestPIDPacket
#define RSSMXV_VAL_PID_TEST_PACKET_NULL                                     0
#define RSSMXV_VAL_PID_TEST_PACKET_VARIABLE                                 1

// RsSMxV_rngTestSignalPayload
#define RSSMXV_VAL_TEST_SIGNAL_PAYLOAD_PRBS                                 0
#define RSSMXV_VAL_TEST_SIGNAL_PAYLOAD_HFF                                  1
#define RSSMXV_VAL_TEST_SIGNAL_PAYLOAD_H00                                  2

// RsSMxV_rngPRBSSequence
#define RSSMXV_VAL_PRBS_SEQUENCE_15                                         0
#define RSSMXV_VAL_PRBS_SEQUENCE_23                                         1

// RsSMxV_rngATSCMHConstellation
#define RSSMXV_VAL_ATSCMH_CONSTELLATION_VSB8                                0

// RsSMxV_rngATSCMHRollOff
#define RSSMXV_VAL_ATSCMH_ROLL_OFF_R115                                     0

// RsSMxV_rngATSCMHBuryRatio
#define RSSMXV_VAL_ATSCMH_BURY_RATIO_DB21                                   0
#define RSSMXV_VAL_ATSCMH_BURY_RATIO_DB24                                   1
#define RSSMXV_VAL_ATSCMH_BURY_RATIO_DB27                                   2
#define RSSMXV_VAL_ATSCMH_BURY_RATIO_DB30                                   3
#define RSSMXV_VAL_ATSCMH_BURY_RATIO_DB33                                   4
#define RSSMXV_VAL_ATSCMH_BURY_RATIO_DB36                                   5
#define RSSMXV_VAL_ATSCMH_BURY_RATIO_DB39                                   6

// RsSMxV_rngATSC3TestSignal
#define RSSMXV_VAL_ATSC3_TEST_SIGNAL_TS_PACKETS                             0
#define RSSMXV_VAL_ATSC3_TEST_SIGNAL_IP_PACKETS                             1

// RsSMxV_rngTestIPPacketStructure
#define RSSMXV_VAL_IP_PACKET_HUDP                                           0

// RsSMxV_rngATSC3PLPLDMLayer
#define RSSMXV_VAL_LAYER_DIVISION_MULTIPLEXING_CORE                         0
#define RSSMXV_VAL_LAYER_DIVISION_MULTIPLEXING_ENHANCED                     1

// RsSMxV_rngATSC3LDMInjectionLevel
#define RSSMXV_LDM_INJECTION_LEVEL_0_0DB                                    0
#define RSSMXV_LDM_INJECTION_LEVEL_0_5DB                                    1
#define RSSMXV_LDM_INJECTION_LEVEL_1_0DB                                    2
#define RSSMXV_LDM_INJECTION_LEVEL_1_5DB                                    3
#define RSSMXV_LDM_INJECTION_LEVEL_2_0DB                                    4
#define RSSMXV_LDM_INJECTION_LEVEL_2_5DB                                    5
#define RSSMXV_LDM_INJECTION_LEVEL_3_0DB                                    6
#define RSSMXV_LDM_INJECTION_LEVEL_3_5DB                                    7
#define RSSMXV_LDM_INJECTION_LEVEL_4_0DB                                    8
#define RSSMXV_LDM_INJECTION_LEVEL_4_5DB                                    9
#define RSSMXV_LDM_INJECTION_LEVEL_5_0DB                                    10
#define RSSMXV_LDM_INJECTION_LEVEL_6_0DB                                    11
#define RSSMXV_LDM_INJECTION_LEVEL_7_0DB                                    12
#define RSSMXV_LDM_INJECTION_LEVEL_8_0DB                                    13
#define RSSMXV_LDM_INJECTION_LEVEL_9_0DB                                    14
#define RSSMXV_LDM_INJECTION_LEVEL_10_0DB                                   15
#define RSSMXV_LDM_INJECTION_LEVEL_11_0DB                                   16
#define RSSMXV_LDM_INJECTION_LEVEL_12_0DB                                   17
#define RSSMXV_LDM_INJECTION_LEVEL_13_0DB                                   18
#define RSSMXV_LDM_INJECTION_LEVEL_14_0DB                                   19
#define RSSMXV_LDM_INJECTION_LEVEL_15_0DB                                   20
#define RSSMXV_LDM_INJECTION_LEVEL_16_0DB                                   21
#define RSSMXV_LDM_INJECTION_LEVEL_17_0DB                                   22
#define RSSMXV_LDM_INJECTION_LEVEL_18_0DB                                   23
#define RSSMXV_LDM_INJECTION_LEVEL_19_0DB                                   24
#define RSSMXV_LDM_INJECTION_LEVEL_20_0DB                                   25
#define RSSMXV_LDM_INJECTION_LEVEL_21_0DB                                   26
#define RSSMXV_LDM_INJECTION_LEVEL_22_0DB                                   27
#define RSSMXV_LDM_INJECTION_LEVEL_23_0DB                                   28
#define RSSMXV_LDM_INJECTION_LEVEL_24_0DB                                   29
#define RSSMXV_LDM_INJECTION_LEVEL_25_0DB                                   30

// RsSMxV_rngATSC3PLPFECType
#define RSSMXV_VAL_ATSC3_FORWARD_ERROR_CORRECTION_BCH_16K                   0
#define RSSMXV_VAL_ATSC3_FORWARD_ERROR_CORRECTION_BCH_64K                   1
#define RSSMXV_VAL_ATSC3_FORWARD_ERROR_CORRECTION_CRC_16K                   2
#define RSSMXV_VAL_ATSC3_FORWARD_ERROR_CORRECTION_CRC_64K                   3
#define RSSMXV_VAL_ATSC3_FORWARD_ERROR_CORRECTION_NONE_16K                  4
#define RSSMXV_VAL_ATSC3_FORWARD_ERROR_CORRECTION_NONE_64K                  5

// RsSMxV_rngATSC3PLPCodeRate
#define RSSMXV_VAL_ATSC3_PLP_CODE_RATE_R2_15                                2
#define RSSMXV_VAL_ATSC3_PLP_CODE_RATE_R3_15                                3
#define RSSMXV_VAL_ATSC3_PLP_CODE_RATE_R4_15                                4
#define RSSMXV_VAL_ATSC3_PLP_CODE_RATE_R5_15                                5
#define RSSMXV_VAL_ATSC3_PLP_CODE_RATE_R6_15                                6
#define RSSMXV_VAL_ATSC3_PLP_CODE_RATE_R7_15                                7
#define RSSMXV_VAL_ATSC3_PLP_CODE_RATE_R8_15                                8
#define RSSMXV_VAL_ATSC3_PLP_CODE_RATE_R9_15                                9
#define RSSMXV_VAL_ATSC3_PLP_CODE_RATE_R10_15                               10
#define RSSMXV_VAL_ATSC3_PLP_CODE_RATE_R11_15                               11
#define RSSMXV_VAL_ATSC3_PLP_CODE_RATE_R12_15                               12
#define RSSMXV_VAL_ATSC3_PLP_CODE_RATE_R13_15                               13

// RsSMxV_rngATSC3PLPConstellation
#define RSSMXV_VAL_PLP_CONSTELLATION_QPSK                                   0
#define RSSMXV_VAL_PLP_CONSTELLATION_16QAM                                  1
#define RSSMXV_VAL_PLP_CONSTELLATION_64QAM                                  2
#define RSSMXV_VAL_PLP_CONSTELLATION_256QAM                                 3
#define RSSMXV_VAL_PLP_CONSTELLATION_1024QAM                                4
#define RSSMXV_VAL_PLP_CONSTELLATION_4096QAM                                5

// RsSMxV_rngATSC3NetworkLowLevelSignaling
#define RSSMXV_VAL_ABSENT                                                   0
#define RSSMXV_VAL_PRESENT                                                  1

// RsSMxV_rngATSC3PLPInterleavingType
#define RSSMXV_VAL_ATSC3_PLP_INTERLEAVING_NONDISPERSED                      0
#define RSSMXV_VAL_ATSC3_PLP_INTERLEAVING_DISPERSED                         1

// RsSMxV_rngATSC3PLPTimeInterleaverMode
#define RSSMXV_VAL_ATSC3_TIME_INTERLEAVER_MODE_OFF                          0
#define RSSMXV_VAL_ATSC3_TIME_INTERLEAVER_MODE_CONVOLUTIONAL                1
#define RSSMXV_VAL_ATSC3_TIME_INTERLEAVER_MODE_HYBRID                       2

// RsSMxV_rngATSC3PLPInterleavingDepth
#define RSSMXV_VAL_ATSC3_INTERLEAVING_DEPTH_512                             0
#define RSSMXV_VAL_ATSC3_INTERLEAVING_DEPTH_724                             1
#define RSSMXV_VAL_ATSC3_INTERLEAVING_DEPTH_887                             2
#define RSSMXV_VAL_ATSC3_INTERLEAVING_DEPTH_1024                            3
#define RSSMXV_VAL_ATSC3_INTERLEAVING_DEPTH_1254                            4
#define RSSMXV_VAL_ATSC3_INTERLEAVING_DEPTH_1448                            5

// RsSMxV_rngATSC3OFDMChannelBandwidth
#define RSSMXV_VAL_ATSC3_OFDM_CHANNEL_BANDWIDTH_6MHZ                        6
#define RSSMXV_VAL_ATSC3_OFDM_CHANNEL_BANDWIDTH_7MHZ                        7
#define RSSMXV_VAL_ATSC3_OFDM_CHANNEL_BANDWIDTH_8MHZ                        8

// RsSMxV_rngATSC3OFDMMISO
#define RSSMXV_VAL_ATSC3_OFDM_MISO_OFF                                      0
#define RSSMXV_VAL_ATSC3_OFDM_MISO_64_COEFFICIENTS                          1
#define RSSMXV_VAL_ATSC3_OFDM_MISO_256_COEFFICIENTS                         2

// RsSMxV_rngATSC3FFTSize
#define RSSMXV_VAL_ATSC3_FFT_SIZE_8K                                        0
#define RSSMXV_VAL_ATSC3_FFT_SIZE_16K                                       1
#define RSSMXV_VAL_ATSC3_FFT_SIZE_32K                                       2

// RsSMxV_rngATSC3GuardInterval
#define RSSMXV_VAL_ATSC3_GUARD_INTERVAL_192                                 0
#define RSSMXV_VAL_ATSC3_GUARD_INTERVAL_384                                 1
#define RSSMXV_VAL_ATSC3_GUARD_INTERVAL_512                                 2
#define RSSMXV_VAL_ATSC3_GUARD_INTERVAL_768                                 3
#define RSSMXV_VAL_ATSC3_GUARD_INTERVAL_1024                                4
#define RSSMXV_VAL_ATSC3_GUARD_INTERVAL_1536                                5
#define RSSMXV_VAL_ATSC3_GUARD_INTERVAL_2048                                6
#define RSSMXV_VAL_ATSC3_GUARD_INTERVAL_2432                                7
#define RSSMXV_VAL_ATSC3_GUARD_INTERVAL_3072                                8
#define RSSMXV_VAL_ATSC3_GUARD_INTERVAL_3648                                9
#define RSSMXV_VAL_ATSC3_GUARD_INTERVAL_4096                                10
#define RSSMXV_VAL_ATSC3_GUARD_INTERVAL_4864                                11

// RsSMxV_rngATSC3OFDMSISOPilotPattern
#define RSSMXV_VAL_ATSC4_SISO_PILOT_PATTERN_3_2                             0
#define RSSMXV_VAL_ATSC4_SISO_PILOT_PATTERN_3_4                             1
#define RSSMXV_VAL_ATSC4_SISO_PILOT_PATTERN_4_2                             2
#define RSSMXV_VAL_ATSC4_SISO_PILOT_PATTERN_4_4                             3
#define RSSMXV_VAL_ATSC4_SISO_PILOT_PATTERN_6_2                             4
#define RSSMXV_VAL_ATSC4_SISO_PILOT_PATTERN_6_4                             5
#define RSSMXV_VAL_ATSC4_SISO_PILOT_PATTERN_8_2                             6
#define RSSMXV_VAL_ATSC4_SISO_PILOT_PATTERN_8_4                             7
#define RSSMXV_VAL_ATSC4_SISO_PILOT_PATTERN_12_2                            8
#define RSSMXV_VAL_ATSC4_SISO_PILOT_PATTERN_12_4                            9
#define RSSMXV_VAL_ATSC4_SISO_PILOT_PATTERN_16_2                            10
#define RSSMXV_VAL_ATSC4_SISO_PILOT_PATTERN_16_4                            11
#define RSSMXV_VAL_ATSC4_SISO_PILOT_PATTERN_24_2                            12
#define RSSMXV_VAL_ATSC4_SISO_PILOT_PATTERN_24_4                            13
#define RSSMXV_VAL_ATSC4_SISO_PILOT_PATTERN_32_2                            14
#define RSSMXV_VAL_ATSC4_SISO_PILOT_PATTERN_32_4                            15

// RsSMxV_rngATSC3NetworkTime
#define RSSMXV_VAL_ATSC3_NETWORK_TIME_OFF                                   0
#define RSSMXV_VAL_ATSC3_NETWORK_TIME_MILLISECONDS                          1
#define RSSMXV_VAL_ATSC3_NETWORK_TIME_MICROSECONDS                          2
#define RSSMXV_VAL_ATSC3_NETWORK_TIME_NANOSECONDS                           3

// RsSMxV_rngATSC3TxIDMode
#define RSSMXV_VAL_ATSC3_TXID_MODE_OFF                                      0
#define RSSMXV_VAL_ATSC3_TXID_MODE_MANUAL                                   1
#define RSSMXV_VAL_ATSC3_TXID_MODE_AUTO                                     2

// RsSMxV_rngATSC3TxIDInjectionLevel
#define RSSMXV_VAL_ATSC3_TXID_INJECTION_OFF                                 0
#define RSSMXV_VAL_ATSC3_TXID_INJECTION_9DB                                 1
#define RSSMXV_VAL_ATSC3_TXID_INJECTION_12DB                                2
#define RSSMXV_VAL_ATSC3_TXID_INJECTION_15DB                                3
#define RSSMXV_VAL_ATSC3_TXID_INJECTION_18DB                                4
#define RSSMXV_VAL_ATSC3_TXID_INJECTION_21DB                                5
#define RSSMXV_VAL_ATSC3_TXID_INJECTION_24DB                                6
#define RSSMXV_VAL_ATSC3_TXID_INJECTION_27DB                                7
#define RSSMXV_VAL_ATSC3_TXID_INJECTION_30DB                                8
#define RSSMXV_VAL_ATSC3_TXID_INJECTION_33DB                                9
#define RSSMXV_VAL_ATSC3_TXID_INJECTION_36DB                                10
#define RSSMXV_VAL_ATSC3_TXID_INJECTION_39DB                                11
#define RSSMXV_VAL_ATSC3_TXID_INJECTION_42DB                                12
#define RSSMXV_VAL_ATSC3_TXID_INJECTION_45DB                                13

// RsSMxV_rngATSC3PeakToAveragePowerRatio
#define RSSMXV_VAL_ATSC3_PEAK_TO_AVERAGE_RATIO_OFF                          0
#define RSSMXV_VAL_ATSC3_PEAK_TO_AVERAGE_RATIO_TR                           1

// RsSMxV_rngATSC3FrameMode
#define RSSMXV_VAL_ATSC3_FRAME_MODE_TIME                                    0
#define RSSMXV_VAL_ATSC3_FRAME_MODE_SYMBOL                                  1

// RsSMxV_rngATSC3BasicFECType
#define RSSMXV_VAL_ATSC3_FEC_MODE_1                                         1
#define RSSMXV_VAL_ATSC3_FEC_MODE_2                                         2
#define RSSMXV_VAL_ATSC3_FEC_MODE_3                                         3
#define RSSMXV_VAL_ATSC3_FEC_MODE_4                                         4
#define RSSMXV_VAL_ATSC3_FEC_MODE_5                                         5

// RsSMxV_rngATSC3DetailFECType
#define RSSMXV_VAL_ATSC3_FEC_MODE_6                                         6
#define RSSMXV_VAL_ATSC3_FEC_MODE_7                                         7

// RsSMxV_rngATSC3DetailAdditionalParityMode
#define RSSMXV_VAL_ATSC3_DETAIL_ADDITIONAL_PARITY_OFF                       0
#define RSSMXV_VAL_ATSC3_DETAIL_ADDITIONAL_PARITY_MODE_K1                   1
#define RSSMXV_VAL_ATSC3_DETAIL_ADDITIONAL_PARITY_MODE_K2                   2

// RsSMxV_rngATSC3PilotPatternPreamble
#define RSSMXV_VAL_ATSC3_PILOT_PATTERN_PREAMBLE_3                           0
#define RSSMXV_VAL_ATSC3_PILOT_PATTERN_PREAMBLE_4                           1
#define RSSMXV_VAL_ATSC3_PILOT_PATTERN_PREAMBLE_6                           2
#define RSSMXV_VAL_ATSC3_PILOT_PATTERN_PREAMBLE_8                           3
#define RSSMXV_VAL_ATSC3_PILOT_PATTERN_PREAMBLE_12                          4
#define RSSMXV_VAL_ATSC3_PILOT_PATTERN_PREAMBLE_16                          5
#define RSSMXV_VAL_ATSC3_PILOT_PATTERN_PREAMBLE_24                          6
#define RSSMXV_VAL_ATSC3_PILOT_PATTERN_PREAMBLE_32                          7

// RsSMxV_rngATSC3BootstrapMinTimeToNext
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_NOT_APPLICABLE          0
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N50                     1
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N100                    2
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N150                    3
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N200                    4
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N250                    5
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N300                    6
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N350                    7
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N400                    8
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N500                    9
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N600                    10
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N700                    11
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N800                    12
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N900                    13
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N1000                   14
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N1100                   15
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N1200                   16
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N1300                   17
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N1400                   18
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N1500                   19
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N1600                   20
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N1700                   21
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N1900                   22
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N2100                   23
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N2300                   24
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N2500                   25
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N2700                   26
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N2900                   27
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N3300                   28
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N3700                   29
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N4100                   30
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N4500                   31
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N4900                   32
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_MIN_TIME_TO_NEXT_N5300                   33

// RsSMxV_rngATSC3BootstrapEAS
#define RSSMXV_VAL_ATSC3_EMERGENCY_ALERT_NO_EMERGENCY                       0
#define RSSMXV_VAL_ATSC3_EMERGENCY_ALERT_SETTING_1                          1
#define RSSMXV_VAL_ATSC3_EMERGENCY_ALERT_SETTING_2                          2
#define RSSMXV_VAL_ATSC3_EMERGENCY_ALERT_SETTING_3                          3

// RsSMxV_rngATSC3BootstrapBandwidth
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_BANDWIDTH_6MHZ                           6
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_BANDWIDTH_7MHZ                           7
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_BANDWIDTH_8MHZ                           8
#define RSSMXV_VAL_ATSC3_BOOTSTRAP_BANDWIDTH_8GHZ                           9

// RsSMxV_rngATSC3SFNMode
#define RSSMXV_VAL_ATSC3_SFN_MODE_RELATIVE                                  0
#define RSSMXV_VAL_ATSC3_SFN_MODE_ABSOLUTE                                  1

// RsSMxV_rngDVBTInputSource
#define RSSMXV_VAL_DVBT_INPUT_SOURCE_TSGEN                                  0
#define RSSMXV_VAL_DVBT_INPUT_SOURCE_EXT                                    1
#define RSSMXV_VAL_DVBT_INPUT_SOURCE_TEST                                   2

// RsSMxV_rngDVBTInputFormat
#define RSSMXV_DVBT_INPUT_FORMAT_ASI                                        0
#define RSSMXV_DVBT_INPUT_FORMAT_SMPTE                                      1

// RsSMxV_rngDVBTTestSignal
#define RSSMXV_VAL_DVBT_TEST_SIGNAL_TTSP                                    0
#define RSSMXV_VAL_DVBT_TEST_SIGNAL_PBEC                                    1
#define RSSMXV_VAL_DVBT_TEST_SIGNAL_PAFC                                    2

// RsSMxV_rngDVBTPacketLength
#define RSSMXV_VAL_DVBT_PACKET_LENGTH_INVALID                               0
#define RSSMXV_VAL_DVBT_PACKET_LENGTH_188                                   1
#define RSSMXV_VAL_DVBT_PACKET_LENGTH_204                                   2
#define RSSMXV_VAL_DVBT_PACKET_LENGTH_208                                   3

// RsSMxV_rngDVBTHiearchy
#define RSSMXV_VAL_DVBT_HIERARCHY_NONHIER                                   0
#define RSSMXV_VAL_DVBT_HIERARCHY_A1                                        1
#define RSSMXV_VAL_DVBT_HIERARCHY_A2                                        2
#define RSSMXV_VAL_DVBT_HIERARCHY_A4                                        3

// RsSMxV_rngDVBTCodingConstellation
#define RSSMXV_VAL_DVBT_CODING_CONSTELLATION_T4                             0
#define RSSMXV_VAL_DVBT_CODING_CONSTELLATION_T16                            1
#define RSSMXV_VAL_DVBT_CODING_CONSTELLATION_T64                            2

// RsSMxV_rngDVBTCodingChannelBW
#define RSSMXV_VAL_DVBT_CODING_CHANNEL_BW_VAR                               0
#define RSSMXV_VAL_DVBT_CODING_CHANNEL_BW_5                                 1
#define RSSMXV_VAL_DVBT_CODING_CHANNEL_BW_6                                 2
#define RSSMXV_VAL_DVBT_CODING_CHANNEL_BW_7                                 3
#define RSSMXV_VAL_DVBT_CODING_CHANNEL_BW_8                                 4

// RsSMxV_rngDVBTCodingFFTMode
#define RSSMXC_VAL_DVBT_FFT_MODE_M2K                                        0
#define RSSMXC_VAL_DVBT_FFT_MODE_M4K                                        1
#define RSSMXC_VAL_DVBT_FFT_MODE_M8K                                        2

// RsSMxV_rngDVBTCodingGuardInterval
#define RSSMXV_VAL_DVBT_GUARD_INTERVAL_G1_4                                 0
#define RSSMXV_VAL_DVBT_GUARD_INTERVAL_G1_8                                 1
#define RSSMXV_VAL_DVBT_GUARD_INTERVAL_G1_16                                2
#define RSSMXV_VAL_DVBT_GUARD_INTERVAL_G1_32                                3

// RsSMxV_rngDVBTSymbolInterleaving
#define RSSMXV_VAL_SYMBOL_INTERLEAVING_INDEPTH                              0
#define RSSMXV_VAL_SYMBOL_INTERLEAVING_NATIVE                               1

// RsSMxV_rngDVBTCodeRate
#define RSSMXV_VAL_DVBT_CODE_RATE_R1_2                                      0
#define RSSMXV_VAL_DVBT_CODE_RATE_R2_3                                      1
#define RSSMXV_VAL_DVBT_CODE_RATE_R3_4                                      2
#define RSSMXV_VAL_DVBT_CODE_RATE_R5_6                                      3
#define RSSMXV_VAL_DVBT_CODE_RATE_R7_8                                      4

// RsSMxV_rngDVBT2InputFormat
#define RSSMXV_DVBT2_INPUT_FORMAT_ASI                                       0
#define RSSMXV_DVBT2_INPUT_FORMAT_SMPTE                                     1

// RsSMxV_rngDVBT2InputPLPTestSignal
#define RSSMXV_VAL_DVBT2_PLP_TEST_SIGNAL_TTSP                               0

// RsSMxV_rngDVBT2PLPInputFormat
#define RSSMXV_VAL_DVBT2_PLP_INPUT_FORMAT_GFPS                              0
#define RSSMXV_VAL_DVBT2_PLP_INPUT_FORMAT_GCS                               1
#define RSSMXV_VAL_DVBT2_PLP_INPUT_FORMAT_GSE                               2
#define RSSMXV_VAL_DVBT2_PLP_INPUT_FORMAT_TS                                3

// RsSMxV_rngDVBT2PacketLength
#define RSSMXV_VAL_DVBT2_PACKET_LENGTH_INVALID                              0
#define RSSMXV_VAL_DVBT2_PACKET_LENGTH_188                                  1
#define RSSMXV_VAL_DVBT2_PACKET_LENGTH_204                                  2
#define RSSMXV_VAL_DVBT2_PACKET_LENGTH_208                                  3

// RsSMxV_rngDVBT2T2MIMeasMode
#define RSSMEXV_VAL_DVBT2_INPUT_T2MI_MEAS_MODE_ABSOLUTE                     0
#define RSSMEXV_VAL_DVBT2_INPUT_T2MI_MEAS_MODE_DELTA                        1

// RsSMxV_rngDVBT2PLPType
#define RSSMXV_VAL_DVBT2_MODE_STREAM_PLP_TYPE_COMMON                        0
#define RSSMXV_VAL_DVBT2_MODE_STREAM_PLP_TYPE_DT1                           1
#define RSSMXV_VAL_DVBT2_MODE_STREAM_PLP_TYPE_DT2                           2

// RsSMxV_rngDVBT2PLPBBMode
#define RSSMEX_VAL_DVBT2_PLP_BB_MODE_HEM                                    0
#define RSSMEX_VAL_DVBT2_PLP_BB_MODE_NM                                     1

// RsSMxV_rngDVBT2PLPISSY
#define RSSMXV_VAL_DVBT2_PLP_ISSY_OFF                                       0
#define RSSMXV_VAL_DVBT2_PLP_ISSY_SHORT                                     1
#define RSSMXV_VAL_DVBT2_PLP_ISSY_LONG                                      2

// RsSMxV_rngDVBT2PLPCMType
#define RSSMXV_VAL_DVBT2_PLP_CM_TYPE_CCM                                    0
#define RSSMXV_VAL_DVBT2_PLP_CM_TYPE_ACM                                    1

// RsSMxV_rngDVBT2BCIMPLPFECFrame
#define RSSMXV_VAL_DVBT2_BCIM_PLP_FEC_FRAME_NORMAL                          0
#define RSSMXV_VAL_DVBT2_BCIM_PLP_FEC_FRAME_SHORT                           1

// RsSMxV_rngDVBT2CodeRate
#define RSSMXV_VAL_DVBT2_CODE_RATE_R1_2                                     0
#define RSSMXV_VAL_DVBT2_CODE_RATE_R2_3                                     1
#define RSSMXV_VAL_DVBT2_CODE_RATE_R3_4                                     2
#define RSSMXV_VAL_DVBT2_CODE_RATE_R5_6                                     3
#define RSSMXV_VAL_DVBT2_CODE_RATE_R1_3                                     4
#define RSSMXV_VAL_DVBT2_CODE_RATE_R3_5                                     5
#define RSSMXV_VAL_DVBT2_CODE_RATE_R2_5                                     6

// RsSMxV_rngDVBT2BCIMPLPConstellation
#define RSSMXV_VAL_DVBT2_BCIM_PLP_CONSTELLATION_T4                          0
#define RSSMXV_VAL_DVBT2_BCIM_PLP_CONSTELLATION_T16                         1
#define RSSMXV_VAL_DVBT2_BCIM_PLP_CONSTELLATION_T64                         2
#define RSSMXV_VAL_DVBT2_BCIM_PLP_CONSTELLATION_T256                        3

// RsSMxV_rngDVBT2OFDMChannelBW
#define RSSMXV_VAL_DVBT2_OFDM_CHAN_BW_2                                     0
#define RSSMXV_VAL_DVBT2_OFDM_CHAN_BW_5                                     1
#define RSSMXV_VAL_DVBT2_OFDM_CHAN_BW_6                                     2
#define RSSMXV_VAL_DVBT2_OFDM_CHAN_BW_7                                     3
#define RSSMXV_VAL_DVBT2_OFDM_CHAN_BW_8                                     4

// RsSMxV_rngDVBT2OFDMFFTMode
#define RSSMXV_VAL_DVBT2_OFDM_FFT_MODE_M1K                                  0
#define RSSMXV_VAL_DVBT2_OFDM_FFT_MODE_M2K                                  1
#define RSSMXV_VAL_DVBT2_OFDM_FFT_MODE_M4K                                  2
#define RSSMXV_VAL_DVBT2_OFDM_FFT_MODE_M8K                                  3
#define RSSMXV_VAL_DVBT2_OFDM_FFT_MODE_M16K                                 4
#define RSSMXV_VAL_DVBT2_OFDM_FFT_MODE_M8E                                  5
#define RSSMXV_VAL_DVBT2_OFDM_FFT_MODE_M16E                                 6
#define RSSMXV_VAL_DVBT2_OFDM_FFT_MODE_M32E                                 7

// RsSMxV_rngDVBT2CodingGuardInterval
#define RSSMXV_VAL_DVBT2_GUARD_INTERVAL_G1_4                                0
#define RSSMXV_VAL_DVBT2_GUARD_INTERVAL_G1_8                                1
#define RSSMXV_VAL_DVBT2_GUARD_INTERVAL_G1_16                               2
#define RSSMXV_VAL_DVBT2_GUARD_INTERVAL_G1_32                               3
#define RSSMXV_VAL_DVBT2_GUARD_INTERVAL_G1128                               4
#define RSSMXV_VAL_DVBT2_GUARD_INTERVAL_G19128                              5
#define RSSMXV_VAL_DVBT2_GUARD_INTERVAL_G19256                              6

// RsSMxV_rngDVBT2OFDMPilotPattern
#define RSSMXV_VAL_DVBT2_OFDM_PILOT_PATTERN_1                               1
#define RSSMXV_VAL_DVBT2_OFDM_PILOT_PATTERN_2                               2
#define RSSMXV_VAL_DVBT2_OFDM_PILOT_PATTERN_3                               3
#define RSSMXV_VAL_DVBT2_OFDM_PILOT_PATTERN_4                               4
#define RSSMXV_VAL_DVBT2_OFDM_PILOT_PATTERN_5                               5
#define RSSMXV_VAL_DVBT2_OFDM_PILOT_PATTERN_6                               6
#define RSSMXV_VAL_DVBT2_OFDM_PILOT_PATTERN_7                               7

// RsSMxV_rngDVBT2ProfileMode
#define RSSMXV_VAL_DVBT2_PROFILE_MODE_SINGLE                                0
#define RSSMXV_VAL_DVBT2_PROFILE_MODE_MULTI                                 1

// RsSMxV_rngDVBT2TransmissionSystem
#define RSSMXV_VAL_DVBT2_TRANSMISSION_SYSTEM_T2LM                           0
#define RSSMXV_VAL_DVBT2_TRANSMISSION_SYSTEM_T2LS                           1
#define RSSMXV_VAL_DVBT2_TRANSMISSION_SYSTEM_NONT2                          2
#define RSSMXV_VAL_DVBT2_TRANSMISSION_SYSTEM_MISO                           3
#define RSSMXV_VAL_DVBT2_TRANSMISSION_SYSTEM_SISO                           4

// RsSMxV_rngManualAuto
#define RSSMXV_VAL_AUTO                                                     0
#define RSSMXV_VAL_MAN                                                      1

// RsSMxV_rngDVBT2NetworkMISOGroup
#define RSSMXV_VAL_DVBT2_NETWORK_MISO_GROUP_1                               1
#define RSSMXV_VAL_DVBT2_NETWORK_MISO_GROUP_2                               2

// RsSMxV_rngDVBT2NetworkPAPR
#define RSSMXV_VAL_DVBT2_NETWORK_PAPR_OFF                                   0
#define RSSMXV_VAL_DVBT2_NETWORK_PAPR_TR                                    1

// RsSMxV_rngDVBT2NetworkFEFPayload
#define RSSMXV_VAL_DVBT2_NETWORK_FEF_PAYLOAD_NULL                           0
#define RSSMXV_VAL_DVBT2_NETWORK_FEF_PAYLOAD_NOISE                          1

// RsSMxV_rngDVBT2NetworkTFS
#define RSSMXV_VAL_DVBT2_NETWORK_TFS_OFF                                    0

// RsSMxV_rngDVBT2L1T2Version
#define RSSMXV_VAL_DVBT2_L1_T2_VERSION_111                                  0
#define RSSMXV_VAL_DVBT2_L1_T2_VERSION_121                                  1
#define RSSMXV_VAL_DVBT2_L1_T2_VERSION_131                                  2

// RsSMxV_rngDVBT2L1PostModulation
#define RSSMXV_VAL_DVBT2_L1_POST_MODULATION_BPSK                            0
#define RSSMXV_VAL_DVBT2_L1_POST_MODULATION_QPSK                            1
#define RSSMXV_VAL_DVBT2_L1_POST_MODULATION_QAM16                           2
#define RSSMXV_VAL_DVBT2_L1_POST_MODULATION_QAM64                           3

// RsSMxV_rngDVBT2L1PostExtension
#define RSSMXV_VAL_DVBT2_L1_POST_EXTENSION_OFF                              0

// RsSMxV_rngRelAbs
#define RSSMXV_VAL_REL                                                      0
#define RSSMXV_VAL_ABS                                                      1

// RsSMxV_rngISDBTControl
#define RSSMXV_VAL_ISDBT_CONTROL_AUTO                                       0
#define RSSMXV_VAL_ISDBT_CONTROL_MANUAL                                     1

// RsSMxV_rngISDBTSystem
#define RSSMXV_VAL_ISDBT_SYSTEM_T                                           0
#define RSSMXV_VAL_ISDBT_SYSTEM_TSB1                                        1
#define RSSMXV_VAL_ISDBT_SYSTEM_TSB3                                        2

// RsSMxV_rngISDBTInputSourceLayer
#define RSSMXV_VAL_ISDBT_INPUT_SOURCE_LAYER_EXTERNAL                        0
#define RSSMXV_VAL_ISDBT_INPUT_SOURCE_LAYER_TS_PLAYER                       1
#define RSSMXV_VAL_ISDBT_INPUT_SOURCE_LAYER_TEST_SIGNAL                     2

// RsSMxV_rngISDBTInputFormat
#define RSSMVX_VAL_ISDBT_INPUT_FORMAT_ASI                                   0
#define RSSMVX_VAL_ISDBT_INPUT_FORMAT_SMPTE                                 1

// RsSMxV_rngISDBTInfoPacketLength
#define RSSMXV_VAL_ISDBT_INPUT_INFO_PACKET_LENGTH_INVALID                   0
#define RSSMXV_VAL_ISDBT_INPUT_INFO_PACKET_LENGTH_P188                      1
#define RSSMXV_VAL_ISDBT_INPUT_INFO_PACKET_LENGTH_P204                      2
#define RSSMXV_VAL_ISDBT_INPUT_INFO_PACKET_LENGTH_P208                      3

// RsSMxV_rngISDBTTestSignalData
#define RSSMXV_VAL_ISDBT_TEST_SIGNAL_DATA_PAFC                              0
#define RSSMXV_VAL_ISDBT_TEST_SIGNAL_DATA_PBEC                              1
#define RSSMXV_VAL_ISDBT_TEST_SIGNAL_DATA_TTSP                              2

// RsSMxV_rngISDBTCodingMode
#define RSSMXV_VAL_ISDBT_CODING_MODE_M12K                                   0
#define RSSMXV_VAL_ISDBT_CODING_MODE_M24K                                   1
#define RSSMXV_VAL_ISDBT_CODING_MODE_M38K                                   2

// RsSMxV_rngISDBTCodingChannelBandwidth
#define RSSMXV_VAL_ISDBT_CODING_CHANNEL_BANDWIDTH_6MHZ                      0
#define RSSMXV_VAL_ISDBT_CODING_CHANNEL_BANDWIDTH_7MHZ                      1
#define RSSMXV_VAL_ISDBT_CODING_CHANNEL_BANDWIDTH_8MHZ                      2

// RsSMxV_rngISDBTCodingGuardInterval
#define RSSMXV_VAL_ISDBT_CODING_GUARD_INTERVAL_G14                          0
#define RSSMXV_VAL_ISDBT_CODING_GUARD_INTERVAL_G18                          1
#define RSSMXV_VAL_ISDBT_CODING_GUARD_INTERVAL_G116                         2
#define RSSMXV_VAL_ISDBT_CODING_GUARD_INTERVAL_G132                         3

// RsSMxV_rngISDBTCodingPortion
#define RSSMXV_VAL_ISDBT_CODING_PORTION_PCM                                 0
#define RSSMXV_VAL_ISDBT_CODING_PORTION_PDM                                 1
#define RSSMXV_VAL_ISDBT_CODING_PORTION_PRR                                 2

// RsSMxV_rngISDBTCodingConstellation
#define RSSMXV_VAL_ISDBT_CODING_CONSTELLATION_DQPSK                         0
#define RSSMXV_VAL_ISDBT_CODING_CONSTELLATION_QPSK                          1
#define RSSMXV_VAL_ISDBT_CODING_CONSTELLATION_16QAM                         2
#define RSSMXV_VAL_ISDBT_CODING_CONSTELLATION_64QAM                         3

// RsSMxV_rngISDBTCodingRate
#define RSSMXV_VAL_ISDBT_CODING_RATE_78                                     0
#define RSSMXV_VAL_ISDBT_CODING_RATE_56                                     1
#define RSSMXV_VAL_ISDBT_CODING_RATE_34                                     2
#define RSSMXV_VAL_ISDBT_CODING_RATE_23                                     3
#define RSSMXV_VAL_ISDBT_CODING_RATE_12                                     4

// RsSMxV_rngISDBTEEWSignalType
#define RSSMXV_VAL_ISDBT_EEW_SIGNAL_TYPE_WWA                                0
#define RSSMXV_VAL_ISDBT_EEW_SIGNAL_TYPE_WWOA                               1
#define RSSMXV_VAL_ISDBT_EEW_SIGNAL_TYPE_TWA                                2
#define RSSMXV_VAL_ISDBT_EEW_SIGNAL_TYPE_TWOA                               3

// RsSMxV_rngISDBTEEWInformationType
#define RSSMXV_VAL_ISDBT_EEW_INFORMATION_TYPE_CANCELED                      0
#define RSSMXV_VAL_ISDBT_EEW_INFORMATION_TYPE_ISSUED                        1

// RsSMxV_rngISDBTSpecialACData
#define RSSMXV_VAL_ISDBT_SPECIAL_AC_DATA_ALL1                               0
#define RSSMXV_VAL_ISDBT_SPECIAL_AC_DATA_PRBS                               1

// RsSMxV_rngISDBTSpecialTXParameterSwitchingIndicator
#define RSSMXV_VAL_ISDBT_SPECIAL_TX_INDICATOR_NORMAL                        0
#define RSSMXV_VAL_ISDBT_SPECIAL_TX_INDICATOR_N1                            1
#define RSSMXV_VAL_ISDBT_SPECIAL_TX_INDICATOR_N2                            2
#define RSSMXV_VAL_ISDBT_SPECIAL_TX_INDICATOR_N4                            4
#define RSSMXV_VAL_ISDBT_SPECIAL_TX_INDICATOR_N5                            5
#define RSSMXV_VAL_ISDBT_SPECIAL_TX_INDICATOR_N6                            6
#define RSSMXV_VAL_ISDBT_SPECIAL_TX_INDICATOR_N7                            7
#define RSSMXV_VAL_ISDBT_SPECIAL_TX_INDICATOR_N8                            8
#define RSSMXV_VAL_ISDBT_SPECIAL_TX_INDICATOR_N9                            9
#define RSSMXV_VAL_ISDBT_SPECIAL_TX_INDICATOR_N10                           10
#define RSSMXV_VAL_ISDBT_SPECIAL_TX_INDICATOR_N11                           11
#define RSSMXV_VAL_ISDBT_SPECIAL_TX_INDICATOR_N12                           12
#define RSSMXV_VAL_ISDBT_SPECIAL_TX_INDICATOR_N13                           13
#define RSSMXV_VAL_ISDBT_SPECIAL_TX_INDICATOR_N14                           14
#define RSSMXV_VAL_ISDBT_SPECIAL_TX_INDICATOR_N15                           15

// RsSMxV_rngISDBTSpecialTMCCNext
#define RSSMXV_VAL_ISDBT_SPECIAL_TMCC_NEXT_UNUSED                           0
#define RSSMXV_VAL_ISDBT_SPECIAL_TMCC_NEXT_CURRENT                          1

// RsSMxV_rngDTMBInputFormat
#define RSSMXV_VAL_DTMB_SIGNAL_INPUT_FORMAT_ASI                             0
#define RSSMXV_VAL_DTMB_SIGNAL_INPUT_FORMAT_SMPTE                           1

// RsSMxV_rngDTMBTestSignal
#define RSSMXV_VAL_DTMB_TEST_SIGNAL_TTSP                                    0

// RsSMxV_rngDTMBCodingChannelBandwidth
#define RSSMXV_VAL_DTMB_CODING_CHANNEL_BANDWIDTH_BW6                        0
#define RSSMXV_VAL_DTMB_CODING_CHANNEL_BANDWIDTH_BW7                        1
#define RSSMXV_VAL_DTMB_CODING_CHANNEL_BANDWIDTH_BW8                        2

// RsSMxV_rngDTMBCodingConstellation
#define RSSMXV_VAL_DTMB_CODING_CONSTELLATION_D4                             0
#define RSSMXV_VAL_DTMB_CODING_CONSTELLATION_D4NR                           1
#define RSSMXV_VAL_DTMB_CODING_CONSTELLATION_D16                            2
#define RSSMXV_VAL_DTMB_CODING_CONSTELLATION_D32                            3
#define RSSMXV_VAL_DTMB_CODING_CONSTELLATION_D64                            4

// RsSMxV_rngDTMBCodingCodeRate
#define RSSMXV_VAL_DTMB_CODING_CODE_RATE_R04                                0
#define RSSMXV_VAL_DTMB_CODING_CODE_RATE_R06                                1
#define RSSMXV_VAL_DTMB_CODING_CODE_RATE_R08                                2

// RsSMxV_rngDTMBCodingGuardInterval
#define RSSMXV_VAL_DTMB_CODING_GUARD_INTERVAL_G420                          0
#define RSSMXV_VAL_DTMB_CODING_GUARD_INTERVAL_G595                          1
#define RSSMXV_VAL_DTMB_CODING_GUARD_INTERVAL_G945                          2

// RsSMxV_rngDTMBCodingGuardIntervalPN
#define RSSMXV_VAL_DTMB_CODING_GUARD_INTERVAL_PN_VAR                        0
#define RSSMXV_VAL_DTMB_CODING_GUARD_INTERVAL_PN_CONST                      1

// RsSMxV_rngDTMBCodingTimeInterleaver
#define RSSMXV_VAL_DTMB_CODING_TIME_INTERLEAVER_OFF                         0
#define RSSMXV_VAL_DTMB_CODING_TIME_INTERLEAVER_I240                        1
#define RSSMXV_VAL_DTMB_CODING_TIME_INTERLEAVER_I720                        2

// RsSMxV_rngDVBSInputFormat
#define RSSMXV_VAL_DVBS_INPUT_FORMAT_ASI                                    0
#define RSSMXV_VAL_DVBS_INPUT_FORMAT_SMPTE                                  1

// RsSMxV_rngDVBSTestSignal
#define RSSMXV_VAL_DVBS_TEST_SIGNAL_TTSP                                    0
#define RSSMXV_VAL_DVBS_TEST_SIGNAL_PBEC                                    1

// RsSMxV_rngDVBSCodingConstellation
#define RSSMXV_VAL_DVBS_CODING_CONSTELLATION_S4                             0
#define RSSMXV_VAL_DVBS_CODING_CONSTELLATION_S8                             1
#define RSSMXV_VAL_DVBS_CODING_CONSTELLATION_S16                            2

// RsSMxV_rngDVBSCodingCodeRate
#define RSSMXV_VAL_DVBS_CODE_RATE_1_2                                       0
#define RSSMXV_VAL_DVBS_CODE_RATE_2_3                                       1
#define RSSMXV_VAL_DVBS_CODE_RATE_3_4                                       2
#define RSSMXV_VAL_DVBS_CODE_RATE_5_6                                       3
#define RSSMXV_VAL_DVBS_CODE_RATE_7_8                                       4
#define RSSMXV_VAL_DVBS_CODE_RATE_8_9                                       5

// RsSMxV_rngDVBS2CMMode
#define RSSMXV_VAL_DVBS2_CM_MODE_CCM                                        0
#define RSSMXV_VAL_DVBS2_CM_MODE_ACM                                        1
#define RSSMXV_VAL_DVBS2_CM_MODE_VCM                                        2

// RsSMxV_rngDVBS2InputFormat
#define RSSMXV_VAL_DVBS2_INPUT_FORMAT_ASI                                   0
#define RSSMXV_VAL_DVBS2_INPUT_FORMAT_SMPTE                                 1

// RsSMxV_rngDVBS2PacketLength
#define RSSMXV_VAL_DVBS2_PACKET_LENGTH_INVALID                              0
#define RSSMXV_VAL_DVBS2_PACKET_LENGTH_188                                  1
#define RSSMXV_VAL_DVBS2_PACKET_LENGTH_204                                  2
#define RSSMXV_VAL_DVBS2_PACKET_LENGTH_208                                  3

// RsSMxV_rngDVBS2TestSignal
#define RSSMXV_VAL_DVBS2_TEST_SIGNAL_TTSP                                   0
#define RSSMXV_VAL_DVBS2_TEST_SIGNAL_TGSP                                   1

// RsSMxV_rngDVBS2ModulationCoding
#define RSSMXV_VAL_DVBS2_MOD_COD_QPSK_1_4_CLASSICAL                         1
#define RSSMXV_VAL_DVBS2_MOD_COD_QPSK_1_3_CLASSICAL                         2
#define RSSMXV_VAL_DVBS2_MOD_COD_QPSK_2_5_CLASSICAL                         3
#define RSSMXV_VAL_DVBS2_MOD_COD_QPSK_1_2_CLASSICAL                         4
#define RSSMXV_VAL_DVBS2_MOD_COD_QPSK_3_5_CLASSICAL                         5
#define RSSMXV_VAL_DVBS2_MOD_COD_QPSK_2_3_CLASSICAL                         6
#define RSSMXV_VAL_DVBS2_MOD_COD_QPSK_3_4_CLASSICAL                         7
#define RSSMXV_VAL_DVBS2_MOD_COD_QPSK_4_5_CLASSICAL                         8
#define RSSMXV_VAL_DVBS2_MOD_COD_QPSK_5_6_CLASSICAL                         9
#define RSSMXV_VAL_DVBS2_MOD_COD_QPSK_8_9_CLASSICAL                         10
#define RSSMXV_VAL_DVBS2_MOD_COD_QPSK_9_10_CLASSICAL                        11
#define RSSMXV_VAL_DVBS2_MOD_COD_8PSK_3_5_CLASSICAL                         12
#define RSSMXV_VAL_DVBS2_MOD_COD_8PSK_2_3_CLASSICAL                         13
#define RSSMXV_VAL_DVBS2_MOD_COD_8PSK_3_4_CLASSICAL                         14
#define RSSMXV_VAL_DVBS2_MOD_COD_8PSK_5_6_CLASSICAL                         15
#define RSSMXV_VAL_DVBS2_MOD_COD_8PSK_8_9_CLASSICAL                         16
#define RSSMXV_VAL_DVBS2_MOD_COD_8PSK_9_10_CLASSICAL                        17
#define RSSMXV_VAL_DVBS2_MOD_COD_16APSK_2_3_CLASSICAL                       18
#define RSSMXV_VAL_DVBS2_MOD_COD_16APSK_3_4_CLASSICAL                       19
#define RSSMXV_VAL_DVBS2_MOD_COD_16APSK_4_5_CLASSICAL                       20
#define RSSMXV_VAL_DVBS2_MOD_COD_16APSK_5_6_CLASSICAL                       21
#define RSSMXV_VAL_DVBS2_MOD_COD_16APSK_8_9_CLASSICAL                       22
#define RSSMXV_VAL_DVBS2_MOD_COD_16APSK_9_10_CLASSICAL                      23
#define RSSMXV_VAL_DVBS2_MOD_COD_32APSK_3_4_CLASSICAL                       24
#define RSSMXV_VAL_DVBS2_MOD_COD_32APSK_4_5_CLASSICAL                       25
#define RSSMXV_VAL_DVBS2_MOD_COD_32APSK_5_6_CLASSICAL                       26
#define RSSMXV_VAL_DVBS2_MOD_COD_32APSK_8_9_CLASSICAL                       27
#define RSSMXV_VAL_DVBS2_MOD_COD_32APSK_9_10_CLASSICAL                      28
#define RSSMXV_VAL_DVBS2_MOD_COD_QPSK_13_45_SX                              29
#define RSSMXV_VAL_DVBS2_MOD_COD_QPSK_9_20_SX                               30
#define RSSMXV_VAL_DVBS2_MOD_COD_QPSK_11_20_SX                              31
#define RSSMXV_VAL_DVBS2_MOD_COD_8APSK_5_9_L_SX                             32
#define RSSMXV_VAL_DVBS2_MOD_COD_8APSK_26_45_L_SX                           33
#define RSSMXV_VAL_DVBS2_MOD_COD_8PSK_23_36_SX                              34
#define RSSMXV_VAL_DVBS2_MOD_COD_8PSK_25_36_SX                              35
#define RSSMXV_VAL_DVBS2_MOD_COD_8PSK_13_18_SX                              36
#define RSSMXV_VAL_DVBS2_MOD_COD_16APSK_1_2_L_SX                            37
#define RSSMXV_VAL_DVBS2_MOD_COD_16APSK_8_15_L_SX                           38
#define RSSMXV_VAL_DVBS2_MOD_COD_16APSK_5_9_L_SX                            39
#define RSSMXV_VAL_DVBS2_MOD_COD_16APSK_26_45_SX                            40
#define RSSMXV_VAL_DVBS2_MOD_COD_16APSK_3_5_SX                              41
#define RSSMXV_VAL_DVBS2_MOD_COD_16APSK_3_5_L_SX                            42
#define RSSMXV_VAL_DVBS2_MOD_COD_16APSK_28_45_SX                            43
#define RSSMXV_VAL_DVBS2_MOD_COD_16APSK_23_36_SX                            44
#define RSSMXV_VAL_DVBS2_MOD_COD_16APSK_2_3_L_SX                            45
#define RSSMXV_VAL_DVBS2_MOD_COD_16APSK_25_36_SX                            46
#define RSSMXV_VAL_DVBS2_MOD_COD_16APSK_13_18_SX                            47
#define RSSMXV_VAL_DVBS2_MOD_COD_16APSK_7_9_SX                              48
#define RSSMXV_VAL_DVBS2_MOD_COD_16APSK_77_90_SX                            49
#define RSSMXV_VAL_DVBS2_MOD_COD_32APSK_2_3_L_SX                            50
#define RSSMXV_VAL_DVBS2_MOD_COD_32APSK_32_45_SX                            51
#define RSSMXV_VAL_DVBS2_MOD_COD_32APSK_11_15_SX                            52
#define RSSMXV_VAL_DVBS2_MOD_COD_32APSK_7_9_SX                              53
#define RSSMXV_VAL_DVBS2_MOD_COD_64APSK_32_45_L_SX                          54
#define RSSMXV_VAL_DVBS2_MOD_COD_64APSK_11_15_SX                            55
#define RSSMXV_VAL_DVBS2_MOD_COD_64APSK_7_9_SX                              56
#define RSSMXV_VAL_DVBS2_MOD_COD_64APSK_4_5_SX                              57
#define RSSMXV_VAL_DVBS2_MOD_COD_64APSK_5_6_SX                              58
#define RSSMXV_VAL_DVBS2_MOD_COD_128APSK_3_4_SX                             59
#define RSSMXV_VAL_DVBS2_MOD_COD_128APSK_7_9_SX                             60
#define RSSMXV_VAL_DVBS2_MOD_COD_256APSK_29_45_L_SX                         61
#define RSSMXV_VAL_DVBS2_MOD_COD_256APSK_2_3_L_SX                           62
#define RSSMXV_VAL_DVBS2_MOD_COD_256APSK_31_45_L_SX                         63
#define RSSMXV_VAL_DVBS2_MOD_COD_256APSK_32_45_SX                           64
#define RSSMXV_VAL_DVBS2_MOD_COD_256APSK_11_15_L_SX                         65
#define RSSMXV_VAL_DVBS2_MOD_COD_256APSK_3_4_SX                             66
#define RSSMXV_VAL_DVBS2_MOD_COD_QPSK_11_45_SX_SHORT                        67
#define RSSMXV_VAL_DVBS2_MOD_COD_QPSK_4_15_SX_SHORT                         68
#define RSSMXV_VAL_DVBS2_MOD_COD_QPSK_14_45_SX_SHORT                        69
#define RSSMXV_VAL_DVBS2_MOD_COD_QPSK_7_15_SX_SHORT                         70
#define RSSMXV_VAL_DVBS2_MOD_COD_QPSK_8_15_SX_SHORT                         71
#define RSSMXV_VAL_DVBS2_MOD_COD_QPSK_32_45_SX_SHORT                        72
#define RSSMXV_VAL_DVBS2_MOD_COD_8PSK_7_15_SX_SHORT                         73
#define RSSMXV_VAL_DVBS2_MOD_COD_8PSK_8_15_SX_SHORT                         74
#define RSSMXV_VAL_DVBS2_MOD_COD_8PSK_26_45_SX_SHORT                        75
#define RSSMXV_VAL_DVBS2_MOD_COD_8PSK_32_45_SX_SHORT                        76
#define RSSMXV_VAL_DVBS2_MOD_COD_16APSK_7_15_SX_SHORT                       77
#define RSSMXV_VAL_DVBS2_MOD_COD_16APSK_8_15_SX_SHORT                       78
#define RSSMXV_VAL_DVBS2_MOD_COD_16APSK_26_45_SX_SHORT                      79
#define RSSMXV_VAL_DVBS2_MOD_COD_16APSK_3_5_SX_SHORT                        80
#define RSSMXV_VAL_DVBS2_MOD_COD_16APSK_32_45_SX_SHORT                      81
#define RSSMXV_VAL_DVBS2_MOD_COD_32APSK_2_3_SX_SHORT                        82
#define RSSMXV_VAL_DVBS2_MOD_COD_32APSK_32_45_SX_SHORT                      83
#define RSSMXV_VAL_DVBS2_MOD_COD_VL1_QPSK_2_9_SX_VL_SNR                     84
#define RSSMXV_VAL_DVBS2_MOD_COD_VL1_BPSK_1_5_SX_VL_SNR                     85
#define RSSMXV_VAL_DVBS2_MOD_COD_VL1_BPSK_11_45_SX_VL_SNR                   86
#define RSSMXV_VAL_DVBS2_MOD_COD_VL1_BPSK_1_3_SX_VL_SNR                     87
#define RSSMXV_VAL_DVBS2_MOD_COD_VL1_BPSK_S_1_5_SX_VL_SNR                   88
#define RSSMXV_VAL_DVBS2_MOD_COD_VL1_BPSK_S_11_45_SX_VL_SNR                 89
#define RSSMXV_VAL_DVBS2_MOD_COD_VL2_BPSK_1_5_SX_VL_SNR                     90
#define RSSMXV_VAL_DVBS2_MOD_COD_VL2_BPSK_4_15_SX_VL_SNR                    91
#define RSSMXV_VAL_DVBS2_MOD_COD_VL2_BPSK_1_3_SX_VL_SNR                     92

// RsSMxV_rngDVBS2FECFrame
#define RSSMXV_VAL_DVBS2_FEC_FRAME_NORMAL                                   0
#define RSSMXV_VAL_DVBS2_FEC_FRAME_MEDIUM                                   1
#define RSSMXV_VAL_DVBS2_FEC_FRAME_SHORT                                    2

// RsSMxV_rngIPChannelType
#define RSSMXV_VAL_IP_CHANNEL_UNICAST                                       0
#define RSSMXV_VAL_IP_CHANNEL_MULTICAST                                     1

// RsSMxV_rngTSPlayerPacketLength
#define RSSMXV_VAL_TS_PLAYER_PACKET_LENGTH_INV                              0
#define RSSMXV_VAL_TS_PLAYER_PACKET_LENGTH_188                              1
#define RSSMXV_VAL_TS_PLAYER_PACKET_LENGTH_204                              2
#define RSSMXV_VAL_TS_PLAYER_PACKET_LENGTH_208                              3

// RsSMxV_rngTSPlayerStopData
#define RSSMXV_VAL_TS_PLAYER_STOP_DATA_NONE                                 0
#define RSSMXV_VAL_TS_PLAYER_STOP_DATA_TTSP                                 1

// RsSMxV_rngTSPlayerTestPacket
#define RSSMXV_VAL_TS_PLAYER_TEST_PACKET_H184                               0
#define RSSMXV_VAL_TS_PLAYER_TEST_PACKET_S187                               1
#define RSSMXV_VAL_TS_PLAYER_TEST_PACKET_H200                               2
#define RSSMXV_VAL_TS_PLAYER_TEST_PACKET_S203                               3
#define RSSMXV_VAL_TS_PLAYER_TEST_PACKET_H204                               4
#define RSSMXV_VAL_TS_PLAYER_TEST_PACKET_S207                               5

// RsSMxV_rngIPDataNetworkAddressMode
#define RSSMXV_VAL_IP_DATA_NETWORK_ADDRESS_DHCP                             0
#define RSSMXV_VAL_IP_DATA_NETWORK_ADDRESS_STATIC                           1

// RsSMxV_rngIPProtocol
#define RSSMXV_VAL_IP_PROTOCOL_UDP                                          0
#define RSSMXV_VAL_IP_PROTOCOL_TCP                                          1

// RsSMxV_rngGraphicsAdd
#define RSSMXV_VAL_GRAPHICS_ADD_MAX                                         0
#define RSSMXV_VAL_GRAPHICS_ADD_MIN                                         1

// RsSMxV_rngARBMultisegmentAppendAction
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT0                  0
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT1                  1
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT2                  2
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT3                  3
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT4                  4
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT5                  5
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT6                  6
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT7                  7
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT8                  8
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT9                  9
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT10                 10
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT11                 11
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT12                 12
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT13                 13
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT14                 14
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT15                 15
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT16                 16
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT17                 17
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT18                 18
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT19                 19
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT20                 20
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT21                 21
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT22                 22
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT23                 23
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT24                 24
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT25                 25
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT26                 26
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT27                 27
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT28                 28
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT29                 29
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT30                 30
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_SEGMENT31                 31
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_NEXT                      32
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_BLANK                     33
#define RSSMXV_VAL_ARB_MULTISEGMENT_APPEND_ACTION_ENDLESS                   34

// RsSMxV_rngALCMode
#define RSSMXV_VAL_ALC_MODE_OFF                                             0
#define RSSMXV_VAL_ALC_MODE_ON                                              1
#define RSSMXV_VAL_ALC_MODE_AUTO                                            2
#define RSSMXV_VAL_ALC_MODE_PRESET                                          3
#define RSSMXV_VAL_ALC_MODE_OFFT                                            4
#define RSSMXV_VAL_ALC_MODE_ONS                                             5
#define RSSMXV_VAL_ALC_MODE_ONT                                             6

// RsSMxV_rngMultiInstrumentMode
#define RSSMXV_VAL_MULTI_INSTRUMENT_MODE_PRIMARY                            0
#define RSSMXV_VAL_MULTI_INSTRUMENT_MODE_SECONDARY                          1

// RsSMxV_rngDisplayPriority
#define RSSMXV_VAL_DISPLAY_PRIORITY_AVERAGE                                 0
#define RSSMXV_VAL_DISPLAY_PRIORITY_PEAK                                    1

// RsSMxV_rngSystemConfigurationMultiInstrumentTriggerSync
#define RSSMXV_VAL_SYSTEM_CONFIGURATION_MULTI_INSTRUMENT_TRIGGER_NOT_SYNCED 0
#define RSSMXV_VAL_SYSTEM_CONFIGURATION_MULTI_INSTRUMENT_TRIGGER_SYNCED     1

// RsSMxV_rngTDMBDABSpecialPRBSSequence
#define RSSMXV_VAL_TDMB_DAB_SEQUENCE_ZERO                                   0
#define RSSMXV_VAL_TDMB_DAB_SEQUENCE_PRBS_15                                1
#define RSSMXV_VAL_TDMB_DAB_SEQUENCE_PRBS_20                                2
#define RSSMXV_VAL_TDMB_DAB_SEQUENCE_PRBS_23                                3

// RsSMxV_rngDVBCConstellation
#define RSSMXV_VAL_DVBC_CONSTELLATION_16_QAM                                0
#define RSSMXV_VAL_DVBC_CONSTELLATION_32_QAM                                1
#define RSSMXV_VAL_DVBC_CONSTELLATION_64_QAM                                2
#define RSSMXV_VAL_DVBC_CONSTELLATION_128_QAM                               3
#define RSSMXV_VAL_DVBC_CONSTELLATION_256_QAM                               4

// RsSMxV_rngInputSignalInterface
#define RSSMXV_VAL_INPUT_SIGNAL_INTERFACE_TS                                0
#define RSSMXV_VAL_INPUT_SIGNAL_INTERFACE_IP                                1

// RsSMxV_rngInputSignalFormat
#define RSSMXV_VAL_INPUT_SIGNAL_FORMAT_ASI                                  0
#define RSSMXV_VAL_INPUT_SIGNAL_FORMAT_SMPTE                                1

// RsSMxV_rngDVBCRollOffFactor
#define RSSMXV_VAL_DVBC_ROLL_OFF_FACTOR_0_13                                0
#define RSSMXV_VAL_DVBC_ROLL_OFF_FACTOR_0_15                                1

// RsSMxV_rngInputSignalSource
#define RSSMXV_VAL_INPUT_SIGNAL_SOURCE_EXTERNAL                             0
#define RSSMXV_VAL_INPUT_SIGNAL_SOURCE_TS_PLAYER                            1
#define RSSMXV_VAL_INPUT_SIGNAL_SOURCE_TEST_SIGNAL                          2

// RsSMxV_rngTestSignal
#define RSSMXV_VAL_TEST_SIGNAL_TTSP                                         0
#define RSSMXV_VAL_TEST_SIGNAL_PBDE                                         1
#define RSSMXV_VAL_TEST_SIGNAL_PBEM                                         2

// RsSMxV_rngDVBCPacketLength
#define RSSMXV_VAL_DVBC_PACKET_LENGTH_INVALID                               0
#define RSSMXV_VAL_DVBC_PACKET_LENGTH_188_BYTES                             1
#define RSSMXV_VAL_DVBC_PACKET_LENGTH_204_BYTES                             2

// RsSMxV_rngJ83BCodingConstellation
#define RSSMXV_VAL_J83B_CODING_CONSTELLATION_64_QAM                         0
#define RSSMXV_VAL_J83B_CODING_CONSTELLATION_256_QAM                        1
#define RSSMXV_VAL_J83B_CODING_CONSTELLATION_1024_QAM                       2

// RsSMxV_rngJ83BCodingInterleaverMode
#define RSSMXV_VAL_J83B_CODING_INTERLEAVER_MODE_0                           0
#define RSSMXV_VAL_J83B_CODING_INTERLEAVER_MODE_1                           1
#define RSSMXV_VAL_J83B_CODING_INTERLEAVER_MODE_2                           2
#define RSSMXV_VAL_J83B_CODING_INTERLEAVER_MODE_3                           3
#define RSSMXV_VAL_J83B_CODING_INTERLEAVER_MODE_4                           4
#define RSSMXV_VAL_J83B_CODING_INTERLEAVER_MODE_5                           5
#define RSSMXV_VAL_J83B_CODING_INTERLEAVER_MODE_6                           6
#define RSSMXV_VAL_J83B_CODING_INTERLEAVER_MODE_7                           7
#define RSSMXV_VAL_J83B_CODING_INTERLEAVER_MODE_8                           8
#define RSSMXV_VAL_J83B_CODING_INTERLEAVER_MODE_9                           9
#define RSSMXV_VAL_J83B_CODING_INTERLEAVER_MODE_10                          10
#define RSSMXV_VAL_J83B_CODING_INTERLEAVER_MODE_12                          12
#define RSSMXV_VAL_J83B_CODING_INTERLEAVER_MODE_14                          14

// RsSMxV_rngJ83BPacketLength
#define RSSMXV_VAL_J83B_PACKET_LENGTH_INVALID                               0
#define RSSMXV_VAL_J83B_PACKET_LENGTH_P188                                  1

// RsSMxV_rngJ83BRollOffFactor
#define RSSMXV_VAL_J83B_ROLL_OFF_FACTOR_0_12                                0
#define RSSMXV_VAL_J83B_ROLL_OFF_FACTOR_0_18                                1

// RsSMxV_rngTimeProtocol
#define RSSMXV_VAL_TIME_PROTOCOL_NONE                                       0
#define RSSMXV_VAL_TIME_PROTOCOL_NTP                                        1

// RsSMxV_rngPowerSensorControlMode
#define RSSMXV_VAL_POWER_SENSOR_CONTROL_MODE_AUTO                           0
#define RSSMXV_VAL_POWER_SENSOR_CONTROL_MODE_SINGLE                         1

// RsSMxV_rngPowerSensors
#define RSSMXV_VAL_POWER_SENSORS_1                                          1
#define RSSMXV_VAL_POWER_SENSORS_2                                          2
#define RSSMXV_VAL_POWER_SENSORS_3                                          3
#define RSSMXV_VAL_POWER_SENSORS_4                                          4

// RsSMxV_rngHUMSDataFormat
#define RSSMXV_VAL_HUMS_DATA_FORMAT_JSON                                    0
#define RSSMXV_VAL_HUMS_DATA_FORMAT_XML                                     1

// RsSMxV_rngSNMPAccess
#define RSSMXV_VAL_SNMP_ACCESS_RO                                           0
#define RSSMXV_VAL_SNMP_ACCESS_RW                                           1

// RsSMxV_rngSNMPLevel
#define RSSMXV_VAL_SNMP_LEVEL_NO_AUTHENTICATION                             0
#define RSSMXV_VAL_SNMP_LEVEL_AUTHENTICATION                                1
#define RSSMXV_VAL_SNMP_LEVEL_AUTHENTICATION_AND_PRIVACY                    2

// RsSMxV_rngHUMSSNMPVersion
#define RSSMXV_VAL_HUMSSNMP_VERSION_OFF                                     0
#define RSSMXV_VAL_HUMSSNMP_VERSION_V12                                     1
#define RSSMXV_VAL_HUMSSNMP_VERSION_V123                                    2
#define RSSMXV_VAL_HUMSSNMP_VERSION_V3                                      3

//repcap Path
#define RSSMXV_REPCAP_PATH_A                                                1

//repcap Marker
#define RSSMXV_REPCAP_MARKER_M1                                             1
#define RSSMXV_REPCAP_MARKER_M2                                             2
#define RSSMXV_REPCAP_MARKER_M3                                             3
#define RSSMXV_REPCAP_MARKER_M4                                             4

//repcap DMSymbol
#define RSSMXV_REPCAP_DMSYMBOL_SYMBOL0                                      0
#define RSSMXV_REPCAP_DMSYMBOL_SYMBOL1                                      1
#define RSSMXV_REPCAP_DMSYMBOL_SYMBOL2                                      2
#define RSSMXV_REPCAP_DMSYMBOL_SYMBOL3                                      3
#define RSSMXV_REPCAP_DMSYMBOL_SYMBOL4                                      4
#define RSSMXV_REPCAP_DMSYMBOL_SYMBOL5                                      5
#define RSSMXV_REPCAP_DMSYMBOL_SYMBOL6                                      6
#define RSSMXV_REPCAP_DMSYMBOL_SYMBOL7                                      7
#define RSSMXV_REPCAP_DMSYMBOL_SYMBOL8                                      8
#define RSSMXV_REPCAP_DMSYMBOL_SYMBOL9                                      9
#define RSSMXV_REPCAP_DMSYMBOL_SYMBOL10                                     10
#define RSSMXV_REPCAP_DMSYMBOL_SYMBOL11                                     11
#define RSSMXV_REPCAP_DMSYMBOL_SYMBOL12                                     12
#define RSSMXV_REPCAP_DMSYMBOL_SYMBOL13                                     13
#define RSSMXV_REPCAP_DMSYMBOL_SYMBOL14                                     14
#define RSSMXV_REPCAP_DMSYMBOL_SYMBOL15                                     15

//repcap ARBCarrier
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER0                                   0
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER1                                   1
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER2                                   2
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER3                                   3
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER4                                   4
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER5                                   5
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER6                                   6
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER7                                   7
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER8                                   8
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER9                                   9
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER10                                  10
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER11                                  11
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER12                                  12
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER13                                  13
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER14                                  14
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER15                                  15
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER16                                  16
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER17                                  17
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER18                                  18
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER19                                  19
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER20                                  20
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER21                                  21
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER22                                  22
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER23                                  23
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER24                                  24
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER25                                  25
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER26                                  26
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER27                                  27
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER28                                  28
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER29                                  29
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER30                                  30
#define RSSMXV_REPCAP_ARBCARRIER_CARRIER31                                  31

//repcap Stream
#define RSSMXV_REPCAP_STREAM_STREAM1                                        1
#define RSSMXV_REPCAP_STREAM_STREAM2                                        2
#define RSSMXV_REPCAP_STREAM_STREAM3                                        3
#define RSSMXV_REPCAP_STREAM_STREAM4                                        4
#define RSSMXV_REPCAP_STREAM_STREAM5                                        5
#define RSSMXV_REPCAP_STREAM_STREAM6                                        6
#define RSSMXV_REPCAP_STREAM_STREAM7                                        7
#define RSSMXV_REPCAP_STREAM_STREAM8                                        8

//repcap HSDigChannel
#define RSSMXV_REPCAP_HSDIGCHANNEL_HSCHAN0                                  0
#define RSSMXV_REPCAP_HSDIGCHANNEL_HSCHAN1                                  1
#define RSSMXV_REPCAP_HSDIGCHANNEL_HSCHAN2                                  2
#define RSSMXV_REPCAP_HSDIGCHANNEL_HSCHAN3                                  3
#define RSSMXV_REPCAP_HSDIGCHANNEL_HSCHAN4                                  4
#define RSSMXV_REPCAP_HSDIGCHANNEL_HSCHAN5                                  5
#define RSSMXV_REPCAP_HSDIGCHANNEL_HSCHAN6                                  6
#define RSSMXV_REPCAP_HSDIGCHANNEL_HSCHAN7                                  7

//repcap HSD
#define RSSMXV_REPCAP_HSD_HSD1                                              1
#define RSSMXV_REPCAP_HSD_HSD2                                              2

//repcap BBINChannel
#define RSSMXV_REPCAP_BBINCHANNEL_BCH0                                      0
#define RSSMXV_REPCAP_BBINCHANNEL_BCH1                                      1
#define RSSMXV_REPCAP_BBINCHANNEL_BCH2                                      2
#define RSSMXV_REPCAP_BBINCHANNEL_BCH3                                      3
#define RSSMXV_REPCAP_BBINCHANNEL_BCH4                                      4
#define RSSMXV_REPCAP_BBINCHANNEL_BCH5                                      5
#define RSSMXV_REPCAP_BBINCHANNEL_BCH6                                      6
#define RSSMXV_REPCAP_BBINCHANNEL_BCH7                                      7

//repcap Sense
#define RSSMXV_REPCAP_SENSE_SENSE1                                          1
#define RSSMXV_REPCAP_SENSE_SENSE2                                          2
#define RSSMXV_REPCAP_SENSE_SENSE3                                          3
#define RSSMXV_REPCAP_SENSE_SENSE4                                          4

//repcap Connector
#define RSSMXV_REPCAP_CONNECTOR_C1                                          1
#define RSSMXV_REPCAP_CONNECTOR_C2                                          2

//repcap StereoChannel
#define RSSMXV_REPCAP_STEREOCHANNEL_RIGHT                                   1
#define RSSMXV_REPCAP_STEREOCHANNEL_LEFT                                    2

//repcap AFAFreq
#define RSSMXV_REPCAP_AFAFREQ_AFAFREQ1                                      1
#define RSSMXV_REPCAP_AFAFREQ_AFAFREQ2                                      2
#define RSSMXV_REPCAP_AFAFREQ_AFAFREQ3                                      3
#define RSSMXV_REPCAP_AFAFREQ_AFAFREQ4                                      4
#define RSSMXV_REPCAP_AFAFREQ_AFAFREQ5                                      5
#define RSSMXV_REPCAP_AFAFREQ_AFAFREQ6                                      6
#define RSSMXV_REPCAP_AFAFREQ_AFAFREQ7                                      7
#define RSSMXV_REPCAP_AFAFREQ_AFAFREQ8                                      8
#define RSSMXV_REPCAP_AFAFREQ_AFAFREQ9                                      9
#define RSSMXV_REPCAP_AFAFREQ_AFAFREQ10                                     10
#define RSSMXV_REPCAP_AFAFREQ_AFAFREQ11                                     11
#define RSSMXV_REPCAP_AFAFREQ_AFAFREQ12                                     12
#define RSSMXV_REPCAP_AFAFREQ_AFAFREQ13                                     13
#define RSSMXV_REPCAP_AFAFREQ_AFAFREQ14                                     14
#define RSSMXV_REPCAP_AFAFREQ_AFAFREQ15                                     15
#define RSSMXV_REPCAP_AFAFREQ_AFAFREQ16                                     16
#define RSSMXV_REPCAP_AFAFREQ_AFAFREQ17                                     17
#define RSSMXV_REPCAP_AFAFREQ_AFAFREQ18                                     18
#define RSSMXV_REPCAP_AFAFREQ_AFAFREQ19                                     19
#define RSSMXV_REPCAP_AFAFREQ_AFAFREQ20                                     20
#define RSSMXV_REPCAP_AFAFREQ_AFAFREQ21                                     21
#define RSSMXV_REPCAP_AFAFREQ_AFAFREQ22                                     22
#define RSSMXV_REPCAP_AFAFREQ_AFAFREQ23                                     23
#define RSSMXV_REPCAP_AFAFREQ_AFAFREQ24                                     24
#define RSSMXV_REPCAP_AFAFREQ_AFAFREQ25                                     25

//repcap AFBList
#define RSSMXV_REPCAP_AFBLIST_AFBLIST1                                      1
#define RSSMXV_REPCAP_AFBLIST_AFBLIST2                                      2
#define RSSMXV_REPCAP_AFBLIST_AFBLIST3                                      3
#define RSSMXV_REPCAP_AFBLIST_AFALIST4                                      4
#define RSSMXV_REPCAP_AFBLIST_AFBLIST5                                      5

//repcap AFBFreq
#define RSSMXV_REPCAP_AFBFREQ_AFBFREQ1                                      1
#define RSSMXV_REPCAP_AFBFREQ_AFBFREQ2                                      2
#define RSSMXV_REPCAP_AFBFREQ_AFBFREQ3                                      3
#define RSSMXV_REPCAP_AFBFREQ_AFBFREQ4                                      4
#define RSSMXV_REPCAP_AFBFREQ_AFBFREQ5                                      5
#define RSSMXV_REPCAP_AFBFREQ_AFBFREQ6                                      6
#define RSSMXV_REPCAP_AFBFREQ_AFBFREQ7                                      7
#define RSSMXV_REPCAP_AFBFREQ_AFBFREQ8                                      8
#define RSSMXV_REPCAP_AFBFREQ_AFBFREQ9                                      9
#define RSSMXV_REPCAP_AFBFREQ_AFBFREQ10                                     10
#define RSSMXV_REPCAP_AFBFREQ_AFBFREQ11                                     11
#define RSSMXV_REPCAP_AFBFREQ_AFBFREQ12                                     12

//repcap AFMapFreq
#define RSSMXV_REPCAP_AFMAPFREQ_AFMAPFREQ1                                  1
#define RSSMXV_REPCAP_AFMAPFREQ_AFMAPFREQ2                                  2
#define RSSMXV_REPCAP_AFMAPFREQ_AFMAPFREQ3                                  3
#define RSSMXV_REPCAP_AFMAPFREQ_AFMAPFREQ4                                  4
#define RSSMXV_REPCAP_AFMAPFREQ_AFMAPFREQ5                                  5
#define RSSMXV_REPCAP_AFMAPFREQ_AFMAPFREQ6                                  6
#define RSSMXV_REPCAP_AFMAPFREQ_AFMAPFREQ7                                  7
#define RSSMXV_REPCAP_AFMAPFREQ_AFMAPFREQ8                                  8
#define RSSMXV_REPCAP_AFMAPFREQ_AFMAPFREQ9                                  9
#define RSSMXV_REPCAP_AFMAPFREQ_AFMAPFREQ10                                 10
#define RSSMXV_REPCAP_AFMAPFREQ_AFMAPFREQ11                                 11
#define RSSMXV_REPCAP_AFMAPFREQ_AFMAPFREQ12                                 12
#define RSSMXV_REPCAP_AFMAPFREQ_AFMAPFREQ13                                 13
#define RSSMXV_REPCAP_AFMAPFREQ_AFMAPFREQ14                                 14
#define RSSMXV_REPCAP_AFMAPFREQ_AFMAPFREQ15                                 15
#define RSSMXV_REPCAP_AFMAPFREQ_AFMAPFREQ16                                 16
#define RSSMXV_REPCAP_AFMAPFREQ_AFMAPFREQ17                                 17
#define RSSMXV_REPCAP_AFMAPFREQ_AFMAPFREQ18                                 18
#define RSSMXV_REPCAP_AFMAPFREQ_AFMAPFREQ19                                 19
#define RSSMXV_REPCAP_AFMAPFREQ_AFMAPFREQ20                                 20
#define RSSMXV_REPCAP_AFMAPFREQ_AFMAPFREQ21                                 21
#define RSSMXV_REPCAP_AFMAPFREQ_AFMAPFREQ22                                 22
#define RSSMXV_REPCAP_AFMAPFREQ_AFMAPFREQ23                                 23
#define RSSMXV_REPCAP_AFMAPFREQ_AFMAPFREQ24                                 24
#define RSSMXV_REPCAP_AFMAPFREQ_AFMAPFREQ25                                 25

//repcap TMCGroup3AVar
#define RSSMXV_REPCAP_TMCGROUP3AVAR_G3AVAR00                                1
#define RSSMXV_REPCAP_TMCGROUP3AVAR_G3AVAR01                                2

//repcap TMCGroup8A
#define RSSMXV_REPCAP_TMCGROUP8A_G8A1                                       1
#define RSSMXV_REPCAP_TMCGROUP8A_G8A2                                       2
#define RSSMXV_REPCAP_TMCGROUP8A_G8A3                                       3
#define RSSMXV_REPCAP_TMCGROUP8A_G8A4                                       4
#define RSSMXV_REPCAP_TMCGROUP8A_G8A5                                       5
#define RSSMXV_REPCAP_TMCGROUP8A_G8A6                                       6

//repcap DarcBIC
#define RSSMXV_REPCAP_DARCBIC_BIC1                                          1
#define RSSMXV_REPCAP_DARCBIC_BIC2                                          2
#define RSSMXV_REPCAP_DARCBIC_BIC3                                          3

//repcap ETIStream
#define RSSMXV_REPCAP_ETISTREAM_ETI1                                        1
#define RSSMXV_REPCAP_ETISTREAM_ETI2                                        2
#define RSSMXV_REPCAP_ETISTREAM_ETI3                                        3
#define RSSMXV_REPCAP_ETISTREAM_ETI4                                        4
#define RSSMXV_REPCAP_ETISTREAM_ETI5                                        5
#define RSSMXV_REPCAP_ETISTREAM_ETI6                                        6
#define RSSMXV_REPCAP_ETISTREAM_ETI7                                        7
#define RSSMXV_REPCAP_ETISTREAM_ETI8                                        8
#define RSSMXV_REPCAP_ETISTREAM_ETI9                                        9
#define RSSMXV_REPCAP_ETISTREAM_ETI10                                       10
#define RSSMXV_REPCAP_ETISTREAM_ETI11                                       11
#define RSSMXV_REPCAP_ETISTREAM_ETI12                                       12
#define RSSMXV_REPCAP_ETISTREAM_ETI13                                       13
#define RSSMXV_REPCAP_ETISTREAM_ETI14                                       14
#define RSSMXV_REPCAP_ETISTREAM_ETI15                                       15
#define RSSMXV_REPCAP_ETISTREAM_ETI16                                       16
#define RSSMXV_REPCAP_ETISTREAM_ETI17                                       17
#define RSSMXV_REPCAP_ETISTREAM_ETI18                                       18
#define RSSMXV_REPCAP_ETISTREAM_ETI19                                       19
#define RSSMXV_REPCAP_ETISTREAM_ETI20                                       20
#define RSSMXV_REPCAP_ETISTREAM_ETI21                                       21
#define RSSMXV_REPCAP_ETISTREAM_ETI22                                       22
#define RSSMXV_REPCAP_ETISTREAM_ETI23                                       23
#define RSSMXV_REPCAP_ETISTREAM_ETI24                                       24
#define RSSMXV_REPCAP_ETISTREAM_ETI25                                       25
#define RSSMXV_REPCAP_ETISTREAM_ETI26                                       26
#define RSSMXV_REPCAP_ETISTREAM_ETI27                                       27
#define RSSMXV_REPCAP_ETISTREAM_ETI28                                       28
#define RSSMXV_REPCAP_ETISTREAM_ETI29                                       29
#define RSSMXV_REPCAP_ETISTREAM_ETI30                                       30
#define RSSMXV_REPCAP_ETISTREAM_ETI31                                       31
#define RSSMXV_REPCAP_ETISTREAM_ETI32                                       32
#define RSSMXV_REPCAP_ETISTREAM_ETI33                                       33
#define RSSMXV_REPCAP_ETISTREAM_ETI34                                       34
#define RSSMXV_REPCAP_ETISTREAM_ETI35                                       35
#define RSSMXV_REPCAP_ETISTREAM_ETI36                                       36
#define RSSMXV_REPCAP_ETISTREAM_ETI37                                       37
#define RSSMXV_REPCAP_ETISTREAM_ETI38                                       38
#define RSSMXV_REPCAP_ETISTREAM_ETI39                                       39
#define RSSMXV_REPCAP_ETISTREAM_ETI40                                       40
#define RSSMXV_REPCAP_ETISTREAM_ETI41                                       41
#define RSSMXV_REPCAP_ETISTREAM_ETI42                                       42
#define RSSMXV_REPCAP_ETISTREAM_ETI43                                       43
#define RSSMXV_REPCAP_ETISTREAM_ETI44                                       44
#define RSSMXV_REPCAP_ETISTREAM_ETI45                                       45
#define RSSMXV_REPCAP_ETISTREAM_ETI46                                       46
#define RSSMXV_REPCAP_ETISTREAM_ETI47                                       47
#define RSSMXV_REPCAP_ETISTREAM_ETI48                                       48
#define RSSMXV_REPCAP_ETISTREAM_ETI49                                       49
#define RSSMXV_REPCAP_ETISTREAM_ETI50                                       50
#define RSSMXV_REPCAP_ETISTREAM_ETI51                                       51
#define RSSMXV_REPCAP_ETISTREAM_ETI52                                       52
#define RSSMXV_REPCAP_ETISTREAM_ETI53                                       53
#define RSSMXV_REPCAP_ETISTREAM_ETI54                                       54
#define RSSMXV_REPCAP_ETISTREAM_ETI55                                       55
#define RSSMXV_REPCAP_ETISTREAM_ETI56                                       56
#define RSSMXV_REPCAP_ETISTREAM_ETI57                                       57
#define RSSMXV_REPCAP_ETISTREAM_ETI58                                       58
#define RSSMXV_REPCAP_ETISTREAM_ETI59                                       59
#define RSSMXV_REPCAP_ETISTREAM_ETI60                                       60
#define RSSMXV_REPCAP_ETISTREAM_ETI61                                       61
#define RSSMXV_REPCAP_ETISTREAM_ETI62                                       62
#define RSSMXV_REPCAP_ETISTREAM_ETI63                                       63
#define RSSMXV_REPCAP_ETISTREAM_ETI64                                       64

//repcap SDCChannel
#define RSSMXV_REPCAP_SDCCHANNEL_SDC1                                       1

//repcap MSCChannel
#define RSSMXV_REPCAP_MSCCHANNEL_MSC1                                       1
#define RSSMXV_REPCAP_MSCCHANNEL_MSC2                                       2
#define RSSMXV_REPCAP_MSCCHANNEL_MSC3                                       3

//repcap PLP
#define RSSMXV_REPCAP_PLP_PLP1                                              1
#define RSSMXV_REPCAP_PLP_PLP2                                              2
#define RSSMXV_REPCAP_PLP_PLP3                                              3
#define RSSMXV_REPCAP_PLP_PLP4                                              4
#define RSSMXV_REPCAP_PLP_PLP5                                              5
#define RSSMXV_REPCAP_PLP_PLP6                                              6
#define RSSMXV_REPCAP_PLP_PLP7                                              7
#define RSSMXV_REPCAP_PLP_PLP8                                              8
#define RSSMXV_REPCAP_PLP_PLP9                                              9
#define RSSMXV_REPCAP_PLP_PLP10                                             10
#define RSSMXV_REPCAP_PLP_PLP11                                             11
#define RSSMXV_REPCAP_PLP_PLP12                                             12
#define RSSMXV_REPCAP_PLP_PLP13                                             13
#define RSSMXV_REPCAP_PLP_PLP14                                             14
#define RSSMXV_REPCAP_PLP_PLP15                                             15
#define RSSMXV_REPCAP_PLP_PLP16                                             16
#define RSSMXV_REPCAP_PLP_PLP17                                             17
#define RSSMXV_REPCAP_PLP_PLP18                                             18
#define RSSMXV_REPCAP_PLP_PLP19                                             19
#define RSSMXV_REPCAP_PLP_PLP20                                             20
#define RSSMXV_REPCAP_PLP_PLP21                                             21
#define RSSMXV_REPCAP_PLP_PLP22                                             22
#define RSSMXV_REPCAP_PLP_PLP23                                             23
#define RSSMXV_REPCAP_PLP_PLP24                                             24
#define RSSMXV_REPCAP_PLP_PLP25                                             25
#define RSSMXV_REPCAP_PLP_PLP26                                             26
#define RSSMXV_REPCAP_PLP_PLP27                                             27
#define RSSMXV_REPCAP_PLP_PLP28                                             28
#define RSSMXV_REPCAP_PLP_PLP29                                             29
#define RSSMXV_REPCAP_PLP_PLP30                                             30
#define RSSMXV_REPCAP_PLP_PLP31                                             31
#define RSSMXV_REPCAP_PLP_PLP32                                             32
#define RSSMXV_REPCAP_PLP_PLP33                                             33
#define RSSMXV_REPCAP_PLP_PLP34                                             34
#define RSSMXV_REPCAP_PLP_PLP35                                             35
#define RSSMXV_REPCAP_PLP_PLP36                                             36
#define RSSMXV_REPCAP_PLP_PLP37                                             37
#define RSSMXV_REPCAP_PLP_PLP38                                             38
#define RSSMXV_REPCAP_PLP_PLP39                                             39
#define RSSMXV_REPCAP_PLP_PLP40                                             40
#define RSSMXV_REPCAP_PLP_PLP41                                             41
#define RSSMXV_REPCAP_PLP_PLP42                                             42
#define RSSMXV_REPCAP_PLP_PLP43                                             43
#define RSSMXV_REPCAP_PLP_PLP44                                             44
#define RSSMXV_REPCAP_PLP_PLP45                                             45
#define RSSMXV_REPCAP_PLP_PLP46                                             46
#define RSSMXV_REPCAP_PLP_PLP47                                             47
#define RSSMXV_REPCAP_PLP_PLP48                                             48
#define RSSMXV_REPCAP_PLP_PLP49                                             49
#define RSSMXV_REPCAP_PLP_PLP50                                             50
#define RSSMXV_REPCAP_PLP_PLP51                                             51
#define RSSMXV_REPCAP_PLP_PLP52                                             52
#define RSSMXV_REPCAP_PLP_PLP53                                             53
#define RSSMXV_REPCAP_PLP_PLP54                                             54
#define RSSMXV_REPCAP_PLP_PLP55                                             55
#define RSSMXV_REPCAP_PLP_PLP56                                             56
#define RSSMXV_REPCAP_PLP_PLP57                                             57
#define RSSMXV_REPCAP_PLP_PLP58                                             58
#define RSSMXV_REPCAP_PLP_PLP59                                             59
#define RSSMXV_REPCAP_PLP_PLP60                                             60
#define RSSMXV_REPCAP_PLP_PLP61                                             61
#define RSSMXV_REPCAP_PLP_PLP62                                             62
#define RSSMXV_REPCAP_PLP_PLP63                                             63
#define RSSMXV_REPCAP_PLP_PLP64                                             64

//repcap ATSC3Subframe
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME1                          1
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME2                          2
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME3                          3
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME4                          4
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME5                          5
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME6                          6
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME7                          7
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME8                          8
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME9                          9
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME10                         10
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME11                         11
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME12                         12
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME13                         13
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME14                         14
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME15                         15
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME16                         16
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME17                         17
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME18                         18
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME19                         19
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME20                         20
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME21                         21
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME22                         22
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME23                         23
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME24                         24
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME25                         25
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME26                         26
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME27                         27
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME28                         28
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME29                         29
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME30                         30
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME31                         31
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME32                         32
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME33                         33
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME34                         34
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME35                         35
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME36                         36
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME37                         37
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME38                         38
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME39                         39
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME40                         40
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME41                         41
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME42                         42
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME43                         43
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME44                         44
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME45                         45
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME46                         46
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME47                         47
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME48                         48
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME49                         49
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME50                         50
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME51                         51
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME52                         52
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME53                         53
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME54                         54
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME55                         55
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME56                         56
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME57                         57
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME58                         58
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME59                         59
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME60                         60
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME61                         61
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME62                         62
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME63                         63
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME64                         64
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME65                         65
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME66                         66
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME67                         67
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME68                         68
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME69                         69
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME70                         70
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME71                         71
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME72                         72
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME73                         73
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME74                         74
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME75                         75
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME76                         76
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME77                         77
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME78                         78
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME79                         79
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME80                         80
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME81                         81
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME82                         82
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME83                         83
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME84                         84
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME85                         85
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME86                         86
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME87                         87
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME88                         88
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME89                         89
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME90                         90
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME91                         91
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME92                         92
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME93                         93
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME94                         94
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME95                         95
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME96                         96
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME97                         97
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME98                         98
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME99                         99
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME100                        100
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME101                        101
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME102                        102
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME103                        103
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME104                        104
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME105                        105
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME106                        106
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME107                        107
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME108                        108
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME109                        109
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME110                        110
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME111                        111
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME112                        112
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME113                        113
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME114                        114
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME115                        115
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME116                        116
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME117                        117
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME118                        118
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME119                        119
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME120                        120
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME121                        121
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME122                        122
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME123                        123
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME124                        124
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME125                        125
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME126                        126
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME127                        127
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME128                        128
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME129                        129
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME130                        130
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME131                        131
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME132                        132
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME133                        133
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME134                        134
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME135                        135
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME136                        136
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME137                        137
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME138                        138
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME139                        139
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME140                        140
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME141                        141
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME142                        142
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME143                        143
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME144                        144
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME145                        145
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME146                        146
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME147                        147
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME148                        148
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME149                        149
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME150                        150
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME151                        151
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME152                        152
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME153                        153
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME154                        154
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME155                        155
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME156                        156
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME157                        157
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME158                        158
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME159                        159
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME160                        160
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME161                        161
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME162                        162
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME163                        163
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME164                        164
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME165                        165
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME166                        166
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME167                        167
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME168                        168
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME169                        169
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME170                        170
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME171                        171
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME172                        172
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME173                        173
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME174                        174
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME175                        175
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME176                        176
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME177                        177
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME178                        178
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME179                        179
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME180                        180
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME181                        181
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME182                        182
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME183                        183
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME184                        184
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME185                        185
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME186                        186
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME187                        187
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME188                        188
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME189                        189
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME190                        190
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME191                        191
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME192                        192
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME193                        193
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME194                        194
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME195                        195
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME196                        196
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME197                        197
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME198                        198
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME199                        199
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME200                        200
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME201                        201
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME202                        202
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME203                        203
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME204                        204
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME205                        205
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME206                        206
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME207                        207
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME208                        208
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME209                        209
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME210                        210
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME211                        211
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME212                        212
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME213                        213
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME214                        214
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME215                        215
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME216                        216
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME217                        217
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME218                        218
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME219                        219
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME220                        220
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME221                        221
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME222                        222
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME223                        223
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME224                        224
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME225                        225
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME226                        226
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME227                        227
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME228                        228
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME229                        229
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME230                        230
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME231                        231
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME232                        232
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME233                        233
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME234                        234
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME235                        235
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME236                        236
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME237                        237
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME238                        238
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME239                        239
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME240                        240
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME241                        241
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME242                        242
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME243                        243
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME244                        244
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME245                        245
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME246                        246
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME247                        247
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME248                        248
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME249                        249
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME250                        250
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME251                        251
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME252                        252
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME253                        253
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME254                        254
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME255                        255
#define RSSMXV_REPCAP_ATSC3SUBFRAME_ATSC3SUBFRAME256                        256

//repcap DVBT2PLP
#define RSSMXV_REPCAP_DVBT2PLP_DVBT2PLP0                                    0

//repcap DVBT2Timing
#define RSSMXV_REPCAP_DVBT2TIMING_DVBT2TIMING1                              1
#define RSSMXV_REPCAP_DVBT2TIMING_DVBT2TIMING2                              2
#define RSSMXV_REPCAP_DVBT2TIMING_DVBT2TIMING3                              3
#define RSSMXV_REPCAP_DVBT2TIMING_DVBT2TIMING4                              4

//repcap ISDBTEpicenter
#define RSSMXV_REPCAP_ISDBTEPICENTER_EPC1                                   1
#define RSSMXV_REPCAP_ISDBTEPICENTER_EPC2                                   2

//repcap DVBS2InputStream
#define RSSMXV_REPCAP_DVBS2INPUTSTREAM_DVBS2IS1                             1
#define RSSMXV_REPCAP_DVBS2INPUTSTREAM_DVBS2IS2                             2
#define RSSMXV_REPCAP_DVBS2INPUTSTREAM_DVBS2IS3                             3
#define RSSMXV_REPCAP_DVBS2INPUTSTREAM_DVBS2IS4                             4
#define RSSMXV_REPCAP_DVBS2INPUTSTREAM_DVBS2IS5                             5
#define RSSMXV_REPCAP_DVBS2INPUTSTREAM_DVBS2IS6                             6
#define RSSMXV_REPCAP_DVBS2INPUTSTREAM_DVBS2IS7                             7
#define RSSMXV_REPCAP_DVBS2INPUTSTREAM_DVBS2IS8                             8

//repcap DVBS2TimeSlice
#define RSSMXV_REPCAP_DVBS2TIMESLICE_DVBS2TS1                               1
#define RSSMXV_REPCAP_DVBS2TIMESLICE_DVBS2TS2                               2
#define RSSMXV_REPCAP_DVBS2TIMESLICE_DVBS2TS3                               3
#define RSSMXV_REPCAP_DVBS2TIMESLICE_DVBS2TS4                               4
#define RSSMXV_REPCAP_DVBS2TIMESLICE_DVBS2TS5                               5
#define RSSMXV_REPCAP_DVBS2TIMESLICE_DVBS2TS6                               6
#define RSSMXV_REPCAP_DVBS2TIMESLICE_DVBS2TS7                               7
#define RSSMXV_REPCAP_DVBS2TIMESLICE_DVBS2TS8                               8

//repcap IPChannel
#define RSSMXV_REPCAP_IPCHANNEL_IP1                                         1
#define RSSMXV_REPCAP_IPCHANNEL_IP2                                         2
#define RSSMXV_REPCAP_IPCHANNEL_IP3                                         3
#define RSSMXV_REPCAP_IPCHANNEL_IP4                                         4

//repcap IPInterface
#define RSSMXV_REPCAP_IPINTERFACE_LAN1                                      1
/* END GENERATE */

/*****************************************************************************
 *- Hidden Attribute Declarations -------------------------------------------*
 *****************************************************************************/

#define RSSMXV_ATTR_OPC_TIMEOUT               (RS_ATTR_OPC_TIMEOUT)           /* ViInt32 */
#define RSSMXV_ATTR_IO_SESSION                (RS_ATTR_IO_SESSION)            /* ViSession */
#define RSSMXV_ATTR_OPC_CALLBACK              (RS_ATTR_OPC_CALLBACK)          /* ViAddr */
#define RSSMXV_ATTR_CHECK_STATUS_CALLBACK     (RS_ATTR_CHECK_STATUS_CALLBACK) /* ViAddr */

/****************************************************************************
 *---------------- Constant definition  ------------------------------------*
 ****************************************************************************/
// Path RepCap
#define RSSMXV_VAL_PATH_A                                                1

#define RSSMXV_VAL_STAT_REG_PRESET                                      0
#define RSSMXV_VAL_STAT_REG_QUESTIONABLE                                1
#define RSSMXV_VAL_STAT_REG_OPERATION                                   2

#define RSSMXV_VAL_STAT_REG_MAIN                                        0

#define RSSMXV_VAL_STAT_REG_QUERY_COND                                  0
#define RSSMXV_VAL_STAT_REG_QUERY_EVENT                                 1

/****************************************************************************
 *---------------- Instrument Driver Function Declarations -----------------*
 ****************************************************************************/
ViStatus _VI_FUNC rssmxv_init(ViRsrc     resourceName,
                              ViBoolean  IDQuery,
                              ViBoolean  resetDevice,
                              ViSession* instrSession);

ViStatus _VI_FUNC rssmxv_InitWithOptions(ViRsrc     resourceName,
                                         ViBoolean  IDQuery,
                                         ViBoolean  resetDevice,
                                         ViString   optionString,
                                         ViSession* instrSession);

ViStatus _VI_FUNC rssmxv_FileDirectory(ViSession instrSession,
                                       ViString  directory,
                                       ViInt32   bufferSize,
                                       ViChar    output[]);

ViStatus _VI_FUNC rssmxv_FileCount(ViSession instrSession,
                                   ViString  directory,
                                   ViInt32*  numberOfFilesAndFolders);

ViStatus _VI_FUNC rssmxv_FileSubDirectory(ViSession instrSession,
                                          ViString  directory,
                                          ViInt32   bufferSize,
                                          ViChar    output[]);

ViStatus _VI_FUNC rssmxv_FileSubDirectoryCount(ViSession instrSession,
                                               ViString  directory,
                                               ViInt32*  numberOfFilesAndFolders);

ViStatus _VI_FUNC rssmxv_MoveFile(ViSession instrSession,
                                  ViString  source,
                                  ViString  destination);

ViStatus _VI_FUNC rssmxv_CopyFile(ViSession instrSession,
                                  ViString  source,
                                  ViString  destination);

ViStatus _VI_FUNC rssmxv_FileDelete(ViSession instrSession,
                                    ViString  filename);

ViStatus _VI_FUNC rssmxv_ChangeDirectory(ViSession instrSession,
                                         ViString  directory);

ViStatus _VI_FUNC rssmxv_MakeDirectory(ViSession instrSession,
                                       ViString  directory);

ViStatus _VI_FUNC rssmxv_DeleteDirectory(ViSession instrSession,
                                         ViString  directory);

ViStatus _VI_FUNC rssmxv_GetUserPath(ViSession instrSession,
                                     ViInt32   arraySize,
                                     ViChar    userPath[]);

ViStatus _VI_FUNC rssmxv_ChangeDrive(ViSession instrSession,
                                     ViString  drive);

ViStatus _VI_FUNC rssmxv_LoadState(ViSession instrSession,
                                   ViInt32   number,
                                   ViString  filename);

ViStatus _VI_FUNC rssmxv_StoreState(ViSession instrSession,
                                    ViInt32   number,
                                    ViString  filename);

ViStatus _VI_FUNC rssmxv_ReadToFileFromInstrument(ViSession instrSession,
                                                  ViString  source,
                                                  ViString  destination);

ViStatus _VI_FUNC rssmxv_WriteFromFileToInstrument(ViSession instrSession,
                                                   ViString  source,
                                                   ViString  destination);

ViStatus _VI_FUNC rssmxv_RecallInstrumentSettings(ViSession instrSession,
                                                  ViString  filename);

ViStatus _VI_FUNC rssmxv_SaveInstrumentSettings(ViSession instrSession,
                                                ViString  filename);

ViStatus _VI_FUNC rssmxv_ConfigureArb(ViSession instrSession,
                                      ViInt32   path,
                                      ViString  waveform,
                                      ViReal64  clockFrequency,
                                      ViBoolean enabled);

ViStatus _VI_FUNC rssmxv_ClearAllArbWaveforms(ViSession instrSession,
                                              ViInt32   path);

ViStatus _VI_FUNC rssmxv_LoadARBWaveformFile(ViSession instrSession,
                                             ViInt32   path,
                                             ViString  fileName);

ViStatus _VI_FUNC rssmxv_WriteARBClockWaveformFile(ViSession instrSession,
                                                   ViInt32   path,
                                                   ViString  fileName,
                                                   ViReal64  clock);

ViStatus _VI_FUNC rssmxv_ARBWaveformDataWrite(ViSession instrSession,
                                              ViInt32   path,
                                              ViString  fileName,
                                              ViInt32   arraySize,
                                              ViUInt8   waveformData[]);

ViStatus _VI_FUNC rssmxv_ARBWaveformDataRead(ViSession instrSession,
                                             ViInt32   path,
                                             ViString  fileName,
                                             ViString  waveformTag,
                                             ViInt32   bufferSize,
                                             ViChar    waveformData[],
                                             ViInt32*  returnedValues);

ViStatus _VI_FUNC rssmxv_LoadARBMultiCarrierSettingFile(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViString  fileName);

ViStatus _VI_FUNC rssmxv_StoreARBMultiCarrierSettingFile(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViString  fileName);

ViStatus _VI_FUNC rssmxv_StoreARBMultisegmentFile(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViString  fileName);

ViStatus _VI_FUNC rssmxv_ARBMultisegmentAddBlankSegment(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViReal64  sampleCount,
                                                        ViReal64  frequency);

ViStatus _VI_FUNC rssmxv_ARBMultisegmentCreateFromConfigurationFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName);

ViStatus _VI_FUNC rssmxv_ARBMultisegmentLoadFromConfigurationFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName);

ViStatus _VI_FUNC rssmxv_ARBMultisegmentDeleteFromConfigurationFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName);

ViStatus _VI_FUNC rssmxv_ARBMultisegmentAppendSegmentToPlayList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViBoolean state,
                            ViInt32   segment,
                            ViInt32   repetition,
                            ViInt32   action);

ViStatus _VI_FUNC rssmxv_AppendARBMultisegmentFile(ViSession instrSession,
                                                   ViInt32   path,
                                                   ViString  fileName);

ViStatus _VI_FUNC rssmxv_GenerateARBSineTestsignal(ViSession instrSession,
                                                   ViInt32   path,
                                                   ViString  fileName);

ViStatus _VI_FUNC rssmxv_ConfigureARBRectangleTestsignalName(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViString  fileName);

ViStatus _VI_FUNC rssmxv_ConfigureARBCIQTestsignalName(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViString  fileName);

ViStatus _VI_FUNC rssmxv_ConfigureAMAudioEnabled(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViBoolean enabled);

ViStatus _VI_FUNC rssmxv_ConfigureAMAudioSetToDefault(ViSession instrSession,
                                                      ViInt32   path);

ViStatus _VI_FUNC rssmxv_RecallAMAudioSettings(ViSession instrSession,
                                               ViInt32   path,
                                               ViString  filename);

ViStatus _VI_FUNC rssmxv_SaveAMAudioSettings(ViSession instrSession,
                                             ViInt32   path,
                                             ViString  fileName);

ViStatus _VI_FUNC rssmxv_DeleteAMAudioSettings(ViSession instrSession,
                                               ViInt32   path,
                                               ViString  filename);

ViStatus _VI_FUNC rssmxv_ConfigureFMAudioEnabled(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViBoolean enabled);

ViStatus _VI_FUNC rssmxv_ConfigureFMAudioSetToDefault(ViSession instrSession,
                                                      ViInt32   path);

ViStatus _VI_FUNC rssmxv_RecallFMAudioSettings(ViSession instrSession,
                                               ViInt32   path,
                                               ViString  filename);

ViStatus _VI_FUNC rssmxv_SaveFMAudioSettings(ViSession instrSession,
                                             ViInt32   path,
                                             ViString  filename);

ViStatus _VI_FUNC rssmxv_DeleteFMAudioSettings(ViSession instrSession,
                                               ViInt32   path,
                                               ViString  filename);

ViStatus _VI_FUNC rssmxv_ConfigureTDMBDABEnabled(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViBoolean enabled);

ViStatus _VI_FUNC rssmxv_ConfigureTDMBDABSetToDefault(ViSession instrSession,
                                                      ViInt32   path);

ViStatus _VI_FUNC rssmxv_RecallTDMBDABSettings(ViSession instrSession,
                                               ViInt32   path,
                                               ViString  filename);

ViStatus _VI_FUNC rssmxv_SaveTDMBDABSettings(ViSession instrSession,
                                             ViInt32   path,
                                             ViString  filename);

ViStatus _VI_FUNC rssmxv_DeleteTDMBDABSettings(ViSession instrSession,
                                               ViInt32   path,
                                               ViString  filename);

ViStatus _VI_FUNC rssmxv_ConfigureDRMEnabled(ViSession instrSession,
                                             ViInt32   path,
                                             ViBoolean enabled);

ViStatus _VI_FUNC rssmxv_ConfigureDRMSetToDefault(ViSession instrSession,
                                                  ViInt32   path);

ViStatus _VI_FUNC rssmxv_RecallDRMSettings(ViSession instrSession,
                                           ViInt32   path,
                                           ViString  filename);

ViStatus _VI_FUNC rssmxv_SaveDRMSettings(ViSession instrSession,
                                         ViInt32   path,
                                         ViString  filename);

ViStatus _VI_FUNC rssmxv_DeleteDRMSettings(ViSession instrSession,
                                           ViInt32   path,
                                           ViString  filename);

ViStatus _VI_FUNC rssmxv_LoadDRMInputFile(ViSession instrSession,
                                          ViInt32   path,
                                          ViString  filename);

ViStatus _VI_FUNC rssmxv_ConfigureATSCMHEnabled(ViSession instrSession,
                                                ViInt32   path,
                                                ViBoolean enabled);

ViStatus _VI_FUNC rssmxv_ConfigureATSCMHSetToDefault(ViSession instrSession,
                                                     ViInt32   path);

ViStatus _VI_FUNC rssmxv_RecallATSCMHSettings(ViSession instrSession,
                                              ViInt32   path,
                                              ViString  filename);

ViStatus _VI_FUNC rssmxv_SaveATSCMHSettings(ViSession instrSession,
                                            ViInt32   path,
                                            ViString  filename);

ViStatus _VI_FUNC rssmxv_DeleteATSCMHSettings(ViSession instrSession,
                                              ViInt32   path,
                                              ViString  filename);

ViStatus _VI_FUNC rssmxv_ConfigureATSC3Enabled(ViSession instrSession,
                                               ViInt32   path,
                                               ViBoolean enabled);

ViStatus _VI_FUNC rssmxv_ConfigureATSC3SetToDefault(ViSession instrSession,
                                                    ViInt32   path);

ViStatus _VI_FUNC rssmxv_RecallATSC3Settings(ViSession instrSession,
                                             ViInt32   path,
                                             ViString  filename);

ViStatus _VI_FUNC rssmxv_SaveATSC3Settings(ViSession instrSession,
                                           ViInt32   path,
                                           ViString  filename);

ViStatus _VI_FUNC rssmxv_DeleteATSC3Settings(ViSession instrSession,
                                             ViInt32   path,
                                             ViString  filename);

ViStatus _VI_FUNC rssmxv_ConfigureDVBTEnabled(ViSession instrSession,
                                              ViInt32   path,
                                              ViBoolean enabled);

ViStatus _VI_FUNC rssmxv_ConfigureDVBTSetToDefault(ViSession instrSession,
                                                   ViInt32   path);

ViStatus _VI_FUNC rssmxv_RecallDVBTSettings(ViSession instrSession,
                                            ViInt32   path,
                                            ViString  filename);

ViStatus _VI_FUNC rssmxv_SaveDVBTSettings(ViSession instrSession,
                                          ViInt32   path,
                                          ViString  filename);

ViStatus _VI_FUNC rssmxv_DeleteDVBTSettings(ViSession instrSession,
                                            ViInt32   path,
                                            ViString  filename);

ViStatus _VI_FUNC rssmxv_ConfigureDVBT2Enabled(ViSession instrSession,
                                               ViInt32   path,
                                               ViBoolean enabled);

ViStatus _VI_FUNC rssmxv_ConfigureDVBT2SetToDefault(ViSession instrSession,
                                                    ViInt32   path);

ViStatus _VI_FUNC rssmxv_RecallDVBT2Settings(ViSession instrSession,
                                             ViInt32   path,
                                             ViString  filename);

ViStatus _VI_FUNC rssmxv_SaveDVBT2Settings(ViSession instrSession,
                                           ViInt32   path,
                                           ViString  filename);

ViStatus _VI_FUNC rssmxv_DeleteDVBT2Settings(ViSession instrSession,
                                             ViInt32   path,
                                             ViString  filename);

ViStatus _VI_FUNC rssmxv_ConfigureDVBCEnabled(ViSession instrSession,
                                              ViInt32   path,
                                              ViBoolean enabled);

ViStatus _VI_FUNC rssmxv_ConfigureDVBCSetToDefault(ViSession instrSession,
                                                   ViInt32   path);

ViStatus _VI_FUNC rssmxv_RecallDVBCSettings(ViSession instrSession,
                                            ViInt32   path,
                                            ViString  filename);

ViStatus _VI_FUNC rssmxv_SaveDVBCSettings(ViSession instrSession,
                                          ViInt32   path,
                                          ViString  filename);

ViStatus _VI_FUNC rssmxv_DeleteDVBCSettings(ViSession instrSession,
                                            ViInt32   path,
                                            ViString  filename);

ViStatus _VI_FUNC rssmxv_QueryDVBCSettingsCatalog(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViInt32   bufferSize,
                                                  ViChar    catalog[],
                                                  ViInt32*  bytesRead);

ViStatus _VI_FUNC rssmxv_ConfigureISDBTEnabled(ViSession instrSession,
                                               ViInt32   path,
                                               ViBoolean enabled);

ViStatus _VI_FUNC rssmxv_ConfigureISDBTSetToDefault(ViSession instrSession,
                                                    ViInt32   path);

ViStatus _VI_FUNC rssmxv_RecallISDBTSettings(ViSession instrSession,
                                             ViInt32   path,
                                             ViString  filename);

ViStatus _VI_FUNC rssmxv_SaveISDBTSettings(ViSession instrSession,
                                           ViInt32   path,
                                           ViString  filename);

ViStatus _VI_FUNC rssmxv_DeleteISDBTSettings(ViSession instrSession,
                                             ViInt32   path,
                                             ViString  filename);

ViStatus _VI_FUNC rssmxv_ConfigureDTMBEnabled(ViSession instrSession,
                                              ViInt32   path,
                                              ViBoolean enabled);

ViStatus _VI_FUNC rssmxv_ConfigureDTMBSetToDefault(ViSession instrSession,
                                                   ViInt32   path);

ViStatus _VI_FUNC rssmxv_RecallDTMBSettings(ViSession instrSession,
                                            ViInt32   path,
                                            ViString  filename);

ViStatus _VI_FUNC rssmxv_SaveDTMBSettings(ViSession instrSession,
                                          ViInt32   path,
                                          ViString  filename);

ViStatus _VI_FUNC rssmxv_DeleteDTMBSettings(ViSession instrSession,
                                            ViInt32   path,
                                            ViString  filename);

ViStatus _VI_FUNC rssmxv_ConfigureDVBSEnabled(ViSession instrSession,
                                              ViInt32   path,
                                              ViBoolean enabled);

ViStatus _VI_FUNC rssmxv_ConfigureDVBSSetToDefault(ViSession instrSession,
                                                   ViInt32   path);

ViStatus _VI_FUNC rssmxv_RecallDVBSSettings(ViSession instrSession,
                                            ViInt32   path,
                                            ViString  filename);

ViStatus _VI_FUNC rssmxv_SaveDVBSSettings(ViSession instrSession,
                                          ViInt32   path,
                                          ViString  filename);

ViStatus _VI_FUNC rssmxv_DeleteDVBSSettings(ViSession instrSession,
                                            ViInt32   path,
                                            ViString  filename);

ViStatus _VI_FUNC rssmxv_ConfigureJ83BEnabled(ViSession instrSession,
                                              ViInt32   path,
                                              ViBoolean enabled);

ViStatus _VI_FUNC rssmxv_ConfigureJ83BSetToDefault(ViSession instrSession,
                                                   ViInt32   path);

ViStatus _VI_FUNC rssmxv_RecallJ83BSettings(ViSession instrSession,
                                            ViInt32   path,
                                            ViString  filename);

ViStatus _VI_FUNC rssmxv_SaveJ83BSettings(ViSession instrSession,
                                          ViInt32   path,
                                          ViString  filename);

ViStatus _VI_FUNC rssmxv_DeleteJ83BSettings(ViSession instrSession,
                                            ViInt32   path,
                                            ViString  filename);

ViStatus _VI_FUNC rssmxv_QueryJ83BSettingsCatalog(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViInt32   bufferSize,
                                                  ViChar    settingsCatalog[],
                                                  ViInt32*  bytesRead);

ViStatus _VI_FUNC rssmxv_ConfigureDVBS2Enabled(ViSession instrSession,
                                               ViInt32   path,
                                               ViBoolean enabled);

ViStatus _VI_FUNC rssmxv_ConfigureDVBS2SetToDefault(ViSession instrSession,
                                                    ViInt32   path);

ViStatus _VI_FUNC rssmxv_RecallDVBS2Settings(ViSession instrSession,
                                             ViInt32   path,
                                             ViString  filename);

ViStatus _VI_FUNC rssmxv_SaveDVBS2Settings(ViSession instrSession,
                                           ViInt32   path,
                                           ViString  filename);

ViStatus _VI_FUNC rssmxv_DeleteDVBS2Settings(ViSession instrSession,
                                             ViInt32   path,
                                             ViString  filename);

ViStatus _VI_FUNC rssmxv_ConfigureIQEnabled(ViSession instrSession,
                                            ViInt32   path,
                                            ViBoolean IQEnabled);

ViStatus _VI_FUNC rssmxv_ConfigureIQ(ViSession instrSession,
                                     ViInt32   path,
                                     ViBoolean swapEnabled);

ViStatus _VI_FUNC rssmxv_ConfigureIQOutputDigitalEnabled(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViBoolean IQDigitalEnabled);

ViStatus _VI_FUNC rssmxv_ConfigureDMEnabled(ViSession instrSession,
                                            ViInt32   path,
                                            ViBoolean DMEnabled);

ViStatus _VI_FUNC rssmxv_SelectDMStandard(ViSession instrSession,
                                          ViInt32   path,
                                          ViInt32   standard);

ViStatus _VI_FUNC rssmxv_ConfigureDMDataSource(ViSession instrSession,
                                               ViInt32   path,
                                               ViInt32   source);

ViStatus _VI_FUNC rssmxv_ConfigureDMPRBSType(ViSession instrSession,
                                             ViInt32   path,
                                             ViInt32   type);

ViStatus _VI_FUNC rssmxv_DeleteDMSettings(ViSession instrSession,
                                          ViInt32   path,
                                          ViString  fileName);

ViStatus _VI_FUNC rssmxv_RecallDMSettings(ViSession instrSession,
                                          ViInt32   path,
                                          ViString  fileName);

ViStatus _VI_FUNC rssmxv_SaveDMSettings(ViSession instrSession,
                                        ViInt32   path,
                                        ViString  fileName);

ViStatus _VI_FUNC rssmxv_DeleteDMFilterList(ViSession instrSession,
                                            ViInt32   path,
                                            ViString  fileName);

ViStatus _VI_FUNC rssmxv_ConfigureDMPowerRampEnabled(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViBoolean enabled);

ViStatus _VI_FUNC rssmxv_ConfigureDMPowerRampAttenuation(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViReal64  attenuation);

ViStatus _VI_FUNC rssmxv_ConfigureDMPowerRamp(ViSession instrSession,
                                              ViInt32   path,
                                              ViInt32   source,
                                              ViInt32   function,
                                              ViReal64  time,
                                              ViReal64  riseDelay,
                                              ViReal64  fallDelay,
                                              ViReal64  attenuation);

ViStatus _VI_FUNC rssmxv_ConfigureDMClockSource(ViSession instrSession,
                                                ViInt32   path,
                                                ViInt32   source);

ViStatus _VI_FUNC rssmxv_CopyDMControlList(ViSession instrSession,
                                           ViInt32   path,
                                           ViString  fileName);

ViStatus _VI_FUNC rssmxv_DeleteDMControlList(ViSession instrSession,
                                             ViInt32   path,
                                             ViString  fileName);

ViStatus _VI_FUNC rssmxv_ConfigureDMControlListDataWrite(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViInt32   numberOfValues,
                                                         ViInt16   data[]);

ViStatus _VI_FUNC rssmxv_ConfigureDMControlListTags(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViString  fileName,
                                                    ViString  tag,
                                                    ViInt32   arraySize,
                                                    ViChar    tagData[]);

ViStatus _VI_FUNC rssmxv_CopyDMDataList(ViSession instrSession,
                                        ViInt32   path,
                                        ViString  fileName);

ViStatus _VI_FUNC rssmxv_DeleteDMDataList(ViSession instrSession,
                                          ViInt32   path,
                                          ViString  fileName);

ViStatus _VI_FUNC rssmxv_ReadDMDataListData(ViSession instrSession,
                                            ViInt32   path,
                                            ViInt32   start,
                                            ViInt32   length,
                                            ViInt32   arraySize,
                                            ViChar    readData[]);

ViStatus _VI_FUNC rssmxv_ReadDMDataListDataAll(ViSession instrSession,
                                               ViInt32   path,
                                               ViInt32   arraySize,
                                               ViChar    readData[]);

ViStatus _VI_FUNC rssmxv_AppendDMDataListData(ViSession instrSession,
                                              ViInt32   path,
                                              ViString  dataAppend);

ViStatus _VI_FUNC rssmxv_DeleteDMMappingList(ViSession instrSession,
                                             ViInt32   path,
                                             ViString  fileName);

ViStatus _VI_FUNC rssmxv_DeleteDMUserStandards(ViSession instrSession,
                                               ViInt32   path,
                                               ViString  fileName);

ViStatus _VI_FUNC rssmxv_LoadDMUserStandards(ViSession instrSession,
                                             ViInt32   path,
                                             ViString  fileName);

ViStatus _VI_FUNC rssmxv_SaveDMUserStandards(ViSession instrSession,
                                             ViInt32   path,
                                             ViString  fileName);

ViStatus _VI_FUNC rssmxv_ConfigureBBINChannelMapping(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   channel,
                                                     ViBoolean activateChannel,
                                                     ViInt32   BBChannel);

ViStatus _VI_FUNC rssmxv_QueryBBINChannelPower(ViSession instrSession,
                                               ViInt32   path,
                                               ViInt32   channel,
                                               ViReal64* crestFactor,
                                               ViReal64* peakLevel);

ViStatus _VI_FUNC rssmxv_ConfigureReferenceOscillator(ViSession instrSession,
                                                      ViInt32   source,
                                                      ViInt32   frequency);

ViStatus _VI_FUNC rssmxv_ConfigureRFFrequency(ViSession instrSession,
                                              ViInt32   path,
                                              ViReal64  frequency);

ViStatus _VI_FUNC rssmxv_ConfigureLFSweep(ViSession instrSession,
                                          ViInt32   path,
                                          ViInt32   mode);

ViStatus _VI_FUNC rssmxv_ConfigureLFFrequencySweepStartStop(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViReal64  start,
                                                            ViReal64  stop);

ViStatus _VI_FUNC rssmxv_ConfigureFrequencyStep(ViSession instrSession,
                                                ViInt32   path,
                                                ViInt32   scaling,
                                                ViReal64  stepSize);

ViStatus _VI_FUNC rssmxv_ConfigureFrequencyStepDwell(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViReal64  dwell);

ViStatus _VI_FUNC rssmxv_ConfigureOutputEnabled(ViSession instrSession,
                                                ViInt32   path,
                                                ViBoolean outputEnabled);

ViStatus _VI_FUNC rssmxv_ConfigureRFAmplitude(ViSession instrSession,
                                              ViInt32   path,
                                              ViReal64  powerLevel);

ViStatus _VI_FUNC rssmxv_ConfigureRFLevel(ViSession instrSession,
                                          ViInt32   path,
                                          ViInt32   mode,
                                          ViInt32   settingCharacteristics,
                                          ViReal64  limit,
                                          ViReal64  offset);

ViStatus _VI_FUNC rssmxv_ConfigureRFLevelAttenuator(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViInt32   mode,
                                                    ViInt32   RFOFFMode);

ViStatus _VI_FUNC rssmxv_QueryRFLevelReference(ViSession instrSession,
                                               ViInt32   path,
                                               ViReal64* reference);

ViStatus _VI_FUNC rssmxv_ConfigurePowerOnEMF(ViSession instrSession,
                                             ViInt32   path,
                                             ViInt32   powerOnState);

ViStatus _VI_FUNC rssmxv_ConfigureFrequencyAndPowerMode(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   frequencyMode,
                                                        ViInt32   powerMode);

ViStatus _VI_FUNC rssmxv_ConfigureUserCorrectionFrequencyValues(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   arraySize,
                            ViReal64  frequencies[]);

ViStatus _VI_FUNC rssmxv_QueryUserCorrectionFrequencyValues(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViInt32   arraySize,
                                                            ViReal64  frequencies[],
                                                            ViInt32*  valuesRead);

ViStatus _VI_FUNC rssmxv_ConfigureUserCorrectionLevelValues(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViInt32   arraySize,
                                                            ViReal64  levels[]);

ViStatus _VI_FUNC rssmxv_QueryUserCorrectionLevelValues(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   arraySize,
                                                        ViReal64  levels[],
                                                        ViInt32*  valuesRead);

ViStatus _VI_FUNC rssmxv_ConfigureListDwellTimeValues(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViInt32   arraySize,
                                                      ViReal64  dwellTimes[]);

ViStatus _VI_FUNC rssmxv_QueryListDwellTimeValues(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViInt32   arraySize,
                                                  ViReal64  dwellTimes[],
                                                  ViInt32*  valuesRead);

ViStatus _VI_FUNC rssmxv_ConfigureListFrequencyValues(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViInt32   arraySize,
                                                      ViReal64  frequencies[]);

ViStatus _VI_FUNC rssmxv_QueryListFrequencyValues(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViInt32   arraySize,
                                                  ViReal64  frequencies[],
                                                  ViInt32*  valuesRead);

ViStatus _VI_FUNC rssmxv_ConfigureListPowerValues(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViInt32   arraySize,
                                                  ViReal64  powers[]);

ViStatus _VI_FUNC rssmxv_QueryListPowerValues(ViSession instrSession,
                                              ViInt32   path,
                                              ViInt32   arraySize,
                                              ViReal64  powerValues[],
                                              ViInt32*  valuesRead);

ViStatus _VI_FUNC rssmxv_ConfigurePowerSensorMappingDirectly(ViSession instrSession,
                                                             ViString  sensorID,
                                                             ViInt32   mapping);

ViStatus _VI_FUNC rssmxv_ConfigurePowerSensorScanUSB(ViSession instrSession,
                                                     ViInt32   serial,
                                                     ViInt32   deviceID);

ViStatus _VI_FUNC rssmxv_SelectPowerSensor(ViSession instrSession,
                                           ViInt32   sense,
                                           ViInt32   powerSensor);

ViStatus _VI_FUNC rssmxv_ConfigurePowerSensorControlEnabled(ViSession instrSession,
                                                            ViInt32   sense,
                                                            ViBoolean powerSensorControl);

ViStatus _VI_FUNC rssmxv_ConfigurePowerSensorControlMode(ViSession instrSession,
                                                         ViInt32   sense,
                                                         ViInt32   mode);

ViStatus _VI_FUNC rssmxv_PowerSensorControlExecuteSingle(ViSession instrSession,
                                                         ViInt32   sense);

ViStatus _VI_FUNC rssmxv_ConfigurePowerSensorControlMeasurementSettings(
                            ViSession instrSession,
                            ViInt32   sense,
                            ViInt32   delay,
                            ViReal64  range);

ViStatus _VI_FUNC rssmxv_ConfigurePowerSensorControlPeakEnabled(
                            ViSession instrSession,
                            ViInt32   sense,
                            ViBoolean peak);

ViStatus _VI_FUNC rssmxv_ConfigurePowerSensorControlTargetLevel(
                            ViSession instrSession,
                            ViInt32   sense,
                            ViReal64  targetLevel);

ViStatus _VI_FUNC rssmxv_PowerSensorControlMeasureAsTargetLevel(
                            ViSession instrSession,
                            ViInt32   sense);

ViStatus _VI_FUNC rssmxv_DiagnosticModulesInfo(ViSession instrSession,
                                               ViInt32   path,
                                               ViString  moduleName,
                                               ViInt32   arraySize,
                                               ViChar    info[]);

ViStatus _VI_FUNC rssmxv_DiagnosticTestPoint(ViSession instrSession,
                                             ViInt32   path,
                                             ViString  pointName,
                                             ViInt32   arraySize,
                                             ViChar    info[]);

ViStatus _VI_FUNC rssmxv_ConfigureDisplayBrightness(ViSession instrSession,
                                                    ViReal64  brightness);

ViStatus _VI_FUNC rssmxv_ConfigureDisplayButtonBrightness(ViSession instrSession,
                                                          ViReal64  buttonBrightness);

ViStatus _VI_FUNC rssmxv_ConfigureHUMSEnabled(ViSession instrSession,
                                              ViBoolean HUMS);

ViStatus _VI_FUNC rssmxv_ConfigureHUMSDataFormat(ViSession instrSession,
                                                 ViInt32   dataFormat);

ViStatus _VI_FUNC rssmxv_DeleteHUMSAllData(ViSession instrSession);

ViStatus _VI_FUNC rssmxv_ExportHUMSHistory(ViSession instrSession,
                                           ViInt32   bufferSize,
                                           ViChar    history[],
                                           ViInt32*  bytesRead);

ViStatus _VI_FUNC rssmxv_ExportHUMSHistoryToFile(ViSession instrSession,
                                                 ViString  filename);

ViStatus _VI_FUNC rssmxv_DeleteHUMSHistory(ViSession instrSession);

ViStatus _VI_FUNC rssmxv_QueryHUMSDeviceTagsCatalog(ViSession instrSession,
                                                    ViInt32   bufferSize,
                                                    ViChar    deviceTagsCatalog[],
                                                    ViInt32*  bytesRead);

ViStatus _VI_FUNC rssmxv_HUMSDeviceTagsDeleteAll(ViSession instrSession);

ViStatus _VI_FUNC rssmxv_DefineHUMSDeviceTag(ViSession instrSession,
                                             ViInt32   ID,
                                             ViString  key,
                                             ViString  value);

ViStatus _VI_FUNC rssmxv_QueryHUMSDeviceTag(ViSession instrSession,
                                            ViInt32   tagID,
                                            ViInt32   bufferSize,
                                            ViChar    keyValuePair[],
                                            ViInt32*  bytesRead);

ViStatus _VI_FUNC rssmxv_DeleteHUMSDeviceTag(ViSession instrSession,
                                             ViInt32   tagID);

ViStatus _VI_FUNC rssmxv_ConfigureSNMPRestEnabled(ViSession instrSession,
                                                  ViBoolean rest);

ViStatus _VI_FUNC rssmxv_ConfigureSNMPVersion(ViSession instrSession,
                                              ViInt32   version);

ViStatus _VI_FUNC rssmxv_QuerySNMPUserCatalog(ViSession instrSession,
                                              ViInt32   bufferSize,
                                              ViChar    userCatalog[],
                                              ViInt32*  bytesRead);

ViStatus _VI_FUNC rssmxv_AddSNMPUser(ViSession instrSession,
                                     ViString  name,
                                     ViInt32   access,
                                     ViInt32   level,
                                     ViString  authenticationPassword,
                                     ViString  privacyPassword);

ViStatus _VI_FUNC rssmxv_DeleteSNMPUser(ViSession instrSession,
                                        ViString  username);

ViStatus _VI_FUNC rssmxv_DeleteAllSNMPUsers(ViSession instrSession);

ViStatus _VI_FUNC rssmxv_ConfigureSNMPAccessStrings(ViSession instrSession,
                                                    ViString  readOnlyAccess,
                                                    ViString  readWriteAccess);

ViStatus _VI_FUNC rssmxv_ConfigureSNMPAdminInfo(ViSession instrSession,
                                                ViString  contact,
                                                ViString  location);

ViStatus _VI_FUNC rssmxv_SystemConfigurationExternalAdd(ViSession instrSession,
                                                        ViString  instrumentName,
                                                        ViString  hardwareChannel,
                                                        ViString  address,
                                                        ViString  RFPath);

ViStatus _VI_FUNC rssmxv_SystemConfigurationExternalRename(ViSession instrSession,
                                                           ViString  currentName,
                                                           ViString  newName);

ViStatus _VI_FUNC rssmxv_SystemConfigurationExternalHSDSelectInstrument(
                            ViSession instrSession,
                            ViString  instrumentName,
                            ViString  RFPath);

ViStatus _VI_FUNC rssmxv_SystemConfigurationExternalHSDConnectInstrument(
                            ViSession instrSession,
                            ViString  instrumentName,
                            ViString  path);

ViStatus _VI_FUNC rssmxv_QuerySystemConfigurationExternalHSDConnectionInfo(
                            ViSession  instrSession,
                            ViBoolean* state,
                            ViInt32*   direction);

ViStatus _VI_FUNC rssmxv_ConfigureSystemConfigurationHSDRFFrequency(
                            ViSession instrSession,
                            ViReal64  RFFrequency);

ViStatus _VI_FUNC rssmxv_QueryConfigurationHSDConnectorName(ViSession instrSession,
                                                            ViInt32   HSD,
                                                            ViInt32   bufferSize,
                                                            ViChar    name[],
                                                            ViInt32*  bytesRead);

ViStatus _VI_FUNC rssmxv_readToFile(ViSession instrSession,
                                    ViString  source,
                                    ViString  destination);

ViStatus _VI_FUNC rssmxv_writeFromFile(ViSession instrSession,
                                       ViString  source,
                                       ViString  destination);

ViStatus _VI_FUNC rssmxv_SendSoftwareTrigger(ViSession instrSession);

ViStatus _VI_FUNC rssmxv_reset(ViSession instrSession);

ViStatus _VI_FUNC rssmxv_self_test(ViSession instrSession,
                                   ViInt16*  selfTestResult,
                                   ViChar    selfTestMessage[]);

ViStatus _VI_FUNC rssmxv_revision_query(ViSession instrSession,
                                        ViChar    instrumentDriverRevision[],
                                        ViChar    firmwareRevision[]);

ViStatus _VI_FUNC rssmxv_error_query(ViSession instrSession,
                                     ViInt32*  errorCode,
                                     ViChar    errorMessage[]);

ViStatus _VI_FUNC rssmxv_SystemStaticError(ViSession instrSession,
                                           ViInt32   bufferSize,
                                           ViChar    buffer[]);

ViStatus _VI_FUNC rssmxv_error_message(ViSession instrSession,
                                       ViStatus  errorCode,
                                       ViChar    errorMessage[]);

ViStatus _VI_FUNC rssmxv_ConfigureErrorChecking(ViSession instrSession,
                                                ViBoolean optionChecking,
                                                ViBoolean rangeChecking,
                                                ViBoolean statusChecking);

ViStatus _VI_FUNC rssmxv_ClearStatus(ViSession instrSession);

ViStatus _VI_FUNC rssmxv_IDQueryResponse(ViSession instrSession,
                                         ViChar    IDQueryResponse[]);

ViStatus _VI_FUNC rssmxv_ProcessAllPreviousCommands(ViSession instrSession);

ViStatus _VI_FUNC rssmxv_QueryOPC(ViSession instrSession,
                                  ViInt32*  OPC);

ViStatus _VI_FUNC rssmxv_SetOPCTimeout(ViSession instrSession,
                                       ViUInt32  OPCTimeout);

ViStatus _VI_FUNC rssmxv_GetOPCTimeout(ViSession instrSession,
                                       ViUInt32* timeout);

ViStatus _VI_FUNC rssmxv_SetVISATimeout(ViSession instrSession,
                                        ViUInt32  VISATimeout);

ViStatus _VI_FUNC rssmxv_GetVISATimeout(ViSession instrSession,
                                        ViUInt32* VISATimeout);

ViStatus _VI_FUNC rssmxv_Calibrate(ViSession instrSession,
                                   ViInt32   reserved,
                                   ViInt32   timeoutInMs,
                                   ViInt32*  calibrationResult);

ViStatus _VI_FUNC rssmxv_LastFactoryCalibration(ViSession instrSession,
                                                ViInt32   bufferSize,
                                                ViChar    calibrationDate[]);

ViStatus _VI_FUNC rssmxv_SecurityVolatileMode(ViSession instrSession,
                                              ViString  password,
                                              ViBoolean memoryProtectionState);

ViStatus _VI_FUNC rssmxv_LockUserInterface(ViSession instrSession,
                                           ViInt32   userInterface);

ViStatus _VI_FUNC rssmxv_ConfigureAutoSystemErrQuery(ViSession instrSession,
                                                     ViBoolean autoSystErrQuery);

ViStatus _VI_FUNC rssmxv_ConfigureMultiThreadLocking(ViSession instrSession,
                                                     ViBoolean multiThreadLocking);

ViStatus _VI_FUNC rssmxv_ConfigureAttributeDataTypeMismatchReporting(
                            ViSession instrSession,
                            ViBoolean state);

ViStatus _VI_FUNC rssmxv_SetStatusRegister(ViSession instrSession,
                                           ViInt32   registerOperation,
                                           ViInt32   questionableRegister,
                                           ViInt32   enable,
                                           ViInt32   PTransition,
                                           ViInt32   NTransition);

ViStatus _VI_FUNC rssmxv_GetStatusRegister(ViSession instrSession,
                                           ViInt32   registerOperation,
                                           ViInt32   questionableRegister,
                                           ViInt32   registerPart,
                                           ViInt32*  registerValue);

ViStatus _VI_FUNC rssmxv_GetError(ViSession instrSession,
                                  ViStatus* code,
                                  ViInt32   bufferSize,
                                  ViChar    description[]);

ViStatus _VI_FUNC rssmxv_ClearError(ViSession instrSession);

ViStatus _VI_FUNC rssmxv_WriteInstrData(ViSession instrSession,
                                        ViString  writeBuffer);

ViStatus _VI_FUNC rssmxv_QueryViBoolean(ViSession  instrSession,
                                        ViString   command,
                                        ViBoolean* value);

ViStatus _VI_FUNC rssmxv_QueryViInt32(ViSession instrSession,
                                      ViString  command,
                                      ViInt32*  value);

ViStatus _VI_FUNC rssmxv_QueryViReal64(ViSession instrSession,
                                       ViString  command,
                                       ViReal64* value);

ViStatus _VI_FUNC rssmxv_QueryViString(ViSession instrSession,
                                       ViString  command,
                                       ViInt32   bufferSize,
                                       ViChar    value[]);

ViStatus _VI_FUNC rssmxv_SetAttributeViInt32(ViSession instrSession,
                                             ViString  repeatedCapabilityName,
                                             ViAttr    attributeId,
                                             ViInt32   attributeValue);

ViStatus _VI_FUNC rssmxv_SetAttributeViInt64(ViSession instrSession,
                                             ViString  repeatedCapabilityName,
                                             ViAttr    attributeId,
                                             ViInt64   attributeValue);

ViStatus _VI_FUNC rssmxv_SetAttributeViReal64(ViSession instrSession,
                                              ViString  repeatedCapabilityName,
                                              ViAttr    attributeId,
                                              ViReal64  attributeValue);

ViStatus _VI_FUNC rssmxv_SetAttributeViString(ViSession instrSession,
                                              ViString  repeatedCapabilityName,
                                              ViAttr    attributeId,
                                              ViString  attributeValue);

ViStatus _VI_FUNC rssmxv_SetAttributeViBoolean(ViSession instrSession,
                                               ViString  repeatedCapabilityName,
                                               ViAttr    attributeId,
                                               ViBoolean attributeValue);

ViStatus _VI_FUNC rssmxv_GetAttributeViInt32(ViSession instrSession,
                                             ViString  repeatedCapabilityName,
                                             ViAttr    attributeId,
                                             ViInt32*  attributeValue);

ViStatus _VI_FUNC rssmxv_GetAttributeViInt64(ViSession instrSession,
                                             ViString  repeatedCapabilityName,
                                             ViAttr    attributeId,
                                             ViInt64*  attributeValue);

ViStatus _VI_FUNC rssmxv_GetAttributeViReal64(ViSession instrSession,
                                              ViString  repeatedCapabilityName,
                                              ViAttr    attributeId,
                                              ViReal64* attributeValue);

ViStatus _VI_FUNC rssmxv_GetAttributeViString(ViSession instrSession,
                                              ViString  repeatedCapabilityName,
                                              ViAttr    attributeId,
                                              ViInt32   bufferSize,
                                              ViChar    attributeValue[]);

ViStatus _VI_FUNC rssmxv_GetAttributeViBoolean(ViSession  instrSession,
                                               ViString   repeatedCapabilityName,
                                               ViAttr     attributeId,
                                               ViBoolean* attributeValue);

ViStatus _VI_FUNC rssmxv_GetAttributeViStringWithRealLen(ViSession instrSession,
                                                         ViString  repeatedCapability,
                                                         ViAttr    attributeID,
                                                         ViInt32   bufferSize,
                                                         ViChar    attributeValue[],
                                                         ViInt32*  responseLength);

ViStatus _VI_FUNC rssmxv_GetAttributeRawStringWithRealLen(ViSession instrSession,
                                                          ViString  repeatedCapability,
                                                          ViAttr    attributeID,
                                                          ViInt32   bufferSize,
                                                          ViChar    attributeValue[],
                                                          ViInt32*  responseLength);

ViStatus _VI_FUNC rssmxv_GetAttributeRepCapName(ViSession instrSession,
                                                ViAttr    attributeId,
                                                ViString  repCapNameID,
                                                ViInt32   index,
                                                ViInt32   bufferSize,
                                                ViChar    repCapName[]);

ViStatus _VI_FUNC rssmxv_GetAttributeRepeatedCapabilityIds(ViSession instrSession,
                                                           ViAttr    attributeId,
                                                           ViInt32   bufferSize,
                                                           ViChar    repeatedCapabilityIds[]);

ViStatus _VI_FUNC rssmxv_GetAttributeRepeatedCapabilityIdNames(
                            ViSession instrSession,
                            ViAttr    attributeId,
                            ViString  repeatedCapabilityId,
                            ViInt32   bufferSize,
                            ViChar    repeatedCapabilityIdNames[]);

ViStatus _VI_FUNC rssmxv_close(ViSession instrSession);

/****************************************************************************
 *------------------------ Error And Completion Codes ----------------------*
 ****************************************************************************/

#define RSSMXV_WARN_MEASURE_UNCALIBRATED    (RS_CLASS_WARN_BASE + 0x0001L)
#define RSSMXV_WARN_OVER_RANGE              (RS_CLASS_WARN_BASE + 0x0002L)

#define RSSMXV_ERROR_INCORRECT_STATE        (RS_CLASS_ERROR_BASE + 0x0003L)
#define RSSMXV_ERROR_INCORRECT_RESPONSE     (RS_CLASS_ERROR_BASE + 0x0002L)
#define RSSMXV_ERROR_MAX_TIME_EXCEEDED      (RS_CLASS_ERROR_BASE + 0x0001L)

#define RSSMXV_WARNMSG_MEASURE_UNCALIBRATED "Uncalibrated measurement"
#define RSSMXV_WARNMSG_OVER_RANGE           "Measurement overrange"

#define RSSMXV_ERRMSG_INCORRECT_STATE     "Instrument in incorrect state"
#define RSSMXV_ERRMSG_INCORRECT_RESPONSE      "Response incorrect; report problem to R&S support!"
#define RSSMXV_ERRMSG_MAX_TIME_EXCEEDED     "Max Time Exceeded"

#define RSSMXV_ERROR_CODES_AND_MSGS \
    {RSSMXV_WARN_MEASURE_UNCALIBRATED,  RSSMXV_WARNMSG_MEASURE_UNCALIBRATED},\
    {RSSMXV_WARN_OVER_RANGE,            RSSMXV_WARNMSG_OVER_RANGE},\
    {RSSMXV_ERROR_INCORRECT_STATE ,   RSSMXV_ERRMSG_INCORRECT_STATE },\
    {RSSMXV_ERROR_INCORRECT_RESPONSE,     RSSMXV_ERRMSG_INCORRECT_RESPONSE},\
    {RSSMXV_ERROR_MAX_TIME_EXCEEDED,    RSSMXV_ERRMSG_MAX_TIME_EXCEEDED}

/****************************************************************************
 *---------------------------- End Include File ----------------------------*
 ****************************************************************************/

#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif
#endif /* __RSSMXV_HEADER */
